package junit.textui;

import java.io.PrintStream;

import junit.framework.Test;
import junit.framework.TestResult;
import junit.framework.TestSuite;
import junit.runner.BaseTestRunner;
import junit.runner.StandardTestSuiteLoader;
import junit.runner.TestSuiteLoader;
import junit.runner.Version;

/**
 * A command line based tool to run tests.
 * 
 * <pre>
 * java junit.textui.TestRunner [-wait] TestCaseClass
 * </pre>
 * 
 * TestRunner expects the name of a TestCase class as argument. If this class defines a static <code>suite</code> method it will be invoked and the returned test is run. Otherwise all the methods starting with "test" having no arguments are run.
 * <p>
 * When the wait command line argument is given TestRunner waits until the users types RETURN.
 * <p>
 * TestRunner prints a trace as the tests are executed followed by a summary at the end.
 */
public class TestRunner extends BaseTestRunner {
	public static void main(final String arguments[]) {
		TestRunner aTestRunner = new TestRunner();
		try {
			TestResult r = aTestRunner.start(arguments);
			if (!r.wasSuccessful()) {
				System.exit(FAILURE_EXIT);
			}
			System.exit(SUCCESS_EXIT);
		} catch (Exception e) {
			System.err.println(e.getMessage());
			System.exit(EXCEPTION_EXIT);
		}
	}

	/**
	 * Runs a suite extracted from a TestCase subclass.
	 */
	static public void run(final Class testClass) {
		run(new TestSuite(testClass));
	}

	/**
	 * Runs a single test and collects its results. This method can be used to start a test run from your program.
	 * 
	 * <pre>
	 * public static void main(final String[] arguments) {
	 * 	test.textui.TestRunner.run(suite());
	 * }
	 * </pre>
	 */
	static public TestResult run(final Test test) {
		TestRunner runner = new TestRunner();
		return runner.doRun(test);
	}

	/**
	 * Runs a single test and waits until the user types RETURN.
	 */
	static public void runAndWait(final Test suite) {
		TestRunner aTestRunner = new TestRunner();
		aTestRunner.doRun(suite, true);
	}

	private ResultPrinter fPrinter;

	public static final int SUCCESS_EXIT = 0;

	public static final int FAILURE_EXIT = 1;

	public static final int EXCEPTION_EXIT = 2;

	/**
	 * Constructs a TestRunner.
	 */
	public TestRunner() {
		this(System.out);
	}

	/**
	 * Constructs a TestRunner using the given stream for all the output
	 */
	public TestRunner(final PrintStream writer) {
		this(new ResultPrinter(writer));
	}

	/**
	 * Constructs a TestRunner using the given ResultPrinter all the output
	 */
	public TestRunner(final ResultPrinter printer) {
		this.fPrinter = printer;
	}

	/**
	 * Creates the TestResult to be used for the test run.
	 */
	protected TestResult createTestResult() {
		return new TestResult();
	}

	public TestResult doRun(final Test test) {
		return doRun(test, false);
	}

	public TestResult doRun(final Test suite, final boolean wait) {
		TestResult result = createTestResult();
		result.addListener(this.fPrinter);
		long startTime = System.currentTimeMillis();
		suite.run(result);
		long endTime = System.currentTimeMillis();
		long runTime = endTime - startTime;
		this.fPrinter.print(result, runTime);
		pause(wait);
		return result;
	}

	/**
	 * Always use the StandardTestSuiteLoader. Overridden from BaseTestRunner.
	 */
	public TestSuiteLoader getLoader() {
		return new StandardTestSuiteLoader();
	}

	protected void pause(final boolean wait) {
		if (!wait) {
			return;
		}
		this.fPrinter.printWaitPrompt();
		try {
			System.in.read();
		} catch (Exception e) {
		}
	}

	protected void runFailed(final String message) {
		System.err.println(message);
		System.exit(FAILURE_EXIT);
	}

	protected TestResult runSingleMethod(final String testCase, final String method, final boolean wait) throws Exception {
		Class testClass = loadSuiteClass(testCase);
		Test test = TestSuite.createTest(testClass, method);
		return doRun(test, wait);
	}

	public void setPrinter(final ResultPrinter printer) {
		this.fPrinter = printer;
	}

	/**
	 * Starts a test run. Analyzes the command line arguments and runs the given test suite.
	 */
	public TestResult start(final String arguments[]) throws Exception {
		String testCase = "";
		String method = "";
		boolean wait = false;
		for (int i = 0; i < arguments.length; i++) {
			if (arguments[i].equals("-wait")) {
				wait = true;
			} else if (arguments[i].equals("-c")) {
				testCase = extractClassName(arguments[++i]);
			} else if (arguments[i].equals("-m")) {
				String arg = arguments[++i];
				int lastIndex = arg.lastIndexOf('.');
				testCase = arg.substring(0, lastIndex);
				method = arg.substring(lastIndex + 1);
			} else if (arguments[i].equals("-v")) {
				System.err.println("JUnit " + Version.id() + " by Kent Beck and Erich Gamma");
			} else {
				testCase = arguments[i];
			}
		}
		if (testCase.equals("")) {
			throw new Exception("Usage: TestRunner [-wait] testCaseName, where name is the name of the TestCase class");
		}
		try {
			if (!method.equals("")) {
				return runSingleMethod(testCase, method, wait);
			}
			Test suite = getTest(testCase);
			return doRun(suite, wait);
		} catch (Exception e) {
			throw new Exception("Could not create and run test suite: " + e);
		}
	}

	public void testEnded(final String testName) {
	}

	public void testFailed(final int status, final Test test, final Throwable t) {
	}

	public void testStarted(final String testName) {
	}
}