package org.eclipse.soda.dk.profile.test.agent;

/*************************************************************************
 * Copyright (c) 2007, 2009 IBM.                                         *
 * All rights reserved. This program and the accompanying materials      *
 * are made available under the terms of the Eclipse Public License v1.0 *
 * which accompanies this distribution, and is available at              *
 * http://www.eclipse.org/legal/epl-v10.html                             *
 *                                                                       *
 * Contributors:                                                         *
 *     IBM - initial API and implementation                              *
 ************************************************************************/
import java.util.Dictionary;
import java.util.Hashtable;
import java.util.Map;
import java.util.StringTokenizer;
import org.eclipse.soda.dk.core.EscObject;
import org.eclipse.soda.dk.device.service.DeviceService;
import org.eclipse.soda.dk.interest.InterestMask;
import org.eclipse.soda.dk.interest.service.InterestService;
import org.eclipse.soda.dk.nls.Nls;
import org.eclipse.soda.dk.notification.service.NotificationService;
import org.eclipse.soda.dk.profile.service.ProfileService;
import org.eclipse.soda.dk.profile.test.agent.service.ProfileTestAgentService;
import org.eclipse.soda.dk.testagent.framework.NotificationClientTestCase;
import org.eclipse.soda.dk.testagent.util.NotificationProbe;

/**
 * @author IBM
 * @version 1.2.0
 * @since 1.0
 */
public abstract class ProfileAbstractTestCase extends NotificationClientTestCase {
	/**
	 * Define the topic delimiter (char) constant.
	 */
	public static final char TOPIC_DELIMITER = '/';

	/**
	 * Define the empty string (String) constant.
	 */
	protected static final String EMPTY_STRING = ""; //$NON-NLS-1$

	/**
	 * Defines the Status external key.
	 */
	public static final String STATUS_EXTERNAL_KEY = "/Status"; //$NON-NLS-1$

	/**
	 * Define the max response waiting time (int) constant.
	 */
	public final static int DEFAULT_MAX_RESPONSE_WAITING_TIME = 2000;

	/**
	 * Define the default max status waiting time (int) constant.
	 */
	public final static int DEFAULT_MAX_STATUS_WAITING_TIME = 180000;

	/**
	 * Define the prefix property (String) field.
	 */
	protected String prefixProperty;

	/**
	 * Define the max response waiting time property (int) field.
	 */
	protected int maxResponseWaitingTimeProperty;

	/**
	 * Define the max status waiting time property (int) field.
	 */
	protected int maxStatusWaitingTimeProperty;

	/**
	 * Constructs a new instance of the ProfileAbstractTestCase class.
	 * @param name
	 *		The name (<code>String</code>) parameter.
	 */
	protected ProfileAbstractTestCase(final String name) {
		super(name);
	}

	/**
	 * Parse comma delimited with the specified comma delimited parameter and return the String[] result.
	 * @param target
	 *		The target (<code>String</code>) parameter.
	 * @param delimiter
	 *		The delimiter (<code>String</code>) parameter.
	 * @return Results of the parse comma delimited (<code>String[]</code>) value.
	 */
	public static String[] parseDelimitedByToken(final String target, final String delimiter) {
		StringTokenizer tokenizer = new StringTokenizer(target, delimiter);
		String[] values = new String[tokenizer.countTokens()];
		int index = 0;
		while (tokenizer.hasMoreTokens()) {
			values[index++] = tokenizer.nextToken().trim();
		}
		return values;
	}

	/**
	 * Parse hex space delimited with the specified space delimited hex parameter and return the byte[] result.
	 * @param spaceDelimitedHex
	 *		The space delimited hex (<code>String</code>) parameter.
	 * @return Results of the parse hex space delimited (<code>byte[]</code>) value.
	 */
	public static byte[] parseHexSpaceDelimited(final String spaceDelimitedHex) {
		StringTokenizer tokenizer = new StringTokenizer(spaceDelimitedHex);
		byte[] value = new byte[tokenizer.countTokens()];
		int index = 0;
		while (tokenizer.hasMoreTokens()) {
			String byteHex = tokenizer.nextToken();
			byte byteValue = 0;
			byteValue = (byte) Integer.parseInt(byteHex, 16);
			value[index++] = byteValue;
		}
		return value;
	}

	/**
	 * Parse interest mask with the specified comma delimited pattern and comma delimited care parameters and return the InterestMask result.
	 * @param maskConfig
	 *		The mask configuration (<code>String</code>) parameter.
	 * @return Result of the parse interest mask (<code>InterestMask</code>) value.
	 */
	public static InterestService parseInterestMask(final String maskConfig) {
		try {
			String[] configParam = parseDelimitedByToken(maskConfig, ","); //$NON-NLS-1$
			int offset = Integer.parseInt(configParam[1]);
			int length = Integer.parseInt(configParam[2]);
			byte[] pattern = parseHexSpaceDelimited(configParam[3]);
			return new InterestMask(offset, length, pattern);
		} catch (final NumberFormatException e) {
			throw new IllegalArgumentException(ProfileTestAgent.DefaultResourceBundle.getString(Integer.toString(ProfileTestAgent.PARSEINTERESTMASK)) + maskConfig);
		}
	}

	/**
	 * To bytes with the specified string parameter and return the byte[] result.
	 * @param string
	 *		The string (<code>String</code>) parameter.
	 * @return Results of the to bytes (<code>byte[]</code>) value.
	 * @throws RuntimeException Runtime Exception.
	 */
	public static byte[] toBytes(final String string) throws RuntimeException {
		final char[] characters = string.toCharArray();
		final int length = (characters.length + 1) >> 1;
		final byte[] tag = new byte[length];
		int index = (characters.length % 2);
		for (int i = 0; i < characters.length; i++) {
			char character = characters[i];
			int digit = character - '0';
			if (digit < 0) {
				throw new NumberFormatException();
			}
			if (digit > 9) {
				digit = character - ('a' - 10);
				if ((digit & 0xfffffff0) != 0) {
					digit = character - ('A' - 10);
					if ((digit & 0xfffffff0) != 0) {
						throw new NumberFormatException();
					}
				}
			}
			if ((index % 2) == 0) {
				tag[index >> 1] = (byte) (digit << 4);
			} else {
				tag[index >> 1] = (byte) ((tag[index >> 1] & 0xf0) + digit);
			}
			index++;
		}
		return tag;
	}

	/**
	 * Execute command with the specified topic key and data value parameters.
	 * @param topicKey
	 *		The topic key (<code>String</code>) parameter.
	 * @param dataValue
	 *		The data value (<code>Object</code>) parameter.
	 * @see #executeCommand(String,String,Object)
	 * @see #executeMultiplexCommand(String,String,Object)
	 * @see #executeMultiplexCommand(String,String,String,Object)
	 */
	protected void executeCommand(final String topicKey, final Object dataValue) {
		executeCommand(topicKey, DeviceService.VALUE_DATA_KEY, dataValue);
	}

	/**
	 * Execute command with the specified topic key, data key and data value parameters.
	 * @param topicKey
	 *		The topic key (<code>String</code>) parameter.
	 * @param dataKey
	 *		The data key (<code>String</code>) parameter.
	 * @param dataValue
	 *		The data value (<code>Object</code>) parameter.
	 * @see #executeCommand(String,Object)
	 * @see #executeMultiplexCommand(String,String,Object)
	 * @see #executeMultiplexCommand(String,String,String,Object)
	 */
	protected void executeCommand(final String topicKey, final String dataKey, final Object dataValue) {
		Dictionary data = new Hashtable();
		data.put(dataKey, dataValue);
		broadcast(topicKey + NotificationService.EXECUTE_EXTERNAL_KEY, data);
	}

	/**
	 * Execute multiplex command with the specified topic key, channel id and data value parameters.
	 * @param topicKey
	 *		The topic key (<code>String</code>) parameter.
	 * @param channelId
	 *		The channel ID (<code>String</code>) parameter.
	 * @param dataValue
	 *		The data value (<code>Object</code>) parameter.
	 * @see #executeMultiplexCommand(String,String,String,Object)
	 * @since 1.2
	 */
	protected void executeMultiplexCommand(final String topicKey, final String channelId, final Object dataValue) {
		executeMultiplexCommand(topicKey, channelId, DeviceService.VALUE_DATA_KEY, dataValue);
	}

	/**
	 * Execute multiplex command with the specified topic key, channel id, data key and data value parameters.
	 * @param topicKey
	 *		The topic key (<code>String</code>) parameter.
	 * @param channelId
	 *		The channel ID (<code>String</code>) parameter.
	 * @param dataKey
	 *		The data key (<code>String</code>) parameter.
	 * @param dataValue
	 *		The data value (<code>Object</code>) parameter.
	 * @see #executeMultiplexCommand(String,String,Object)
	 * @since 1.2
	 */
	protected void executeMultiplexCommand(final String topicKey, final String channelId, final String dataKey, final Object dataValue) {
		Dictionary data = new Hashtable();
		data.put(ProfileService.CHANNEL_ID_DATA_KEY, channelId);
		data.put(dataKey, dataValue);
		broadcast(topicKey + NotificationService.EXECUTE_EXTERNAL_KEY, data);
	}

	/**
	 * Gets the external keys (String[]) value.
	 * @return The external keys (<code>String[]</code>) value.
	 */
	protected String[] getExternalKeys() {
		return new String[] { prependPrefix("*") }; //$NON-NLS-1$
	}

	/**
	 * Get keys and settings from properties.
	 */
	protected void getKeysAndSettingsFromProperties() {
		this.prefixProperty = getString(EscObject.PREFIX_KEY, EMPTY_STRING);
		this.maxResponseWaitingTimeProperty = getInt(ProfileTestAgentService.MAX_RESPONSE_WAITING_TIME_PROPERTY, DEFAULT_MAX_RESPONSE_WAITING_TIME);
		this.maxStatusWaitingTimeProperty = getInt(ProfileTestAgentService.MAX_STATUS_WAITING_TIME_PROPERTY, DEFAULT_MAX_STATUS_WAITING_TIME);
	}

	/**
	 * Get measurement with the specified topic key parameter and return the Object result.
	 * @param topicKey
	 *		The topic key (<code>String</code>) parameter.
	 * @return Results of the get measurement (<code>Object</code>) value.
	 * @throws InterruptedException Interrupted Exception.
	 * @see #getMeasurement(String,String)
	 * @see #getMultiplexMeasurement(String,String)
	 * @see #getMultiplexMeasurement(String,String,String)
	 * @see #readMeasurement(String)
	 * @see #readMeasurement(String,Object)
	 * @see #readMeasurement(String,String)
	 * @see #readMeasurement(String,String,Object)
	 * @see #readMultiplexMeasurement(String,String)
	 * @see #readMultiplexMeasurement(String,String,Object)
	 * @see #readMultiplexMeasurement(String,String,String)
	 * @see #readMultiplexMeasurement(String,String,String,Object)
	 * @see #reportMeasurement(String,Object)
	 * @see #reportMeasurement(String,String,Object)
	 * @see #reportMultiplexMeasurement(String,String,Object)
	 * @see #reportMultiplexMeasurement(String,String,String,Object)
	 * @see #writeInvalidMeasurement(String,Object)
	 * @see #writeInvalidMultiplexMeasurement(String,String,Object)
	 * @see #writeMeasurement(String,Object)
	 * @see #writeMeasurement(String,String,Object)
	 * @see #writeMultiplexMeasurement(String,String,Object)
	 * @see #writeMultiplexMeasurement(String,String,String,Object)
	 */
	protected Object getMeasurement(final String topicKey) throws InterruptedException {
		return getMeasurement(topicKey, DeviceService.VALUE_DATA_KEY);
	}

	/**
	 * Get measurement with the specified topic key and data key parameters and return the Object result.
	 * @param topicKey
	 *		The topic key (<code>String</code>) parameter.
	 * @param dataKey
	 *		The data key (<code>String</code>) parameter.
	 * @return Results of the get measurement (<code>Object</code>) value.
	 * @throws InterruptedException Interrupted Exception.
	 * @see #getMeasurement(String)
	 * @see #getMultiplexMeasurement(String,String)
	 * @see #getMultiplexMeasurement(String,String,String)
	 * @see #readMeasurement(String)
	 * @see #readMeasurement(String,Object)
	 * @see #readMeasurement(String,String)
	 * @see #readMeasurement(String,String,Object)
	 * @see #readMultiplexMeasurement(String,String)
	 * @see #readMultiplexMeasurement(String,String,Object)
	 * @see #readMultiplexMeasurement(String,String,String)
	 * @see #readMultiplexMeasurement(String,String,String,Object)
	 * @see #reportMeasurement(String,Object)
	 * @see #reportMeasurement(String,String,Object)
	 * @see #reportMultiplexMeasurement(String,String,Object)
	 * @see #reportMultiplexMeasurement(String,String,String,Object)
	 * @see #writeInvalidMeasurement(String,Object)
	 * @see #writeInvalidMultiplexMeasurement(String,String,Object)
	 * @see #writeMeasurement(String,Object)
	 * @see #writeMeasurement(String,String,Object)
	 * @see #writeMultiplexMeasurement(String,String,Object)
	 * @see #writeMultiplexMeasurement(String,String,String,Object)
	 */
	protected Object getMeasurement(final String topicKey, final String dataKey) throws InterruptedException {
		NotificationProbe probe = getProbe();
		probe.setMark();
		broadcast(topicKey + NotificationService.GET_EXTERNAL_KEY, new Hashtable());
		assertTrue(Nls.format(ProfileTestAgent.DefaultResourceBundle.getString(Integer.toString(ProfileTestAgent.FAILED_TO_GET)), topicKey), probe.wasKeyPublished(topicKey, this.maxResponseWaitingTimeProperty));
		assertFalse(Nls.format(ProfileTestAgent.DefaultResourceBundle.getString(Integer.toString(ProfileTestAgent.UNEXPECTED_ERROR_RETURNED_FOR_GET)), topicKey), probe.wasKeyPublished(topicKey + NotificationService.ERROR_EXTERNAL_KEY));
		Object data = probe.getLastPublishedValue(topicKey, dataKey);
		probe.clearMark();
		return data;
	}

	/**
	 * Get multiplex measurement with the specified topic key and channel id parameters and return the Object result.
	 * @param topicKey
	 *		The topic key (<code>String</code>) parameter.
	 * @param channelId
	 *		The channel ID (<code>String</code>) parameter.
	 * @return Results of the get multiplex measurement (<code>Object</code>) value.
	 * @throws InterruptedException Interrupted Exception.
	 * @see #getMultiplexMeasurement(String,String,String)
	 * @see #readMultiplexMeasurement(String,String)
	 * @see #readMultiplexMeasurement(String,String,Object)
	 * @see #readMultiplexMeasurement(String,String,String)
	 * @see #readMultiplexMeasurement(String,String,String,Object)
	 * @see #reportMultiplexMeasurement(String,String,Object)
	 * @see #reportMultiplexMeasurement(String,String,String,Object)
	 * @see #writeInvalidMultiplexMeasurement(String,String,Object)
	 * @see #writeMultiplexMeasurement(String,String,Object)
	 * @see #writeMultiplexMeasurement(String,String,String,Object)
	 * @since 1.2
	 */
	protected Object getMultiplexMeasurement(final String topicKey, final String channelId) throws InterruptedException {
		return getMultiplexMeasurement(topicKey, channelId, DeviceService.VALUE_DATA_KEY);
	}

	/**
	 * Get multiplex measurement with the specified topic key, channel id and data key parameters and return the Object result.
	 * @param topicKey
	 *		The topic key (<code>String</code>) parameter.
	 * @param channelId
	 *		The channel ID (<code>String</code>) parameter.
	 * @param dataKey
	 *		The data key (<code>String</code>) parameter.
	 * @return Results of the get multiplex measurement (<code>Object</code>) value.
	 * @throws InterruptedException Interrupted Exception.
	 * @see #getMultiplexMeasurement(String,String)
	 * @see #readMultiplexMeasurement(String,String)
	 * @see #readMultiplexMeasurement(String,String,Object)
	 * @see #readMultiplexMeasurement(String,String,String)
	 * @see #readMultiplexMeasurement(String,String,String,Object)
	 * @see #reportMultiplexMeasurement(String,String,Object)
	 * @see #reportMultiplexMeasurement(String,String,String,Object)
	 * @see #writeInvalidMultiplexMeasurement(String,String,Object)
	 * @see #writeMultiplexMeasurement(String,String,Object)
	 * @see #writeMultiplexMeasurement(String,String,String,Object)
	 * @since 1.2
	 */
	protected Object getMultiplexMeasurement(final String topicKey, final String channelId, final String dataKey) throws InterruptedException {
		NotificationProbe probe = getProbe();
		probe.setMark();
		Dictionary data = new Hashtable();
		data.put(ProfileService.CHANNEL_ID_DATA_KEY, channelId);
		broadcast(topicKey + NotificationService.GET_EXTERNAL_KEY, data);
		assertTrue(Nls.format(ProfileTestAgent.DefaultResourceBundle.getString(Integer.toString(ProfileTestAgent.FAILED_TO_GET)), topicKey), probe.wasKeyPublished(topicKey, this.maxResponseWaitingTimeProperty));
		assertFalse(Nls.format(ProfileTestAgent.DefaultResourceBundle.getString(Integer.toString(ProfileTestAgent.UNEXPECTED_ERROR_RETURNED_FOR_GET)), topicKey), probe.wasKeyPublished(topicKey + NotificationService.ERROR_EXTERNAL_KEY));
		Object result = probe.getLastPublishedValue(topicKey, dataKey);
		probe.clearMark();
		return result;
	}

	/**
	 * Prepend prefix with the specified topic parameter and return the String result.
	 * @param topic
	 *		The topic (<code>String</code>) parameter.
	 * @return Results of the prepend prefix (<code>String</code>) value.
	 */
	protected String prependPrefix(final String topic) {
		if (this.prefixProperty.length() == 0) {
			return topic;
		}
		StringBuffer result = new StringBuffer(this.prefixProperty.length() + topic.length() + 1);
		result.append(this.prefixProperty);
		result.append(TOPIC_DELIMITER);
		result.append(topic);
		return result.toString();
	}

	/**
	 * Read measurement with the specified topic key parameter and return the Object result.
	 * @param topicKey
	 *		The topic key (<code>String</code>) parameter.
	 * @return Results of the read measurement (<code>Object</code>) value.
	 * @throws InterruptedException Interrupted Exception.
	 * @see #getMeasurement(String)
	 * @see #getMeasurement(String,String)
	 * @see #getMultiplexMeasurement(String,String)
	 * @see #getMultiplexMeasurement(String,String,String)
	 * @see #readMeasurement(String,Object)
	 * @see #readMeasurement(String,String)
	 * @see #readMeasurement(String,String,Object)
	 * @see #readMultiplexMeasurement(String,String)
	 * @see #readMultiplexMeasurement(String,String,Object)
	 * @see #readMultiplexMeasurement(String,String,String)
	 * @see #readMultiplexMeasurement(String,String,String,Object)
	 * @see #reportMeasurement(String,Object)
	 * @see #reportMeasurement(String,String,Object)
	 * @see #reportMultiplexMeasurement(String,String,Object)
	 * @see #reportMultiplexMeasurement(String,String,String,Object)
	 * @see #writeInvalidMeasurement(String,Object)
	 * @see #writeInvalidMultiplexMeasurement(String,String,Object)
	 * @see #writeMeasurement(String,Object)
	 * @see #writeMeasurement(String,String,Object)
	 * @see #writeMultiplexMeasurement(String,String,Object)
	 * @see #writeMultiplexMeasurement(String,String,String,Object)
	 */
	protected Object readMeasurement(final String topicKey) throws InterruptedException {
		return readMeasurement(topicKey, DeviceService.VALUE_DATA_KEY);
	}

	/**
	 * Read measurement with the specified topic key and data value parameters and return the Object result.
	 * @param topicKey
	 *		The topic key (<code>String</code>) parameter.
	 * @param dataValue
	 *		The data value (<code>Object</code>) parameter.
	 * @return Results of the read measurement (<code>Object</code>) value.
	 * @throws InterruptedException Interrupted Exception.
	 * @see #getMeasurement(String)
	 * @see #getMeasurement(String,String)
	 * @see #getMultiplexMeasurement(String,String)
	 * @see #getMultiplexMeasurement(String,String,String)
	 * @see #readMeasurement(String)
	 * @see #readMeasurement(String,String)
	 * @see #readMeasurement(String,String,Object)
	 * @see #readMultiplexMeasurement(String,String)
	 * @see #readMultiplexMeasurement(String,String,Object)
	 * @see #readMultiplexMeasurement(String,String,String)
	 * @see #readMultiplexMeasurement(String,String,String,Object)
	 * @see #reportMeasurement(String,Object)
	 * @see #reportMeasurement(String,String,Object)
	 * @see #reportMultiplexMeasurement(String,String,Object)
	 * @see #reportMultiplexMeasurement(String,String,String,Object)
	 * @see #writeInvalidMeasurement(String,Object)
	 * @see #writeInvalidMultiplexMeasurement(String,String,Object)
	 * @see #writeMeasurement(String,Object)
	 * @see #writeMeasurement(String,String,Object)
	 * @see #writeMultiplexMeasurement(String,String,Object)
	 * @see #writeMultiplexMeasurement(String,String,String,Object)
	 * @since 1.2
	 */
	protected Object readMeasurement(final String topicKey, final Object dataValue) throws InterruptedException {
		return readMeasurement(topicKey, DeviceService.VALUE_DATA_KEY, dataValue);
	}

	/**
	 * Read measurement with the specified topic key and data key parameters and return the Object result.
	 * @param topicKey
	 *		The topic key (<code>String</code>) parameter.
	 * @param dataKey
	 *		The data key (<code>String</code>) parameter.
	 * @return Results of the read measurement (<code>Object</code>) value.
	 * @throws InterruptedException Interrupted Exception.
	 * @see #getMeasurement(String)
	 * @see #getMeasurement(String,String)
	 * @see #getMultiplexMeasurement(String,String)
	 * @see #getMultiplexMeasurement(String,String,String)
	 * @see #readMeasurement(String)
	 * @see #readMeasurement(String,Object)
	 * @see #readMeasurement(String,String,Object)
	 * @see #readMultiplexMeasurement(String,String)
	 * @see #readMultiplexMeasurement(String,String,Object)
	 * @see #readMultiplexMeasurement(String,String,String)
	 * @see #readMultiplexMeasurement(String,String,String,Object)
	 * @see #reportMeasurement(String,Object)
	 * @see #reportMeasurement(String,String,Object)
	 * @see #reportMultiplexMeasurement(String,String,Object)
	 * @see #reportMultiplexMeasurement(String,String,String,Object)
	 * @see #writeInvalidMeasurement(String,Object)
	 * @see #writeInvalidMultiplexMeasurement(String,String,Object)
	 * @see #writeMeasurement(String,Object)
	 * @see #writeMeasurement(String,String,Object)
	 * @see #writeMultiplexMeasurement(String,String,Object)
	 * @see #writeMultiplexMeasurement(String,String,String,Object)
	 */
	protected Object readMeasurement(final String topicKey, final String dataKey) throws InterruptedException {
		return readMeasurement(topicKey, dataKey, null);
	}

	/**
	 * Read measurement with the specified topic key, data key and data value parameters and return the Object result.
	 * @param topicKey
	 *		The topic key (<code>String</code>) parameter.
	 * @param dataKey
	 *		The data key (<code>String</code>) parameter.
	 * @param dataValue
	 *		The data value (<code>Object</code>) parameter.
	 * @return Results of the read measurement (<code>Object</code>) value.
	 * @throws InterruptedException Interrupted Exception.
	 * @see #getMeasurement(String)
	 * @see #getMeasurement(String,String)
	 * @see #getMultiplexMeasurement(String,String)
	 * @see #getMultiplexMeasurement(String,String,String)
	 * @see #readMeasurement(String)
	 * @see #readMeasurement(String,Object)
	 * @see #readMeasurement(String,String)
	 * @see #readMultiplexMeasurement(String,String)
	 * @see #readMultiplexMeasurement(String,String,Object)
	 * @see #readMultiplexMeasurement(String,String,String)
	 * @see #readMultiplexMeasurement(String,String,String,Object)
	 * @see #reportMeasurement(String,Object)
	 * @see #reportMeasurement(String,String,Object)
	 * @see #reportMultiplexMeasurement(String,String,Object)
	 * @see #reportMultiplexMeasurement(String,String,String,Object)
	 * @see #writeInvalidMeasurement(String,Object)
	 * @see #writeInvalidMultiplexMeasurement(String,String,Object)
	 * @see #writeMeasurement(String,Object)
	 * @see #writeMeasurement(String,String,Object)
	 * @see #writeMultiplexMeasurement(String,String,Object)
	 * @see #writeMultiplexMeasurement(String,String,String,Object)
	 * @since 1.2
	 */
	protected Object readMeasurement(final String topicKey, final String dataKey, final Object dataValue) throws InterruptedException {
		NotificationProbe probe = getProbe();
		probe.setMark();
		Dictionary data = new Hashtable();
		data.put(dataKey, dataValue);
		broadcast(topicKey + NotificationService.READ_EXTERNAL_KEY, new Hashtable());
		assertTrue(Nls.format(ProfileTestAgent.DefaultResourceBundle.getString(Integer.toString(ProfileTestAgent.FAILED_TO_READ)), topicKey), probe.wasKeyPublished(topicKey, this.maxResponseWaitingTimeProperty));
		assertFalse(Nls.format(ProfileTestAgent.DefaultResourceBundle.getString(Integer.toString(ProfileTestAgent.UNEXPECTED_ERROR_RETURNED_FOR_READ)), topicKey), probe.wasKeyPublished(topicKey + NotificationService.ERROR_EXTERNAL_KEY));
		Object result = probe.getLastPublishedValue(topicKey, dataKey);
		probe.clearMark();
		return result;
	}

	/**
	 * Read multiplex measurement with the specified topic key and channel id parameters and return the Object result.
	 * @param topicKey
	 *		The topic key (<code>String</code>) parameter.
	 * @param channelId
	 *		The channel ID (<code>String</code>) parameter.
	 * @return Results of the read multiplex measurement (<code>Object</code>) value.
	 * @throws InterruptedException Interrupted Exception.
	 * @see #getMultiplexMeasurement(String,String)
	 * @see #getMultiplexMeasurement(String,String,String)
	 * @see #readMultiplexMeasurement(String,String,Object)
	 * @see #readMultiplexMeasurement(String,String,String)
	 * @see #readMultiplexMeasurement(String,String,String,Object)
	 * @see #reportMultiplexMeasurement(String,String,Object)
	 * @see #reportMultiplexMeasurement(String,String,String,Object)
	 * @see #writeInvalidMultiplexMeasurement(String,String,Object)
	 * @see #writeMultiplexMeasurement(String,String,Object)
	 * @see #writeMultiplexMeasurement(String,String,String,Object)
	 * @since 1.2
	 */
	protected Object readMultiplexMeasurement(final String topicKey, final String channelId) throws InterruptedException {
		return readMultiplexMeasurement(topicKey, channelId, DeviceService.VALUE_DATA_KEY);
	}

	/**
	 * Read multiplex measurement with the specified topic key, channel id and data value parameters and return the Object result.
	 * @param topicKey
	 *		The topic key (<code>String</code>) parameter.
	 * @param channelId
	 *		The channel ID (<code>String</code>) parameter.
	 * @param dataValue
	 *		The data value (<code>Object</code>) parameter.
	 * @return Results of the read multiplex measurement (<code>Object</code>) value.
	 * @throws InterruptedException Interrupted Exception.
	 * @see #getMultiplexMeasurement(String,String)
	 * @see #getMultiplexMeasurement(String,String,String)
	 * @see #readMultiplexMeasurement(String,String)
	 * @see #readMultiplexMeasurement(String,String,String)
	 * @see #readMultiplexMeasurement(String,String,String,Object)
	 * @see #reportMultiplexMeasurement(String,String,Object)
	 * @see #reportMultiplexMeasurement(String,String,String,Object)
	 * @see #writeInvalidMultiplexMeasurement(String,String,Object)
	 * @see #writeMultiplexMeasurement(String,String,Object)
	 * @see #writeMultiplexMeasurement(String,String,String,Object)
	 * @since 1.2
	 */
	protected Object readMultiplexMeasurement(final String topicKey, final String channelId, final Object dataValue) throws InterruptedException {
		return readMultiplexMeasurement(topicKey, channelId, DeviceService.VALUE_DATA_KEY, dataValue);
	}

	/**
	 * Read multiplex measurement with the specified topic key, channel id and data key parameters and return the Object result.
	 * @param topicKey
	 *		The topic key (<code>String</code>) parameter.
	 * @param channelId
	 *		The channel ID (<code>String</code>) parameter.
	 * @param dataKey
	 *		The data key (<code>String</code>) parameter.
	 * @return Results of the read multiplex measurement (<code>Object</code>) value.
	 * @throws InterruptedException Interrupted Exception.
	 * @see #getMultiplexMeasurement(String,String)
	 * @see #getMultiplexMeasurement(String,String,String)
	 * @see #readMultiplexMeasurement(String,String)
	 * @see #readMultiplexMeasurement(String,String,Object)
	 * @see #readMultiplexMeasurement(String,String,String,Object)
	 * @see #reportMultiplexMeasurement(String,String,Object)
	 * @see #reportMultiplexMeasurement(String,String,String,Object)
	 * @see #writeInvalidMultiplexMeasurement(String,String,Object)
	 * @see #writeMultiplexMeasurement(String,String,Object)
	 * @see #writeMultiplexMeasurement(String,String,String,Object)
	 * @since 1.2
	 */
	protected Object readMultiplexMeasurement(final String topicKey, final String channelId, final String dataKey) throws InterruptedException {
		return readMultiplexMeasurement(topicKey, channelId, dataKey, null);
	}

	/**
	 * Read multiplex measurement with the specified topic key, channel id, data key and data value parameters and return the Object result.
	 * @param topicKey
	 *		The topic key (<code>String</code>) parameter.
	 * @param channelId
	 *		The channel ID (<code>String</code>) parameter.
	 * @param dataKey
	 *		The data key (<code>String</code>) parameter.
	 * @param dataValue
	 *		The data value (<code>Object</code>) parameter.
	 * @return Results of the read multiplex measurement (<code>Object</code>) value.
	 * @throws InterruptedException Interrupted Exception.
	 * @see #getMultiplexMeasurement(String,String)
	 * @see #getMultiplexMeasurement(String,String,String)
	 * @see #readMultiplexMeasurement(String,String)
	 * @see #readMultiplexMeasurement(String,String,Object)
	 * @see #readMultiplexMeasurement(String,String,String)
	 * @see #reportMultiplexMeasurement(String,String,Object)
	 * @see #reportMultiplexMeasurement(String,String,String,Object)
	 * @see #writeInvalidMultiplexMeasurement(String,String,Object)
	 * @see #writeMultiplexMeasurement(String,String,Object)
	 * @see #writeMultiplexMeasurement(String,String,String,Object)
	 * @since 1.2
	 */
	protected Object readMultiplexMeasurement(final String topicKey, final String channelId, final String dataKey, final Object dataValue) throws InterruptedException {
		NotificationProbe probe = getProbe();
		probe.setMark();
		Dictionary data = new Hashtable();
		data.put(ProfileService.CHANNEL_ID_DATA_KEY, channelId);
		data.put(dataKey, dataValue);
		broadcast(topicKey + NotificationService.READ_EXTERNAL_KEY, data);
		assertTrue(Nls.format(ProfileTestAgent.DefaultResourceBundle.getString(Integer.toString(ProfileTestAgent.FAILED_TO_READ)), topicKey), probe.wasKeyPublished(topicKey, this.maxResponseWaitingTimeProperty));
		assertFalse(Nls.format(ProfileTestAgent.DefaultResourceBundle.getString(Integer.toString(ProfileTestAgent.UNEXPECTED_ERROR_RETURNED_FOR_READ)), topicKey), probe.wasKeyPublished(topicKey + NotificationService.ERROR_EXTERNAL_KEY));
		Object result = probe.getLastPublishedValue(topicKey, dataKey);
		probe.clearMark();
		return result;
	}

	/**
	 * Report measurement with the specified topic key and data value parameters.
	 * @param topicKey
	 *		The topic key (<code>String</code>) parameter.
	 * @param dataValue
	 *		The data value (<code>Object</code>) parameter.
	 * @see #getMeasurement(String)
	 * @see #getMeasurement(String,String)
	 * @see #getMultiplexMeasurement(String,String)
	 * @see #getMultiplexMeasurement(String,String,String)
	 * @see #readMeasurement(String)
	 * @see #readMeasurement(String,Object)
	 * @see #readMeasurement(String,String)
	 * @see #readMeasurement(String,String,Object)
	 * @see #readMultiplexMeasurement(String,String)
	 * @see #readMultiplexMeasurement(String,String,Object)
	 * @see #readMultiplexMeasurement(String,String,String)
	 * @see #readMultiplexMeasurement(String,String,String,Object)
	 * @see #reportMeasurement(String,String,Object)
	 * @see #reportMultiplexMeasurement(String,String,Object)
	 * @see #reportMultiplexMeasurement(String,String,String,Object)
	 * @see #writeInvalidMeasurement(String,Object)
	 * @see #writeInvalidMultiplexMeasurement(String,String,Object)
	 * @see #writeMeasurement(String,Object)
	 * @see #writeMeasurement(String,String,Object)
	 * @see #writeMultiplexMeasurement(String,String,Object)
	 * @see #writeMultiplexMeasurement(String,String,String,Object)
	 */
	protected void reportMeasurement(final String topicKey, final Object dataValue) {
		reportMeasurement(topicKey, DeviceService.VALUE_DATA_KEY, dataValue);
	}

	/**
	 * Report measurement with the specified topic key, data key and data value parameters.
	 * @param topicKey
	 *		The topic key (<code>String</code>) parameter.
	 * @param dataKey
	 *		The data key (<code>String</code>) parameter.
	 * @param dataValue
	 *		The data value (<code>Object</code>) parameter.
	 * @see #getMeasurement(String)
	 * @see #getMeasurement(String,String)
	 * @see #getMultiplexMeasurement(String,String)
	 * @see #getMultiplexMeasurement(String,String,String)
	 * @see #readMeasurement(String)
	 * @see #readMeasurement(String,Object)
	 * @see #readMeasurement(String,String)
	 * @see #readMeasurement(String,String,Object)
	 * @see #readMultiplexMeasurement(String,String)
	 * @see #readMultiplexMeasurement(String,String,Object)
	 * @see #readMultiplexMeasurement(String,String,String)
	 * @see #readMultiplexMeasurement(String,String,String,Object)
	 * @see #reportMeasurement(String,Object)
	 * @see #reportMultiplexMeasurement(String,String,Object)
	 * @see #reportMultiplexMeasurement(String,String,String,Object)
	 * @see #writeInvalidMeasurement(String,Object)
	 * @see #writeInvalidMultiplexMeasurement(String,String,Object)
	 * @see #writeMeasurement(String,Object)
	 * @see #writeMeasurement(String,String,Object)
	 * @see #writeMultiplexMeasurement(String,String,Object)
	 * @see #writeMultiplexMeasurement(String,String,String,Object)
	 */
	protected void reportMeasurement(final String topicKey, final String dataKey, final Object dataValue) {
		Dictionary data = new Hashtable();
		data.put(dataKey, dataValue);
		broadcast(topicKey, data);
	}

	/**
	 * Report multiplex measurement with the specified topic key, channel id and data value parameters.
	 * @param topicKey
	 *		The topic key (<code>String</code>) parameter.
	 * @param channelId
	 *		The channel ID (<code>String</code>) parameter.
	 * @param dataValue
	 *		The data value (<code>Object</code>) parameter.
	 * @see #getMultiplexMeasurement(String,String)
	 * @see #getMultiplexMeasurement(String,String,String)
	 * @see #readMultiplexMeasurement(String,String)
	 * @see #readMultiplexMeasurement(String,String,Object)
	 * @see #readMultiplexMeasurement(String,String,String)
	 * @see #readMultiplexMeasurement(String,String,String,Object)
	 * @see #reportMultiplexMeasurement(String,String,String,Object)
	 * @see #writeInvalidMultiplexMeasurement(String,String,Object)
	 * @see #writeMultiplexMeasurement(String,String,Object)
	 * @see #writeMultiplexMeasurement(String,String,String,Object)
	 * @since 1.2
	 */
	protected void reportMultiplexMeasurement(final String topicKey, final String channelId, final Object dataValue) {
		reportMultiplexMeasurement(topicKey, channelId, DeviceService.VALUE_DATA_KEY, dataValue);
	}

	/**
	 * Report multiplex measurement with the specified topic key, channel id, data key and data value parameters.
	 * @param topicKey
	 *		The topic key (<code>String</code>) parameter.
	 * @param channelId
	 *		The channel ID (<code>String</code>) parameter.
	 * @param dataKey
	 *		The data key (<code>String</code>) parameter.
	 * @param dataValue
	 *		The data value (<code>Object</code>) parameter.
	 * @see #getMultiplexMeasurement(String,String)
	 * @see #getMultiplexMeasurement(String,String,String)
	 * @see #readMultiplexMeasurement(String,String)
	 * @see #readMultiplexMeasurement(String,String,Object)
	 * @see #readMultiplexMeasurement(String,String,String)
	 * @see #readMultiplexMeasurement(String,String,String,Object)
	 * @see #reportMultiplexMeasurement(String,String,Object)
	 * @see #writeInvalidMultiplexMeasurement(String,String,Object)
	 * @see #writeMultiplexMeasurement(String,String,Object)
	 * @see #writeMultiplexMeasurement(String,String,String,Object)
	 * @since 1.2
	 */
	protected void reportMultiplexMeasurement(final String topicKey, final String channelId, final String dataKey, final Object dataValue) {
		Dictionary data = new Hashtable();
		data.put(ProfileService.CHANNEL_ID_DATA_KEY, channelId);
		data.put(dataKey, dataValue);
		broadcast(topicKey, data);
	}

	/**
	 * Set up.
	 * @throws Exception Exception.
	 */
	protected void setUp() throws Exception {
		getContext().log(EscObject.LOG_DEBUG, Nls.format(ProfileTestAgent.DefaultResourceBundle.getString(Integer.toString(ProfileTestAgent.RUNNING_TEST_CASE)), toString()));
		getKeysAndSettingsFromProperties();
		super.setUp();
	}

	/**
	 * Wait for profile start with the specified profile name parameter.
	 * @param profileName
	 *		The profile name (<code>String</code>) parameter.
	 * @throws InterruptedException Interrupted Exception.
	 */
	protected void waitForProfileStart(final String profileName) throws InterruptedException {
		waitForProfileState(profileName, ProfileService.STARTED);
	}

	/**
	 * Wait for profile state with the specified profile name and expected state parameters.
	 * @param profileName
	 *		The profile name (<code>String</code>) parameter.
	 * @param expectedState
	 *		The expected state (<code>int</code>) parameter.
	 * @throws InterruptedException Interrupted Exception.
	 */
	protected void waitForProfileState(final String profileName, final int expectedState) throws InterruptedException {
		NotificationProbe probe = getProbe();
		String profileStatusTopic = prependPrefix(profileName + STATUS_EXTERNAL_KEY);
		Integer expectedStateData = new Integer(expectedState);
		Object stateData = null;
		probe.clear();
		broadcast(profileStatusTopic + NotificationService.GET_EXTERNAL_KEY, new Hashtable());
		long now = System.currentTimeMillis();
		long limitTime = now + this.maxStatusWaitingTimeProperty;
		while (!expectedStateData.equals(stateData) && (now < limitTime)) {
			int time = (int) (limitTime - now);
			if (probe.wasKeyPublished(profileStatusTopic, time)) {
				Map value = (Map) probe.getLastPublishedValue(profileStatusTopic, DeviceService.VALUE_DATA_KEY);
				if (value != null) {
					stateData = value.get(ProfileService.STATE_DATA_KEY);
				}
			}
			probe.clear();
			now = System.currentTimeMillis();
		}
		assertEquals(Nls.format(ProfileTestAgent.DefaultResourceBundle.getString(Integer.toString(ProfileTestAgent.PROFILE_FAILED_TO_CHANGE_STATE)), profileName), expectedStateData, stateData);
	}

	/**
	 * Wait for signal with the specified topic key, expected count and duration parameters and return the int result.
	 * @param topicKey
	 *		The topic key (<code>String</code>) parameter.
	 * @param expectedCount
	 *		The expected count (<code>int</code>) parameter.
	 * @param duration
	 *		The duration (<code>int</code>) parameter.
	 * @return Results of the wait for signal (<code>int</code>) value.
	 * @throws InterruptedException Interrupted Exception.
	 */
	protected int waitForSignal(final String topicKey, final int expectedCount, final int duration) throws InterruptedException {
		NotificationProbe probe = getProbe();
		int count = probe.waitKeyPublished(topicKey, expectedCount, duration);
		assertFalse(Nls.format(ProfileTestAgent.DefaultResourceBundle.getString(Integer.toString(ProfileTestAgent.UNEXPECTED_ERROR_RETURNED_FOR_SIGNAL)), topicKey), probe.wasKeyPublished(topicKey + NotificationService.ERROR_EXTERNAL_KEY));
		return count;
	}

	/**
	 * Write invalid measurement with the specified topic key and data value parameters.
	 * @param topicKey
	 *		The topic key (<code>String</code>) parameter.
	 * @param dataValue
	 *		The data value (<code>Object</code>) parameter.
	 * @throws InterruptedException Interrupted Exception.
	 */
	protected void writeInvalidMeasurement(final String topicKey, final Object dataValue) throws InterruptedException {
		NotificationProbe probe = getProbe();
		Object dataValueOld = getMeasurement(topicKey);
		probe.setMark();
		writeMeasurement(topicKey, DeviceService.VALUE_DATA_KEY, dataValue);
		assertTrue(Nls.format(ProfileTestAgent.DefaultResourceBundle.getString(Integer.toString(ProfileTestAgent.NO_ERROR_RETURNED_FOR_INVALID_WRITE)), topicKey), probe.wasKeyPublished(topicKey + NotificationService.ERROR_EXTERNAL_KEY,
				this.maxResponseWaitingTimeProperty));
		probe.clearMark();
		probe.setMark();
		assertEquals(Nls.format(ProfileTestAgent.DefaultResourceBundle.getString(Integer.toString(ProfileTestAgent.INCORRECT_VALUE_AFTER)), topicKey), dataValueOld, getMeasurement(topicKey));
		probe.clearMark();
	}

	/**
	 * Write invalid multiplex measurement with the specified topic key, channel id and data value parameters.
	 * @param topicKey
	 *		The topic key (<code>String</code>) parameter.
	 * @param channelId
	 *		The channel ID (<code>String</code>) parameter.
	 * @param dataValue
	 *		The data value (<code>Object</code>) parameter.
	 * @throws InterruptedException Interrupted Exception.
	 * @since 1.2
	 */
	protected void writeInvalidMultiplexMeasurement(final String topicKey, final String channelId, final Object dataValue) throws InterruptedException {
		NotificationProbe probe = getProbe();
		Object dataValueOld = getMultiplexMeasurement(topicKey, channelId);
		probe.setMark();
		writeMultiplexMeasurement(topicKey, channelId, DeviceService.VALUE_DATA_KEY, dataValue);
		assertTrue(Nls.format(ProfileTestAgent.DefaultResourceBundle.getString(Integer.toString(ProfileTestAgent.NO_ERROR_RETURNED_FOR_INVALID_WRITE)), topicKey), probe.wasKeyPublished(topicKey + NotificationService.ERROR_EXTERNAL_KEY,
				this.maxResponseWaitingTimeProperty));
		probe.clearMark();
		probe.setMark();
		assertEquals(Nls.format(ProfileTestAgent.DefaultResourceBundle.getString(Integer.toString(ProfileTestAgent.INCORRECT_VALUE_AFTER)), topicKey), dataValueOld, getMultiplexMeasurement(topicKey, channelId));
		probe.clearMark();
	}

	/**
	 * Write measurement with the specified topic key and data value parameters.
	 * @param topicKey
	 *		The topic key (<code>String</code>) parameter.
	 * @param dataValue
	 *		The data value (<code>Object</code>) parameter.
	 * @throws InterruptedException Interrupted Exception.
	 * @see #getMeasurement(String)
	 * @see #getMeasurement(String,String)
	 * @see #getMultiplexMeasurement(String,String)
	 * @see #getMultiplexMeasurement(String,String,String)
	 * @see #readMeasurement(String)
	 * @see #readMeasurement(String,Object)
	 * @see #readMeasurement(String,String)
	 * @see #readMeasurement(String,String,Object)
	 * @see #readMultiplexMeasurement(String,String)
	 * @see #readMultiplexMeasurement(String,String,Object)
	 * @see #readMultiplexMeasurement(String,String,String)
	 * @see #readMultiplexMeasurement(String,String,String,Object)
	 * @see #reportMeasurement(String,Object)
	 * @see #reportMeasurement(String,String,Object)
	 * @see #reportMultiplexMeasurement(String,String,Object)
	 * @see #reportMultiplexMeasurement(String,String,String,Object)
	 * @see #writeInvalidMeasurement(String,Object)
	 * @see #writeInvalidMultiplexMeasurement(String,String,Object)
	 * @see #writeMeasurement(String,String,Object)
	 * @see #writeMultiplexMeasurement(String,String,Object)
	 * @see #writeMultiplexMeasurement(String,String,String,Object)
	 */
	protected void writeMeasurement(final String topicKey, final Object dataValue) throws InterruptedException {
		NotificationProbe probe = getProbe();
		Object dataValueOld = getMeasurement(topicKey);
		probe.setMark();
		writeMeasurement(topicKey, DeviceService.VALUE_DATA_KEY, dataValue);
		if (!dataValue.equals(dataValueOld)) {
			assertTrue(Nls.format(ProfileTestAgent.DefaultResourceBundle.getString(Integer.toString(ProfileTestAgent.FAILED_TO_WRITE)), topicKey), probe.wasKeyPublished(topicKey, this.maxResponseWaitingTimeProperty));
			assertFalse(Nls.format(ProfileTestAgent.DefaultResourceBundle.getString(Integer.toString(ProfileTestAgent.UNEXPECTED_ERROR_RETURNED_FOR_WRITE)), topicKey), probe.wasKeyPublished(topicKey + NotificationService.ERROR_EXTERNAL_KEY));
			Dictionary data = probe.getLastPublishedData(topicKey);
			assertEquals(Nls.format(ProfileTestAgent.DefaultResourceBundle.getString(Integer.toString(ProfileTestAgent.WRITE_REPORTS_INCORRECT_VALUE)), topicKey), dataValue, data.get(DeviceService.VALUE_DATA_KEY));
			assertEquals(Nls.format(ProfileTestAgent.DefaultResourceBundle.getString(Integer.toString(ProfileTestAgent.WRITE_REPORTS_INCORRECT_OLD_VALUE)), topicKey), dataValueOld, data.get(DeviceService.VALUE_OLD_DATA_KEY));
		}
		probe.clearMark();
	}

	/**
	 * Write measurement with the specified topic key, data key and data value parameters.
	 * @param topicKey
	 *		The topic key (<code>String</code>) parameter.
	 * @param dataKey
	 *		The data key (<code>String</code>) parameter.
	 * @param dataValue
	 *		The data value (<code>Object</code>) parameter.
	 * @see #getMeasurement(String)
	 * @see #getMeasurement(String,String)
	 * @see #getMultiplexMeasurement(String,String)
	 * @see #getMultiplexMeasurement(String,String,String)
	 * @see #readMeasurement(String)
	 * @see #readMeasurement(String,Object)
	 * @see #readMeasurement(String,String)
	 * @see #readMeasurement(String,String,Object)
	 * @see #readMultiplexMeasurement(String,String)
	 * @see #readMultiplexMeasurement(String,String,Object)
	 * @see #readMultiplexMeasurement(String,String,String)
	 * @see #readMultiplexMeasurement(String,String,String,Object)
	 * @see #reportMeasurement(String,Object)
	 * @see #reportMeasurement(String,String,Object)
	 * @see #reportMultiplexMeasurement(String,String,Object)
	 * @see #reportMultiplexMeasurement(String,String,String,Object)
	 * @see #writeInvalidMeasurement(String,Object)
	 * @see #writeInvalidMultiplexMeasurement(String,String,Object)
	 * @see #writeMeasurement(String,Object)
	 * @see #writeMultiplexMeasurement(String,String,Object)
	 * @see #writeMultiplexMeasurement(String,String,String,Object)
	 */
	protected void writeMeasurement(final String topicKey, final String dataKey, final Object dataValue) {
		Dictionary data = new Hashtable();
		data.put(dataKey, dataValue);
		broadcast(topicKey + NotificationService.WRITE_EXTERNAL_KEY, data);
	}

	/**
	 * Write multiplex measurement with the specified topic key, channel id and data value parameters.
	 * @param topicKey
	 *		The topic key (<code>String</code>) parameter.
	 * @param channelId
	 *		The channel ID (<code>String</code>) parameter.
	 * @param dataValue
	 *		The data value (<code>Object</code>) parameter.
	 * @throws InterruptedException Interrupted Exception.
	 * @see #getMultiplexMeasurement(String,String)
	 * @see #getMultiplexMeasurement(String,String,String)
	 * @see #readMultiplexMeasurement(String,String)
	 * @see #readMultiplexMeasurement(String,String,Object)
	 * @see #readMultiplexMeasurement(String,String,String)
	 * @see #readMultiplexMeasurement(String,String,String,Object)
	 * @see #reportMultiplexMeasurement(String,String,Object)
	 * @see #reportMultiplexMeasurement(String,String,String,Object)
	 * @see #writeInvalidMultiplexMeasurement(String,String,Object)
	 * @see #writeMultiplexMeasurement(String,String,String,Object)
	 * @since 1.2
	 */
	protected void writeMultiplexMeasurement(final String topicKey, final String channelId, final Object dataValue) throws InterruptedException {
		NotificationProbe probe = getProbe();
		Object dataValueOld = getMultiplexMeasurement(topicKey, channelId);
		probe.setMark();
		writeMultiplexMeasurement(topicKey, channelId, DeviceService.VALUE_DATA_KEY, dataValue);
		if (!dataValue.equals(dataValueOld)) {
			assertTrue(Nls.format(ProfileTestAgent.DefaultResourceBundle.getString(Integer.toString(ProfileTestAgent.FAILED_TO_WRITE)), topicKey), probe.wasKeyPublished(topicKey, this.maxResponseWaitingTimeProperty));
			assertFalse(Nls.format(ProfileTestAgent.DefaultResourceBundle.getString(Integer.toString(ProfileTestAgent.UNEXPECTED_ERROR_RETURNED_FOR_WRITE)), topicKey), probe.wasKeyPublished(topicKey + NotificationService.ERROR_EXTERNAL_KEY));
			Dictionary data = probe.getLastPublishedData(topicKey);
			assertEquals(Nls.format(ProfileTestAgent.DefaultResourceBundle.getString(Integer.toString(ProfileTestAgent.WRITE_REPORTS_INCORRECT_VALUE)), topicKey), dataValue, data.get(DeviceService.VALUE_DATA_KEY));
			assertEquals(Nls.format(ProfileTestAgent.DefaultResourceBundle.getString(Integer.toString(ProfileTestAgent.WRITE_REPORTS_INCORRECT_OLD_VALUE)), topicKey), dataValueOld, data.get(DeviceService.VALUE_OLD_DATA_KEY));
		}
		probe.clearMark();
	}

	/**
	 * Write multiplex measurement with the specified topic key, channel id, data key and data value parameters.
	 * @param topicKey
	 *		The topic key (<code>String</code>) parameter.
	 * @param channelId
	 *		The channel ID (<code>String</code>) parameter.
	 * @param dataKey
	 *		The data key (<code>String</code>) parameter.
	 * @param dataValue
	 *		The data value (<code>Object</code>) parameter.
	 * @see #getMultiplexMeasurement(String,String)
	 * @see #getMultiplexMeasurement(String,String,String)
	 * @see #readMultiplexMeasurement(String,String)
	 * @see #readMultiplexMeasurement(String,String,Object)
	 * @see #readMultiplexMeasurement(String,String,String)
	 * @see #readMultiplexMeasurement(String,String,String,Object)
	 * @see #reportMultiplexMeasurement(String,String,Object)
	 * @see #reportMultiplexMeasurement(String,String,String,Object)
	 * @see #writeInvalidMultiplexMeasurement(String,String,Object)
	 * @see #writeMultiplexMeasurement(String,String,Object)
	 * @since 1.2
	 */
	protected void writeMultiplexMeasurement(final String topicKey, final String channelId, final String dataKey, final Object dataValue) {
		Dictionary data = new Hashtable();
		data.put(ProfileService.CHANNEL_ID_DATA_KEY, channelId);
		data.put(dataKey, dataValue);
		broadcast(topicKey + NotificationService.WRITE_EXTERNAL_KEY, data);
	}
}
