package org.eclipse.soda.dk.testagent;

/*************************************************************************
 * Copyright (c) 2007, 2009 IBM.                                         *
 * All rights reserved. This program and the accompanying materials      *
 * are made available under the terms of the Eclipse Public License v1.0 *
 * which accompanies this distribution, and is available at              *
 * http://www.eclipse.org/legal/epl-v10.html                             *
 *                                                                       *
 * Contributors:                                                         *
 *     IBM - initial API and implementation                              *
 ************************************************************************/
import java.util.Dictionary;
import java.util.Enumeration;
import java.util.Hashtable;
import org.eclipse.soda.dk.core.EscConfiguration;
import org.eclipse.soda.dk.core.service.ConfigurationService;
import org.eclipse.soda.dk.notification.service.NotificationService;
import org.eclipse.soda.dk.testagent.framework.TestContext;
import org.eclipse.soda.dk.testagent.service.TestAgentService;
import org.eclipse.soda.dk.testagent.service.TestSynchronizationService;

/**
 * @author IBM
 * @version 1.2.0
 * @since 1.0
 */
public class ConfiguredTestContext implements TestContext {
	/**
	 * Define the test agent (TestAgent) field.
	 */
	private TestAgent testAgent;

	/**
	 * Define the sync service (TestSynchronizationService) field.
	 */
	private TestSynchronizationService syncService;

	/**
	 * Define the configuration service (ConfigurationService) field.
	 */
	private ConfigurationService configurationService;

	/**
	 * Constructs an instance of this class from the specified agent, sync service and properties parameters.
	 * @param agent	The agent (<code>TestAgent</code>) parameter.
	 * @param syncService	The sync service (<code>TestSynchronizationService</code>) parameter.
	 * @param properties	The properties (<code>Dictionary</code>) parameter.
	 */
	ConfiguredTestContext(final TestAgent agent, final TestSynchronizationService syncService, final Dictionary properties) {
		this.testAgent = agent;
		this.syncService = syncService;
		Dictionary agentProperties;
		if (agent.getConfigurationService() == null) {
			agentProperties = new Hashtable();
		} else {
			agentProperties = agent.getConfigurationService().getProperties();
		}
		this.configurationService = new EscConfiguration(updateProperties(agentProperties, properties));
	}

	/**
	 * Gets the configuration information (Dictionary) value.
	 * @return	The configuration information (<code>Dictionary</code>) value.
	 * @see #getConfigurationInformation(String)
	 */
	public Dictionary getConfigurationInformation() {
		return this.configurationService.getProperties();
	}

	/**
	 * Get configuration information with the specified property name parameter and return the Object result.
	 * @param propertyName	The property name (<code>String</code>) parameter.
	 * @return	Results of the get configuration information (<code>Object</code>) value.
	 * @see #getConfigurationInformation()
	 */
	public Object getConfigurationInformation(final String propertyName) {
		return getConfigurationInformation().get(propertyName);
	}

	/**
	 * Gets the configuration service value.
	 * @return	The configuration service (<code>ConfigurationService</code>) value.
	 */
	public ConfigurationService getConfigurationService() {
		return this.configurationService;
	}

	/**
	 * Gets the notification service value.
	 * @return	The notification service (<code>NotificationService</code>) value.
	 */
	public NotificationService getNotificationService() {
		return this.testAgent.getNotificationService();
	}

	/**
	 * Gets the synchronization service (TestSynchronizationService) value.
	 * @return	The synchronization service (<code>TestSynchronizationService</code>) value.
	 */
	public TestSynchronizationService getSynchronizationService() {
		return this.syncService;
	}

	/**
	 * Gets the test agent (TestAgentService) value.
	 * @return	The test agent (<code>TestAgentService</code>) value.
	 */
	public TestAgentService getTestAgent() {
		return this.testAgent;
	}

	/**
	 * Log with the specified severity and message parameters.
	 * @param severity	The severity (<code>int</code>) parameter.
	 * @param message	The message (<code>String</code>) parameter.
	 * @see #log(int,String,Throwable)
	 */
	public void log(final int severity, final String message) {
		this.testAgent.log(severity, message);
	}

	/**
	 * Log with the specified severity, message and exception parameters.
	 * @param severity	The severity (<code>int</code>) parameter.
	 * @param message	The message (<code>String</code>) parameter.
	 * @param exception	The exception (<code>Throwable</code>) parameter.
	 * @see #log(int,String)
	 */
	public void log(final int severity, final String message, final Throwable exception) {
		this.testAgent.log(severity, message, exception);
	}

	/**
	 * Update properties with the specified default properties and properties parameters and return the Dictionary result.
	 * @param defaultProperties	The default properties (<code>Dictionary</code>) parameter.
	 * @param properties	The properties (<code>Dictionary</code>) parameter.
	 * @return	Results of the update properties (<code>Dictionary</code>) value.
	 */
	private Dictionary updateProperties(final Dictionary defaultProperties, final Dictionary properties) {
		if (properties == null) {
			return defaultProperties;
		}
		Dictionary mergedProperties = new Hashtable(defaultProperties.size() + properties.size());
		for (Enumeration e = defaultProperties.keys(); e.hasMoreElements();) {
			Object key = e.nextElement();
			mergedProperties.put(key, defaultProperties.get(key));
		}
		for (Enumeration e = properties.keys(); e.hasMoreElements();) {
			Object key = e.nextElement();
			mergedProperties.put(key, properties.get(key));
		}
		return mergedProperties;
	}
}
