package org.eclipse.soda.dk.testagent;

/*************************************************************************
 * Copyright (c) 2006, 2009 IBM.                                         *
 * All rights reserved. This program and the accompanying materials      *
 * are made available under the terms of the Eclipse Public License v1.0 *
 * which accompanies this distribution, and is available at              *
 * http://www.eclipse.org/legal/epl-v10.html                             *
 *                                                                       *
 * Contributors:                                                         *
 *     IBM - initial API and implementation                              *
 ************************************************************************/
import java.lang.reflect.InvocationTargetException;
import java.lang.reflect.Method;
import java.lang.reflect.Modifier;
import java.util.Dictionary;
import java.util.ResourceBundle;
import junit.framework.Test;
import junit.framework.TestResult;
import junit.framework.TestSuite;
import junit.runner.BaseTestRunner;
import junit.textui.TestRunner;
import org.eclipse.soda.dk.core.ConfigurableObject;
import org.eclipse.soda.dk.core.EscConfiguration;
import org.eclipse.soda.dk.core.EscObject;
import org.eclipse.soda.dk.nls.Nls;
import org.eclipse.soda.dk.notification.Notification;
import org.eclipse.soda.dk.notification.service.NotificationService;
import org.eclipse.soda.dk.testagent.framework.TestContext;
import org.eclipse.soda.dk.testagent.service.TestAgentRunService;
import org.eclipse.soda.dk.testagent.service.TestAgentService;
import org.eclipse.soda.dk.testagent.service.TestSynchronizationService;

/**
 * @author IBM
 * @version 1.2.0
 * @since 1.0
 */
public class TestAgent extends ConfigurableObject implements TestAgentService {
	/**
	 * Define the test manager class name (String) constant.
	 */
	public static final String TEST_MANAGER_CLASS_NAME = "org.eclipse.soda.dk.testmanager.TestManager"; //$NON-NLS-1$

	/**
	 * Define the notification sevice not available (int) constant.
	 */
	public static final int NOTIFICATION_SEVICE_NOT_AVAILABLE = 7000;

	/**
	 * Define the missing configuration (int) constant.
	 */
	public static final int MISSING_CONFIGURATION = 7001;

	/**
	 * Define the missing synchronization (int) constant.
	 */
	public static final int MISSING_SYNCHRONIZATION = 7002;

	/**
	 * Define the methode suite must be static (int) constant.
	 */
	public static final int METHODE_SUITE_MUST_BE_STATIC = 7003;

	/**
	 * Define the failed to invole suite (int) constant.
	 */
	public static final int FAILED_TO_INVOLE_SUITE = 7004;

	/**
	 * Define the failed to run test (int) constant.
	 */
	public static final int FAILED_TO_RUN_TEST = 7005;

	/**
	 * Define the usage (int) constant.
	 */
	public static final int USAGE = 7006;

	/**
	 * Define the OK cancel prompt (int) constant.
	 */
	public static final int OK_CANCEL_PROMPT = 7007;

	/**
	 * Define the prompt (int) constant.
	 */
	public static final int PROMPT = 7008;

	/**
	 * Define the yes char (int) constant.
	 */
	public static final int YES_CHAR = 7009;

	/**
	 * Define the no char (int) constant.
	 */
	public static final int NO_CHAR = 7010;

	/**
	 * Define the invalid reply (int) constant.
	 */
	public static final int INVALID_REPLY = 7011;

	/**
	 * Define the thread local (ThreadLocal) field.
	 */
	private static ThreadLocal threadLocal = new ThreadLocal();

	/**
	 * Define the default resource bundle (ResourceBundle) field.
	 */
	public static ResourceBundle DefaultResourceBundle = Nls.getResourceBundle(TestAgentResourceBundle.class);

	/**
	 * Define the test class (Class) field.
	 */
	private Class testClass;

	/**
	 * Define the notification service (NotificationService) field.
	 */
	private NotificationService notificationService;

	/**
	 * Constructs an instance of this class.
	 */
	public TestAgent() {
		super();
		setConfigurationInformation(EscConfiguration.getConfiguration().getProperties());
		setTestClass(loadTestClass());
	}

	/**
	 * Gets the test context value.
	 * @return The test context (<code>TestContext</code>) value.
	 * @see #setTestContext(TestContext)
	 */
	public static TestContext getTestContext() {
		return (TestContext) threadLocal.get();
	}

	/**
	 * Main with the specified arguments parameter.
	 * @param arguments
	 *		The arguments (<code>String[]</code>) parameter.
	 */
	public static void main(final String[] arguments) {
		if (arguments.length == 1) {
			try {
				TestAgent testAgent = new TestAgent();
				testAgent.setTestClass(Class.forName(arguments[0]));
				Notification notification = new Notification();
				testAgent.setNotificationService(notification);
				notification.start();
				testAgent.runTest(new ConsoleSynchronization(), null);
				notification.stop();
				System.exit(0);
			} catch (final Exception e) {
				e.printStackTrace();
			}
		} else {
			System.err.println(TestAgent.DefaultResourceBundle.getString(Integer.toString(TestAgent.USAGE)));
		}
	}

	/**
	 * Run main with the specified arguments parameter.
	 * @param arguments
	 *		The arguments (<code>String[]</code>) parameter.
	 */
	protected static void runMain(final String[] arguments) {
		Class runnerClass;
		try {
			runnerClass = Class.forName(TEST_MANAGER_CLASS_NAME);
			TestAgentRunService runner = (TestAgentRunService) runnerClass.newInstance();
			runner.runMain(arguments);
		} catch (final Exception e) {
			e.printStackTrace();
		}
	}

	/**
	 * Sets the test context value.
	 * @param context
	 *		The context (<code>TestContext</code>) parameter.
	 * @see #getTestContext()
	 */
	public static void setTestContext(final TestContext context) {
		threadLocal.set(context);
	}

	/**
	 * Create test context with the specified sync service and properties parameters and return the TestContext result.
	 * @param syncService
	 *		The sync service (<code>TestSynchronizationService</code>) parameter.
	 * @param properties
	 *		The properties (<code>Dictionary</code>) parameter.
	 * @return Results of the create test context (<code>TestContext</code>) value.
	 * @see #getTestContext()
	 * @see #setTestContext(TestContext)
	 */
	private TestContext createTestContext(final TestSynchronizationService syncService, final Dictionary properties) {
		return new ConfiguredTestContext(this, syncService, properties);
	}

	/**
	 * Gets the description (String) value.
	 * @return The description (<code>String</code>) value.
	 */
	public String getDescription() {
		return this.testClass.toString();
	}

	/**
	 * Gets the ID (String) value.
	 * @return The id (<code>String</code>) value.
	 */
	public String getId() {
		return getString(EscObject.ID_KEY, ""); //$NON-NLS-1$
	}

	/**
	 * Gets the notification service value.
	 * @return The notification service (<code>NotificationService</code>) value.
	 * @see #setNotificationService(NotificationService)
	 */
	public NotificationService getNotificationService() {
		return this.notificationService;
	}

	/**
	 * Gets the suite (Test) value.
	 * @return The suite (<code>Test</code>) value.
	 */
	private Test getSuite() {
		Method suiteMethod = null;
		try {
			suiteMethod = getTestClass().getMethod(BaseTestRunner.SUITE_METHODNAME, new Class[0]);
		} catch (Exception e) {
			// try to extract a test suite automatically
			return new TestSuite(getTestClass());
		}
		if (!Modifier.isStatic(suiteMethod.getModifiers())) {
			log(LOG_ERROR, TestAgent.DefaultResourceBundle.getString(Integer.toString(TestAgent.METHODE_SUITE_MUST_BE_STATIC)));
			return null;
		}
		Test test = null;
		try {
			test = (Test) suiteMethod.invoke(null, new Class[0]); // static
			// method
			if (test == null) {
				return test;
			}
		} catch (InvocationTargetException e) {
			log(LOG_ERROR, TestAgent.DefaultResourceBundle.getString(Integer.toString(TestAgent.FAILED_TO_INVOLE_SUITE)), e.getTargetException());
			return null;
		} catch (IllegalAccessException e) {
			log(LOG_ERROR, TestAgent.DefaultResourceBundle.getString(Integer.toString(TestAgent.FAILED_TO_INVOLE_SUITE)), e);
			return null;
		}
		return test;
	}

	/**
	 * Gets the test class (Class) value.
	 * @return The test class (<code>Class</code>) value.
	 * @see #loadTestClass()
	 * @see #setTestClass(Class)
	 */
	public Class getTestClass() {
		return this.testClass;
	}

	/**
	 * Load test class and return the Class result.
	 * @return Results of the load test class (<code>Class</code>) value.
	 * @see #getTestClass()
	 * @see #setTestClass(Class)
	 */
	protected Class loadTestClass() {
		return null;
	}

	/**
	 * Run test with the specified sync service and properties parameters and return the TestResult result.
	 * @param syncService
	 *		The sync service (<code>TestSynchronizationService</code>) parameter.
	 * @param properties
	 *		The properties (<code>Dictionary</code>) parameter.
	 * @return Results of the run test (<code>TestResult</code>) value.
	 * @see #runTest(TestSynchronizationService,Dictionary,TestResult)
	 */
	public TestResult runTest(final TestSynchronizationService syncService, final Dictionary properties) {
		TestResult result = null;
		setTestContext(createTestContext(syncService, properties));
		Test suite = getSuite();
		if (suite != null) {
			try {
				setUp();
				result = TestRunner.run(suite);
				tearDown();
			} catch (final Exception e) {
				log(LOG_ERROR, TestAgent.DefaultResourceBundle.getString(Integer.toString(TestAgent.FAILED_TO_RUN_TEST)), e);
			}
		}
		setTestContext(null);
		return result;
	}

	/**
	 * Run test with the specified sync service, properties and result parameters.
	 * @param syncService
	 *		The sync service (<code>TestSynchronizationService</code>) parameter.
	 * @param properties
	 *		The properties (<code>Dictionary</code>) parameter.
	 * @param result
	 *		The result (<code>TestResult</code>) parameter.
	 * @see #runTest(TestSynchronizationService,Dictionary)
	 */
	public void runTest(final TestSynchronizationService syncService, final Dictionary properties, final TestResult result) {
		setTestContext(createTestContext(syncService, properties));
		Test suite = getSuite();
		if (suite != null) {
			try {
				setUp();
				suite.run(result);
				tearDown();
			} catch (final Exception e) {
				log(LOG_ERROR, TestAgent.DefaultResourceBundle.getString(Integer.toString(TestAgent.FAILED_TO_RUN_TEST)), e);
			}
		}
		setTestContext(null);
	}

	/**
	 * Sets the notification service value.
	 * @param notificationService
	 *		The notification service (<code>NotificationService</code>) parameter.
	 * @see #getNotificationService()
	 */
	public void setNotificationService(final NotificationService notificationService) {
		this.notificationService = notificationService;
	}

	/**
	 * Sets the test class value.
	 * @param testClass
	 *		The test class (<code>Class</code>) parameter.
	 * @see #getTestClass()
	 * @see #loadTestClass()
	 */
	public void setTestClass(final Class testClass) {
		this.testClass = testClass;
	}

	/**
	 * Sets up before testing, for example by acquiring a device. This method is called before the test is executed.
	 * @throws Exception Exception.
	 */
	protected void setUp() throws Exception {
		/* do nothing */
	}

	/**
	 * Tears down after testing, for example by releasing a device. This method is called after the test is executed.
	 * @throws Exception Exception.
	 */
	protected void tearDown() throws Exception {
		/* do nothing */
	}
}
