package org.eclipse.soda.dk.testagent.bundle;

/*************************************************************************
 * Copyright (c) 2005, 2009 IBM.                                         *
 * All rights reserved. This program and the accompanying materials      *
 * are made available under the terms of the Eclipse Public License v1.0 *
 * which accompanies this distribution, and is available at              *
 * http://www.eclipse.org/legal/epl-v10.html                             *
 *                                                                       *
 * Contributors:                                                         *
 *     IBM - initial API and implementation                              *
 ************************************************************************/
import java.util.Hashtable;
import org.eclipse.soda.dk.core.ConfigurableObject;
import org.eclipse.soda.dk.notification.service.NotificationService;
import org.eclipse.soda.dk.testagent.TestAgent;
import org.eclipse.soda.dk.testagent.service.TestAgentService;
import org.eclipse.soda.sat.core.framework.BaseBundleActivator;
import org.osgi.framework.Bundle;
import org.osgi.framework.BundleActivator;

/**
 * The AgentBundle is the base bundle activator for all agent bundle activators.
 * @author IBM
 * @version 1.2.0
 * @since 1.0
 */
public abstract class TestAgentBundle extends BaseBundleActivator implements BundleActivator {
	/**
	 * Defines the class name.
	 */
	public static final String CLASS_NAME = "org.eclipse.soda.dk.testagent.bundle.TestAgentBundle"; //$NON-NLS-1$

	/**
	 * Defines the service name(s) to be registered by the testAdapter bundle.
	 * <table>
	 * <tr>
	 * <td>org.eclipse.soda.dk.adapter.service.AdapterService</td>
	 * </tr>
	 * </table>
	 */
	public static final String[] EXPORTED_SERVICE_NAMES = new String[] { TestAgentService.SERVICE_NAME };

	/**
	 * Defines the imported service name(s) to be registered by the testAdapter
	 * bundle. <table>
	 * <tr>
	 * <td>org.eclipse.soda.dk.test.adapter.service.testAdapterService</td>
	 * </tr>
	 * <tr>
	 * <td>org.eclipse.soda.dk.notification.service.NotificationService</td>
	 * </tr>
	 * </table>
	 */
	public static final String[] IMPORTED_SERVICE_NAMES = BaseBundleActivator.NO_SERVICES;

	/**
	 * Define the optional services (String[]) constant.
	 */
	public static final String[] OPTIONAL_SERVICES = new String[] { NotificationService.SERVICE_NAME };

	/**
	 * Define the test agent (TestAgentService) field.
	 */
	private TestAgent testAgent;

	/**
	 * Create service and return the Object result.
	 */
	public void activate() {
		if (this.testAgent == null) {
			setService(createService());
			if (this.testAgent != null) {
				final Hashtable properties = createProperties();
				final String[] names = getExportedServiceNames();
				addExportedServices(names, this.testAgent, properties);
			}
		}
		final NotificationService notificationService = (NotificationService) getOptionalImportedService(NotificationService.SERVICE_NAME);
		if (notificationService != null) {
			this.testAgent.setNotificationService(notificationService);
		}
	}

	/**
	 * Gets the properties to be registered with the service(s).
	 * @return Hashtable of properties.
	 * <table width=100%>
	 * <tr>
	 * <td width=50%>org.eclipse.soda.dk.connection.bundle.ServiceBundle.DEVICE_CLASS</td>
	 * <td width=50%>testAdapter</td>
	 * </tr>
	 * <tr>
	 * <td width=50%>org.osgi.framework.Constants.SERVICE_PID</td>
	 * <td width=50%>testAdapterService.SERVICE_NAME</td>
	 * </tr>
	 * <tr>
	 * <td width=50%>org.osgi.framework.Constants.SERVICE_DESCRIPTION</td>
	 * <td width=50%>test Adapter</td>
	 * </tr>
	 * <tr>
	 * <td width=50%>org.osgi.framework.Constants.SERVICE_VENDOR</td>
	 * <td width=50%>Eclipse</td>
	 * </tr>
	 * <tr>
	 * <td width=50%>org.osgi.service.device.Constants.DRIVER_ID</td>
	 * <td width=50%>testAdapter</td>
	 * </tr>
	 * <tr>
	 * <td width=50%>org.osgi.framework.Constants.SERVICE_RANKING</td>
	 * <td width=50%>6</td>
	 * </tr>
	 * <tr>
	 * <td width=50%>org.osgi.service.device.Constants.DEVICE_CATEGORY</td>
	 * <td width=50%>DeviceService</td>
	 * </tr>
	 * <tr>
	 * <td width=50%>org.eclipse.soda.dk.connection.bundle.ServiceBundle.DEVICE_REVISION</td>
	 * <td width=50%>1.0.0</td>
	 * </tr>
	 * </table>
	 */
	public Hashtable createProperties() {
		final Bundle bundle = getBundle();
		final Hashtable properties = ConfigurableObject.createPropertiesFromHeaders(bundle.getHeaders());
		return properties;
	}

	/**
	 * Create service and return the TestAgent result.
	 * @return Results of the create service (<code>Object</code>) value.
	 * The device (<code>DeviceService</code>) parameter.
	 * @see #getService()
	 * @see #handleAcquiredOptionalImportedService(String,Object)
	 * @see #handleReleasedOptionalImportedService(String,Object)
	 * @see #setService(TestAgent)
	 */
	public abstract TestAgent createService();

	/**
	 * Deactivate.
	 */
	protected void deactivate() {
		final TestAgent agent = getService();
		if (agent != null) {
			setService(null);
		}
	}

	/**
	 * Return the exported service name(s) to be registered by the testAdapter
	 * bundle.
	 * @return String[] The device's service names.
	 */
	public String[] getExportedServiceNames() {
		return EXPORTED_SERVICE_NAMES;
	}

	/**
	 * Return the imported service name(s) to be registered by the testAdapter
	 * bundle.
	 * @return String[] The imported service names.
	 * @see #getOptionalImportedServiceNames()
	 */
	public String[] getImportedServiceNames() {
		return IMPORTED_SERVICE_NAMES;
	}

	/**
	 * Gets the optional imported service names (String[]) value.
	 * @return The optional imported service names (<code>String[]</code>)
	 * value.
	 */
	protected String[] getOptionalImportedServiceNames() {
		return OPTIONAL_SERVICES;
	}

	/**
	 * Return the agent.
	 * @return Results of the get agent (<code>AgentService</code>) value.
	 * @see #createService()
	 * @see #handleAcquiredOptionalImportedService(String,Object)
	 * @see #handleReleasedOptionalImportedService(String,Object)
	 * @see #setService(TestAgent)
	 */
	public TestAgent getService() {
		return this.testAgent;
	}

	/**
	 * Handle acquired optional imported service with the specified service name and service parameters.
	 * @param serviceName	The service name (<code>String</code>) parameter.
	 * @param service	The service (<code>Object</code>) parameter.
	 */
	protected void handleAcquiredOptionalImportedService(final String serviceName, final Object service) {
		if (this.testAgent != null) {
			if (service instanceof NotificationService) {
				this.testAgent.setNotificationService((NotificationService) service);
			}
		}
	}

	/**
	 * Handle released optional imported service with the specified service name and service parameters.
	 * @param serviceName	The service name (<code>String</code>) parameter.
	 * @param service	The service (<code>Object</code>) parameter.
	 */
	protected void handleReleasedOptionalImportedService(final String serviceName, final Object service) {
		if (this.testAgent != null) {
			if (service instanceof NotificationService) {
				this.testAgent.setNotificationService((NotificationService) null);
			}
		}
	}

	/**
	 * Sets the service value.
	 * @param agent	The agent (<code>TestAgent</code>) parameter.
	 * @see #createService()
	 * @see #getService()
	 * @see #handleAcquiredOptionalImportedService(String,Object)
	 * @see #handleReleasedOptionalImportedService(String,Object)
	 */
	public void setService(final TestAgent agent) {
		this.testAgent = agent;
	}
}
