package org.eclipse.soda.dk.testagent.factory;

/*************************************************************************
 * Copyright (c) 2006, 2009 IBM.                                         *
 * All rights reserved. This program and the accompanying materials      *
 * are made available under the terms of the Eclipse Public License v1.0 *
 * which accompanies this distribution, and is available at              *
 * http://www.eclipse.org/legal/epl-v10.html                             *
 *                                                                       *
 * Contributors:                                                         *
 *     IBM - initial API and implementation                              *
 ************************************************************************/
import java.util.Dictionary;
import java.util.Enumeration;
import java.util.Hashtable;
import org.eclipse.soda.dk.core.ConfigurableObject;
import org.eclipse.soda.dk.core.EscObject;
import org.eclipse.soda.dk.notification.service.NotificationService;
import org.eclipse.soda.dk.testagent.TestAgent;
import org.eclipse.soda.dk.testagent.service.TestAgentService;
import org.eclipse.soda.sat.core.framework.BaseBundleActivator;
import org.eclipse.soda.sat.core.framework.ManagedServiceFactoryBundleActivator;
import org.eclipse.soda.sat.core.framework.interfaces.IBundleActivationManager;
import org.eclipse.soda.sat.core.framework.interfaces.IManagedServiceFactoryAdvisor;
import org.osgi.framework.Bundle;
import org.osgi.service.cm.ConfigurationException;
import org.osgi.service.cm.ManagedServiceFactory;
import org.osgi.service.metatype.MetaTypeInformation;
import org.osgi.service.metatype.MetaTypeService;

/**
 * @author IBM
 * @version 1.2.0
 * @since 1.0
 */
public abstract class TestAgentFactory extends ManagedServiceFactoryBundleActivator implements ManagedServiceFactory, IManagedServiceFactoryAdvisor {
	/**
	 * Defines the class name.
	 */
	public static final String CLASS_NAME = "org.eclipse.soda.dk.testagent.factory.TestAgentFactory"; //$NON-NLS-1$

	/**
	 * Define the id key (String) constant.
	 */
	public static final String ID_KEY = EscObject.ID_KEY;

	/**
	 * Define the id key filter (String) constant.
	 */
	public static final String ID_KEY_FILTER = "(id="; //$NON-NLS-1$

	/**
	 * Define the metatype service (String) constant.
	 */
	public static final String METATYPE_SERVICE = MetaTypeService.class.getName();

	/**
	 * Defines the service name(s) to be registered by the testAdapter bundle.
	 * <table>
	 * <tr>
	 * <td>org.eclipse.soda.dk.adapter.service.AdapterService</td>
	 * </tr>
	 * </table>
	 */
	public static final String[] EXPORTED_SERVICE_NAMES = new String[] { TestAgentService.SERVICE_NAME };

	/**
	 * Defines the imported service name(s) to be registered by the testAdapter
	 * bundle. <table>
	 * <tr>
	 * <td>org.eclipse.soda.dk.test.adapter.service.testAdapterService</td>
	 * </tr>
	 * <tr>
	 * <td>org.eclipse.soda.dk.notification.service.NotificationService</td>
	 * </tr>
	 * </table>
	 */
	public static final String[] IMPORTED_SERVICE_NAMES = BaseBundleActivator.NO_SERVICES;

	/**
	 * Define the optional services (String[]) constant.
	 */
	public static final String[] OPTIONAL_SERVICES = new String[] { METATYPE_SERVICE, NotificationService.SERVICE_NAME };

	/**
	 * Create with the specified pid, properties and manager parameters and
	 * return the Object result.
	 * The pid (<code>String</code>) parameter.
	 * The properties (<code>Dictionary</code>) parameter.
	 * The manager (<code>IBundleActivationManager</code>)
	 * parameter.
	 * @param pid	The pid (<code>String</code>) parameter.
	 * @param properties	The properties (<code>Dictionary</code>) parameter.
	 * @param manager	The manager (<code>IBundleActivationManager</code>) parameter.
	 * @return Results of the create (<code>Object</code>) value.
	 */
	public Object create(final String pid, final Dictionary properties, final IBundleActivationManager manager) {
		final TestAgent testAgent = createService();
		if (testAgent != null) {
			testAgent.setConfigurationInformation(properties);
			final NotificationService notificationService = (NotificationService) getOptionalImportedService(NotificationService.SERVICE_NAME);
			if (notificationService != null) {
				testAgent.setNotificationService(notificationService);
			}
			final Hashtable serviceProperties = createProperties();
			updateProperties(serviceProperties, properties);
			manager.addExportedServices(getExportedServiceNames(pid, new Hashtable(), properties, manager), testAgent, serviceProperties);
		}
		return testAgent;
	}

	/**
	 * @see org.eclipse.soda.sat.core.framework.ManagedServiceFactoryBundleActivator#createAdvisor()
	 * @return Results of the create advisor (<code>IManagedServiceFactoryAdvisor</code>)
	 * value.
	 */
	protected IManagedServiceFactoryAdvisor createAdvisor() {
		return this;
	}

	/**
	 * Create imported service filters with the specified pid, old properties,
	 * properties and manager parameters.
	 * The pid (<code>String</code>) parameter.
	 * The old properties (<code>Dictionary</code>) parameter.
	 * The properties (<code>Dictionary</code>) parameter.
	 * The manager (<code>IBundleActivationManager</code>)
	 * parameter.
	 * @param pid	The pid (<code>String</code>) parameter.
	 * @param oldProperties	The old properties (<code>Dictionary</code>) parameter.
	 * @param properties	The properties (<code>Dictionary</code>) parameter.
	 * @param manager	The manager (<code>IBundleActivationManager</code>) parameter.
	 */
	public void createImportedServiceFilters(final String pid, final Dictionary oldProperties, final Dictionary properties, final IBundleActivationManager manager) {
		/* do nothing */
	}

	/**
	 * Gets the properties to be registered with the service(s).
	 * @return Hashtable of properties.
	 * <table width=100%>
	 * <tr>
	 * <td width=50%>org.eclipse.soda.dk.connection.bundle.ServiceBundle.DEVICE_CLASS</td>
	 * <td width=50%>testAdapter</td>
	 * </tr>
	 * <tr>
	 * <td width=50%>org.osgi.framework.Constants.SERVICE_PID</td>
	 * <td width=50%>testAdapterService.SERVICE_NAME</td>
	 * </tr>
	 * <tr>
	 * <td width=50%>org.osgi.framework.Constants.SERVICE_DESCRIPTION</td>
	 * <td width=50%>test Adapter</td>
	 * </tr>
	 * <tr>
	 * <td width=50%>org.osgi.framework.Constants.SERVICE_VENDOR</td>
	 * <td width=50%>Eclipse</td>
	 * </tr>
	 * <tr>
	 * <td width=50%>org.osgi.service.device.Constants.DRIVER_ID</td>
	 * <td width=50%>testAdapter</td>
	 * </tr>
	 * <tr>
	 * <td width=50%>org.osgi.framework.Constants.SERVICE_RANKING</td>
	 * <td width=50%>6</td>
	 * </tr>
	 * <tr>
	 * <td width=50%>org.osgi.service.device.Constants.DEVICE_CATEGORY</td>
	 * <td width=50%>DeviceService</td>
	 * </tr>
	 * <tr>
	 * <td width=50%>org.eclipse.soda.dk.connection.bundle.ServiceBundle.DEVICE_REVISION</td>
	 * <td width=50%>1.0.0</td>
	 * </tr>
	 * </table>
	 * @see #updateProperties(Dictionary,Dictionary)
	 */
	public Hashtable createProperties() {
		final Bundle bundle = getBundle();
		final Hashtable properties = ConfigurableObject.createPropertiesFromHeaders(bundle.getHeaders());
		return properties;
	}

	/**
	 * Create service and return the TestAgent result.
	 * @return Results of the create service (<code>Object</code>) value.
	 * @see #handleAcquiredOptionalImportedService(String,Object)
	 * @see #handleAcquiredOptionalImportedService(String,Object,String,Object,IBundleActivationManager)
	 * @see #handleReleasedOptionalImportedService(String,Object)
	 * @see #handleReleasedOptionalImportedService(String,Object,String,Object,IBundleActivationManager)
	 */
	public abstract TestAgent createService();

	/**
	 * Destroy with the specified pid, object, properties and manager
	 * parameters.
	 * The pid (<code>String</code>) parameter.
	 * The object (<code>Object</code>) parameter.
	 * The properties (<code>Dictionary</code>) parameter.
	 * The manager (<code>IBundleActivationManager</code>)
	 * parameter.
	 * @param pid	The pid (<code>String</code>) parameter.
	 * @param object	The object (<code>Object</code>) parameter.
	 * @param properties	The properties (<code>Dictionary</code>) parameter.
	 * @param manager	The manager (<code>IBundleActivationManager</code>) parameter.
	 */
	public void destroy(final String pid, final Object object, final Dictionary properties, final IBundleActivationManager manager) {
		/* do nothing */
	}

	/**
	 * Return the exported service name(s) to be registered by the testAdapter
	 * bundle.
	 * @param pid	The pid (<code>String</code>) parameter.
	 * @param oldProperties	The old properties (<code>Dictionary</code>) parameter.
	 * @param properties	The properties (<code>Dictionary</code>) parameter.
	 * @param manager	The manager (<code>IBundleActivationManager</code>) parameter.
	 * @return String[] The device's service names.
	 */
	public String[] getExportedServiceNames(final String pid, final Dictionary oldProperties, final Dictionary properties, final IBundleActivationManager manager) {
		return EXPORTED_SERVICE_NAMES;
	}

	/**
	 * Return the imported service name(s) to be registered by the testAdapter
	 * bundle.
	 * @param pid	The pid (<code>String</code>) parameter.
	 * @param oldProperties	The old properties (<code>Dictionary</code>) parameter.
	 * @param properties	The properties (<code>Dictionary</code>) parameter.
	 * @param manager	The manager (<code>IBundleActivationManager</code>) parameter.
	 * @return String[] The imported service names.
	 * @see #getOptionalImportedServiceNames()
	 * @see #getOptionalImportedServiceNames(String,Dictionary,Dictionary,IBundleActivationManager)
	 */
	public String[] getImportedServiceNames(final String pid, final Dictionary oldProperties, final Dictionary properties, final IBundleActivationManager manager) {
		return IMPORTED_SERVICE_NAMES;
	}

	/**
	 * Gets the optional imported service names (String[]) value.
	 * @return	The optional imported service names (<code>String[]</code>) value.
	 * @see #getOptionalImportedServiceNames(String,Dictionary,Dictionary,IBundleActivationManager)
	 */
	protected String[] getOptionalImportedServiceNames() {
		return OPTIONAL_SERVICES;
	}

	/**
	 * Get optional imported service names with the specified arg0, arg1, arg2
	 * and arg3 parameters and return the String[] result.
	 * The arg0 (<code>String</code>) parameter.
	 * The arg1 (<code>Dictionary</code>) parameter.
	 * The arg2 (<code>Dictionary</code>) parameter.
	 * The arg3 (<code>IBundleActivationManager</code>)
	 * parameter.
	 * @param arg0	The arg0 (<code>String</code>) parameter.
	 * @param arg1	The arg1 (<code>Dictionary</code>) parameter.
	 * @param arg2	The arg2 (<code>Dictionary</code>) parameter.
	 * @param arg3	The arg3 (<code>IBundleActivationManager</code>) parameter.
	 * @return Results of the get optional imported service names (<code>String[]</code>)
	 * value.
	 * @see #getOptionalImportedServiceNames()
	 */
	public String[] getOptionalImportedServiceNames(final String arg0, final Dictionary arg1, final Dictionary arg2, final IBundleActivationManager arg3) {
		return OPTIONAL_SERVICES;
	}

	/**
	 * Handle acquired optional imported service with the specified arg0 and arg1 parameters.
	 * @param arg0	The arg0 (<code>String</code>) parameter.
	 * @param arg1	The arg1 (<code>Object</code>) parameter.
	 * @see #handleAcquiredOptionalImportedService(String,Object,String,Object,IBundleActivationManager)
	 */
	protected void handleAcquiredOptionalImportedService(final String arg0, final Object arg1) {
		/* do nothing */
	}

	/**
	 * Handle acquired optional imported service with the specified pid, object,
	 * imported service name, imported service and manager parameters.
	 * The pid (<code>String</code>) parameter.
	 * The object (<code>Object</code>) parameter.
	 * The imported service name (<code>String</code>) parameter.
	 * The imported service (<code>Object</code>) parameter.
	 * The manager (<code>IBundleActivationManager</code>)
	 * parameter.
	 * @param pid	The pid (<code>String</code>) parameter.
	 * @param object	The object (<code>Object</code>) parameter.
	 * @param importedServiceName	The imported service name (<code>String</code>) parameter.
	 * @param importedService	The imported service (<code>Object</code>) parameter.
	 * @param manager	The manager (<code>IBundleActivationManager</code>) parameter.
	 * @see #handleAcquiredOptionalImportedService(String,Object)
	 */
	public void handleAcquiredOptionalImportedService(final String pid, final Object object, final String importedServiceName, final Object importedService, final IBundleActivationManager manager) {
		if (object instanceof TestAgent) {
			final TestAgent testAgent = (TestAgent) object;
			if (importedService instanceof NotificationService) {
				testAgent.setNotificationService((NotificationService) importedService);
			}
		}
	}

	/**
	 * Handle released optional imported service with the specified arg0 and arg1 parameters.
	 * @param arg0	The arg0 (<code>String</code>) parameter.
	 * @param arg1	The arg1 (<code>Object</code>) parameter.
	 * @see #handleReleasedOptionalImportedService(String,Object,String,Object,IBundleActivationManager)
	 */
	protected void handleReleasedOptionalImportedService(final String arg0, final Object arg1) {
		/* do nothing */
	}

	/**
	 * Handle released optional imported service with the specified pid, object,
	 * imported service name, imported service and manager parameters.
	 * The pid (<code>String</code>) parameter.
	 * The object (<code>Object</code>) parameter.
	 * The imported service name (<code>String</code>) parameter.
	 * The imported service (<code>Object</code>) parameter.
	 * The manager (<code>IBundleActivationManager</code>)
	 * parameter.
	 * @param pid	The pid (<code>String</code>) parameter.
	 * @param object	The object (<code>Object</code>) parameter.
	 * @param importedServiceName	The imported service name (<code>String</code>) parameter.
	 * @param importedService	The imported service (<code>Object</code>) parameter.
	 * @param manager	The manager (<code>IBundleActivationManager</code>) parameter.
	 * @see #handleReleasedOptionalImportedService(String,Object)
	 */
	public void handleReleasedOptionalImportedService(final String pid, final Object object, final String importedServiceName, final Object importedService, final IBundleActivationManager manager) {
		if (object instanceof TestAgent) {
			final TestAgent testAgent = (TestAgent) object;
			if (importedService instanceof NotificationService) {
				testAgent.setNotificationService(null);
			}
		}
	}

	/**
	 * Update with the specified pid, object, old properties, properties and
	 * manager parameters and return the Object result.
	 * The pid (<code>String</code>) parameter.
	 * The object (<code>Object</code>) parameter.
	 * The old properties (<code>Dictionary</code>) parameter.
	 * The properties (<code>Dictionary</code>) parameter.
	 * The manager (<code>IBundleActivationManager</code>)
	 * parameter.
	 * @param pid	The pid (<code>String</code>) parameter.
	 * @param object	The object (<code>Object</code>) parameter.
	 * @param oldProperties	The old properties (<code>Dictionary</code>) parameter.
	 * @param properties	The properties (<code>Dictionary</code>) parameter.
	 * @param manager	The manager (<code>IBundleActivationManager</code>) parameter.
	 * @return Results of the update (<code>Object</code>) value.
	 */
	public Object update(final String pid, final Object object, final Dictionary oldProperties, final Dictionary properties, final IBundleActivationManager manager) {
		final TestAgent testAgent = (TestAgent) object;
		testAgent.setConfigurationInformation(properties);
		final Hashtable serviceProperties = createProperties();
		updateProperties(serviceProperties, properties);
		manager.setExportedServiceProperties(TestAgentService.SERVICE_NAME, serviceProperties);
		return testAgent;
	}

	/**
	 * Update properties with the specified table and from parameters.
	 * @param table	The table (<code>Dictionary</code>) parameter.
	 * @param from	The from (<code>Dictionary</code>) parameter.
	 * @see #createProperties()
	 */
	public void updateProperties(final Dictionary table, final Dictionary from) {
		final Enumeration enumeration = from.keys();
		while (enumeration.hasMoreElements()) {
			final Object key = enumeration.nextElement();
			final Object value = from.get(key);
			table.put(key, value);
		}
	}

	/**
	 * Validate configuration with the specified pid and properties parameters.
	 * @param pid	The pid (<code>String</code>) parameter.
	 * @param properties	The properties (<code>Dictionary</code>) parameter.
	 * @throws ConfigurationException Configuration Exception.
	 */
	public void validateConfiguration(final String pid, final Dictionary properties) throws ConfigurationException {
		final Object service = getOptionalImportedService(METATYPE_SERVICE);
		if (service instanceof MetaTypeService) {
			final MetaTypeService metatypeService = (MetaTypeService) service;
			final MetaTypeInformation info = metatypeService.getMetaTypeInformation(getBundle());
			ConfigurableObject.validateConfiguration(pid, properties, info);
		}
	}
}
