package org.eclipse.soda.dk.testagent.framework;

/*************************************************************************
 * Copyright (c) 2006, 2009 IBM.                                         *
 * All rights reserved. This program and the accompanying materials      *
 * are made available under the terms of the Eclipse Public License v1.0 *
 * which accompanies this distribution, and is available at              *
 * http://www.eclipse.org/legal/epl-v10.html                             *
 *                                                                       *
 * Contributors:                                                         *
 *     IBM - initial API and implementation                              *
 ************************************************************************/
import java.io.IOException;
import java.util.Dictionary;

import junit.framework.Assert;
import junit.framework.TestCase;

import org.eclipse.soda.dk.core.service.ConfigurationService;
import org.eclipse.soda.dk.testagent.TestAgent;
import org.eclipse.soda.dk.testagent.service.TestSynchronizationService;

/**
 * @author IBM
 * @version 1.2.0
 */
public abstract class ConfiguredTestCase extends TestCase implements ConfigurationService {
	/**
	 * Asserts that two booleans are equal. This method is re-implemented to avoid a runtime error due to incompatibility of junit with OSGi/Minimum.
	 * 
	 * @see Assert
	 * @param expected
	 *            The expected (<code>boolean</code>) parameter.
	 * @param actual
	 *            The actual (<code>boolean</code>) parameter.
	 * @see #assertEquals(double,double,double)
	 * @see #assertEquals(String,boolean,boolean)
	 * @see #assertEquals(String,double,double,double)
	 */
	public static void assertEquals(final boolean expected, final boolean actual) {
		assertEquals(null, expected, actual);
	}

	/**
	 * Asserts that two doubles are equal concerning a delta. If the expected value is infinity then the delta value is ignored. This method is re-implemented to avoid a runtime error due to incompatibility of junit with OSGi/Minimum.
	 * 
	 * @see Assert
	 * @param expected
	 *            The expected (<code>double</code>) parameter.
	 * @param actual
	 *            The actual (<code>double</code>) parameter.
	 * @param delta
	 *            The delta (<code>double</code>) parameter.
	 * @see #assertEquals(boolean,boolean)
	 * @see #assertEquals(String,boolean,boolean)
	 * @see #assertEquals(String,double,double,double)
	 */
	public static void assertEquals(final double expected, final double actual, final double delta) {
		assertEquals(null, expected, actual, delta);
	}

	/**
	 * Asserts that two booleans are equal. If they are not an AssertionFailedError is thrown with the given message. This method is re-implemented to avoid a runtime error due to incompatibility of junit with OSGi/Minimum.
	 * 
	 * @see Assert
	 * @param message
	 *            The message (<code>String</code>) parameter.
	 * @param expected
	 *            The expected (<code>boolean</code>) parameter.
	 * @param actual
	 *            The actual (<code>boolean</code>) parameter.
	 * @see #assertEquals(boolean,boolean)
	 * @see #assertEquals(double,double,double)
	 * @see #assertEquals(String,double,double,double)
	 */
	public static void assertEquals(final String message, final boolean expected, final boolean actual) {
		assertEquals(message, expected ? Boolean.TRUE : Boolean.FALSE, actual ? Boolean.TRUE : Boolean.FALSE);
	}

	/**
	 * Asserts that two doubles are equal concerning a delta. If they are not an AssertionFailedError is thrown with the given message. If the expected value is infinity then the delta value is ignored. This method is re-implemented to avoid a runtime
	 * error due to incompatibility of junit with OSGi/Minimum.
	 * 
	 * @see Assert
	 * @param message
	 *            The message (<code>String</code>) parameter.
	 * @param expected
	 *            The expected (<code>double</code>) parameter.
	 * @param actual
	 *            The actual (<code>double</code>) parameter.
	 * @param delta
	 *            The delta (<code>double</code>) parameter.
	 * @see #assertEquals(boolean,boolean)
	 * @see #assertEquals(double,double,double)
	 * @see #assertEquals(String,boolean,boolean)
	 */
	public static void assertEquals(final String message, final double expected, final double actual, final double delta) {
		if (new Double(expected).compareTo(new Double(actual)) == 0) {
			return;
		}
		if (!(Math.abs(expected - actual) <= delta)) {
			failNotEquals(message, new Double(expected), new Double(actual));
		}
	}

	/**
	 * Define the context (TestContext) field.
	 */
	private TestContext context = TestAgent.getTestContext();

	/**
	 * No-arg constructor to enable serialization. This method is not intended to be used by mere mortals without calling setName().
	 * 
	 * @see #ConfiguredTestCase(String)
	 */
	protected ConfiguredTestCase() {
		super();
	}

	/**
	 * Constructs a test case with the given name.
	 * 
	 * @param name
	 *            The name (<code>String</code>) parameter.
	 * @see #ConfiguredTestCase()
	 */
	protected ConfiguredTestCase(final String name) {
		super(name);
	}

	/**
	 * Get boolean with the specified key and default boolean parameters and return the boolean result.
	 * 
	 * @param key
	 *            The key (<code>String</code>) parameter.
	 * @param defaultBoolean
	 *            The default boolean (<code>boolean</code>) parameter.
	 * @return Results of the get boolean (<code>boolean</code>) value.
	 */
	public boolean getBoolean(final String key, final boolean defaultBoolean) {
		return getConfigurationService().getBoolean(key, defaultBoolean);
	}

	/**
	 * Get boolean array with the specified key and default boolean array parameters and return the boolean[] result.
	 * 
	 * @param key
	 *            The key (<code>String</code>) parameter.
	 * @param defaultBooleanArray
	 *            The default boolean array (<code>boolean[]</code>) parameter.
	 * @return Results of the get boolean array (<code>boolean[]</code>) value.
	 * @since 1.2
	 */
	public boolean[] getBooleanArray(final String key, final boolean[] defaultBooleanArray) {
		return getConfigurationService().getBooleanArray(key, defaultBooleanArray);
	}

	/**
	 * Get byte with the specified key and default byte parameters and return the byte result.
	 * 
	 * @param key
	 *            The key (<code>String</code>) parameter.
	 * @param defaultByte
	 *            The default byte (<code>byte</code>) parameter.
	 * @return Results of the get byte (<code>byte</code>) value.
	 * @since 1.2
	 */
	public byte getByte(final String key, final byte defaultByte) {
		return getConfigurationService().getByte(key, defaultByte);
	}

	/**
	 * Get byte array with the specified key and default bytes parameters and return the byte result.
	 * 
	 * @param key
	 *            The key (<code>String</code>) parameter.
	 * @param defaultBytes
	 *            The default bytes (<code>byte[]</code>) parameter.
	 * @return Results of the get byte array (<code>byte[]</code>) value.
	 * @since 1.2
	 */
	public byte[] getByteArray(final String key, final byte[] defaultBytes) {
		return getConfigurationService().getByteArray(key, defaultBytes);
	}

	/**
	 * Get char with the specified key and default char parameters and return the char result.
	 * 
	 * @param key
	 *            The key (<code>String</code>) parameter.
	 * @param defaultChar
	 *            The default char (<code>char</code>) parameter.
	 * @return Results of the get char (<code>char</code>) value.
	 * @since 1.2
	 */
	public char getChar(final String key, final char defaultChar) {
		return getConfigurationService().getChar(key, defaultChar);
	}

	/**
	 * Get char array with the specified key and default char array parameters and return the char[] result.
	 * 
	 * @param key
	 *            The key (<code>String</code>) parameter.
	 * @param defaultCharArray
	 *            The default char array (<code>char[]</code>) parameter.
	 * @return Results of the get char array (<code>char[]</code>) value.
	 * @since 1.2
	 */
	public char[] getCharArray(final String key, final char[] defaultCharArray) {
		return getConfigurationService().getCharArray(key, defaultCharArray);
	}

	/**
	 * Gets the configuration service value.
	 * 
	 * @return the configurationService
	 */
	public ConfigurationService getConfigurationService() {
		ConfigurationService configurationService = this.context.getConfigurationService();
		assertNotNull(TestAgent.DefaultResourceBundle.getString(Integer.toString(TestAgent.MISSING_CONFIGURATION)), configurationService);
		return configurationService;
	}

	/**
	 * Gets the context (TestContext) value.
	 * 
	 * @return the context
	 */
	public TestContext getContext() {
		return this.context;
	}

	/**
	 * Get double with the specified key and default float parameters and return the double result.
	 * 
	 * @param key
	 *            The key (<code>String</code>) parameter.
	 * @param defaultDouble
	 *            The default double (<code>double</code>) parameter.
	 * @return Results of the get double (<code>double</code>) value.
	 * @see #getDoubleArray(String,double[])
	 * @since 1.2
	 */
	public double getDouble(final String key, final double defaultDouble) {
		return getConfigurationService().getDouble(key, defaultDouble);
	}

	/**
	 * Get double with the specified key and default double array parameters and return the double[] result.
	 * 
	 * @param key
	 *            The key (<code>String</code>) parameter.
	 * @param defaultDoubleArray
	 *            The default double array (<code>double[]</code>) parameter.
	 * @return Results of the get double (<code>double[]</code>) value.
	 * @see #getDouble(String,double)
	 * @since 1.2
	 * @deprecated Use {@link #getDoubleArray(String,double[])} instead
	 */
	public double[] getDouble(final String key, final double[] defaultDoubleArray) {
		return getDoubleArray(key, defaultDoubleArray);
	}

	/**
	 * Get double with the specified key and default double array parameters and return the double[] result.
	 * 
	 * @param key
	 *            The key (<code>String</code>) parameter.
	 * @param defaultDoubleArray
	 *            The default double array (<code>double[]</code>) parameter.
	 * @return Results of the get double (<code>double[]</code>) value.
	 * @see #getDouble(String,double)
	 * @since 1.2
	 */
	public double[] getDoubleArray(final String key, final double[] defaultDoubleArray) {
		return getConfigurationService().getDoubleArray(key, defaultDoubleArray);
	}

	/**
	 * Get float with the specified key and default float parameters and return the float result.
	 * 
	 * @param key
	 *            The key (<code>String</code>) parameter.
	 * @param defaultFloat
	 *            The default float (<code>float</code>) parameter.
	 * @return Results of the get float (<code>float</code>) value.
	 */
	public float getFloat(final String key, final float defaultFloat) {
		return getConfigurationService().getFloat(key, defaultFloat);
	}

	/**
	 * Get float array with the specified key and default float array parameters and return the float[] result.
	 * 
	 * @param key
	 *            The key (<code>String</code>) parameter.
	 * @param defaultFloatArray
	 *            The default float array (<code>float[]</code>) parameter.
	 * @return Results of the get float array (<code>float[]</code>) value.
	 * @since 1.2
	 */
	public float[] getFloatArray(final String key, final float[] defaultFloatArray) {
		return getConfigurationService().getFloatArray(key, defaultFloatArray);
	}

	/**
	 * Get int with the specified key and default int parameters and return the int result.
	 * 
	 * @param key
	 *            The key (<code>String</code>) parameter.
	 * @param defaultInt
	 *            The default int (<code>int</code>) parameter.
	 * @return Results of the get int (<code>int</code>) value.
	 */
	public int getInt(final String key, final int defaultInt) {
		return getConfigurationService().getInt(key, defaultInt);
	}

	/**
	 * Get int array with the specified key and default int array parameters and return the int[] result.
	 * 
	 * @param key
	 *            The key (<code>String</code>) parameter.
	 * @param defaultIntArray
	 *            The default int array (<code>int[]</code>) parameter.
	 * @return Results of the get int array (<code>int[]</code>) value.
	 * @since 1.2
	 */
	public int[] getIntArray(final String key, final int[] defaultIntArray) {
		return getConfigurationService().getIntArray(key, defaultIntArray);
	}

	/**
	 * Get long with the specified key and default long parameters and return the long result.
	 * 
	 * @param key
	 *            The key (<code>String</code>) parameter.
	 * @param defaultLong
	 *            The default long (<code>long</code>) parameter.
	 * @return Results of the get long (<code>long</code>) value.
	 */
	public long getLong(final String key, final long defaultLong) {
		return getConfigurationService().getLong(key, defaultLong);
	}

	/**
	 * Get long array with the specified key and default long array parameters and return the long[] result.
	 * 
	 * @param key
	 *            The key (<code>String</code>) parameter.
	 * @param defaultLongArray
	 *            The default long array (<code>long[]</code>) parameter.
	 * @return Results of the get long array (<code>long[]</code>) value.
	 * @since 1.2
	 */
	public long[] getLongArray(final String key, final long[] defaultLongArray) {
		return getConfigurationService().getLongArray(key, defaultLongArray);
	}

	/**
	 * Get object with the specified key and default object parameters and return the Object result.
	 * 
	 * @param key
	 *            The key (<code>String</code>) parameter.
	 * @param defaultObject
	 *            The default object (<code>Object</code>) parameter.
	 * @return Results of the get object (<code>Object</code>) value.
	 * @see #getObjectArray(String,Object[])
	 */
	public Object getObject(final String key, final Object defaultObject) {
		return getConfigurationService().getObject(key, defaultObject);
	}

	/**
	 * Get object with the specified key and default object array parameters and return the Object[] result.
	 * 
	 * @param key
	 *            The key (<code>String</code>) parameter.
	 * @param defaultObjectArray
	 *            The default object array (<code>Object[]</code>) parameter.
	 * @return Results of the get object (<code>Object[]</code>) value.
	 * @see #getObject(String,Object)
	 * @since 1.2
	 * @deprecated Use {@link #getObjectArray(String,Object[])} instead
	 */
	public Object[] getObject(final String key, final Object[] defaultObjectArray) {
		return getObjectArray(key, defaultObjectArray);
	}

	/**
	 * Get object with the specified key and default object array parameters and return the Object[] result.
	 * 
	 * @param key
	 *            The key (<code>String</code>) parameter.
	 * @param defaultObjectArray
	 *            The default object array (<code>Object[]</code>) parameter.
	 * @return Results of the get object (<code>Object[]</code>) value.
	 * @see #getObject(String,Object)
	 * @since 1.2
	 */
	public Object[] getObjectArray(final String key, final Object[] defaultObjectArray) {
		return getConfigurationService().getObjectArray(key, defaultObjectArray);
	}

	/**
	 * Gets the properties (Dictionary) value.
	 * 
	 * @return The properties (<code>Dictionary</code>) value.
	 */
	public Dictionary getProperties() {
		return getConfigurationService().getProperties();
	}

	/**
	 * Get short with the specified key and default short parameters and return the short result.
	 * 
	 * @param key
	 *            The key (<code>String</code>) parameter.
	 * @param defaultShort
	 *            The default short (<code>short</code>) parameter.
	 * @return Results of the get short (<code>short</code>) value.
	 * @see #getShortArray(String,short[])
	 * @since 1.2
	 */
	public short getShort(final String key, final short defaultShort) {
		return getConfigurationService().getShort(key, defaultShort);
	}

	/**
	 * Get short with the specified key and default short array parameters and return the short[] result.
	 * 
	 * @param key
	 *            The key (<code>String</code>) parameter.
	 * @param defaultShortArray
	 *            The default short array (<code>short[]</code>) parameter.
	 * @return Results of the get short (<code>short[]</code>) value.
	 * @see #getShort(String,short)
	 * @since 1.2
	 * @deprecated Use {@link #getShortArray(String,short[])} instead
	 */
	public short[] getShort(final String key, final short[] defaultShortArray) {
		return getShortArray(key, defaultShortArray);
	}

	/**
	 * Get short with the specified key and default short array parameters and return the short[] result.
	 * 
	 * @param key
	 *            The key (<code>String</code>) parameter.
	 * @param defaultShortArray
	 *            The default short array (<code>short[]</code>) parameter.
	 * @return Results of the get short (<code>short[]</code>) value.
	 * @see #getShort(String,short)
	 * @since 1.2
	 */
	public short[] getShortArray(final String key, final short[] defaultShortArray) {
		return getConfigurationService().getShortArray(key, defaultShortArray);
	}

	/**
	 * Get string with the specified key and default string parameters and return the String result.
	 * 
	 * @param key
	 *            The key (<code>String</code>) parameter.
	 * @param defaultString
	 *            The default string (<code>String</code>) parameter.
	 * @return Results of the get string (<code>String</code>) value.
	 * @see #toString()
	 */
	public String getString(final String key, final String defaultString) {
		return getConfigurationService().getString(key, defaultString);
	}

	/**
	 * Get string array with the specified key and default string array parameters and return the String[] result.
	 * 
	 * @param key
	 *            The key (<code>String</code>) parameter.
	 * @param defaultStringArray
	 *            The default string array (<code>String[]</code>) parameter.
	 * @return Results of the get string array (<code>String[]</code>) value.
	 * @since 1.2
	 */
	public String[] getStringArray(final String key, final String[] defaultStringArray) {
		return getConfigurationService().getStringArray(key, defaultStringArray);
	}

	/**
	 * Gets the synchronization service (TestSynchronizationService) value.
	 * 
	 * @return the synchronizationService
	 */
	public TestSynchronizationService getSynchronizationService() {
		TestSynchronizationService synchronizationService = this.context.getSynchronizationService();
		assertNotNull(TestAgent.DefaultResourceBundle.getString(Integer.toString(TestAgent.MISSING_SYNCHRONIZATION)), synchronizationService);
		return synchronizationService;
	}

	/**
	 * Request confirmation with the specified prompt parameter and return the boolean result.
	 * 
	 * @param prompt
	 *            The prompt (<code>String</code>) parameter.
	 * @return Results of the request confirmation (<code>boolean</code>) value.
	 * @throws InterruptedException
	 *             Interrupted Exception.
	 * @throws IOException
	 *             IOException.
	 */
	public boolean requestConfirmation(final String prompt) throws InterruptedException, IOException {
		return getSynchronizationService().requestConfirmation(prompt, this.toString());
	}

	/**
	 * Returns the string value.
	 * 
	 * @return The string (<code>String</code>) value.
	 * @see #getString(String,String)
	 */
	public String toString() {
		return this.context.getTestAgent().getDescription() + ": " + super.toString(); //$NON-NLS-1$
	}
}
