package org.eclipse.soda.dk.testagent.util;

/*************************************************************************
 * Copyright (c) 2007, 2009 IBM.                                         *
 * All rights reserved. This program and the accompanying materials      *
 * are made available under the terms of the Eclipse Public License v1.0 *
 * which accompanies this distribution, and is available at              *
 * http://www.eclipse.org/legal/epl-v10.html                             *
 *                                                                       *
 * Contributors:                                                         *
 *     IBM - initial API and implementation                              *
 ************************************************************************/
import java.util.ArrayList;
import java.util.Dictionary;
import java.util.Enumeration;
import java.util.Hashtable;
import org.eclipse.soda.dk.notification.service.NotificationListener;

/**
 * @author IBM
 * @version 1.2.0
 * @since 1.0
 */
public class NotificationProbe implements NotificationListener {
	/**
	 * Define the received keys (ArrayList) field.
	 */
	private ArrayList receivedKeys;

	/**
	 * Define the received data (ArrayList) field.
	 */
	private ArrayList receivedData;

	/**
	 * Define the mark index (int) field.
	 */
	private int markIndex;

	/**
	 * Constructs an instance of this class.
	 */
	public NotificationProbe() {
		clear();
	}

	/**
	 * Contains with the specified received and data parameters and return the boolean result.
	 * @param received
	 *		The received (<code>Dictionary</code>) parameter.
	 * @param data
	 *		The data (<code>Dictionary</code>) parameter.
	 * @return Results of the contains (<code>boolean</code>) value.
	 */
	private static boolean contains(final Dictionary received, final Dictionary data) {
		for (final Enumeration e = data.keys(); e.hasMoreElements();) {
			final Object dataKey = e.nextElement();
			final Object value = data.get(dataKey);
			final Object receivedValue = received.get(dataKey);
			if (receivedValue != value) {
				if ((value == null) || !value.equals(receivedValue)) {
					return false;
				}
			}
		}
		return true;
	}

	/**
	 * To data with the specified data key and data value parameters and return the Dictionary result.
	 * @param dataKey
	 *		The data key (<code>Object</code>) parameter.
	 * @param dataValue
	 *		The data value (<code>Object</code>) parameter.
	 * @return Results of the to data (<code>Dictionary</code>) value.
	 * @see #getAllPublishedData(String)
	 * @see #getFirstPublishedData(String)
	 * @see #getLastPublishedData(String)
	 */
	public static Dictionary toData(final Object dataKey, final Object dataValue) {
		Dictionary data = new Hashtable(1);
		data.put(dataKey, dataValue);
		return data;
	}

	/**
	 * Clear.
	 */
	public synchronized void clear() {
		clearMark();
		this.receivedKeys = new ArrayList();
		this.receivedData = new ArrayList();
	}

	/**
	 * Clear mark.
	 * @see #setMark()
	 */
	public synchronized void clearMark() {
		this.markIndex = 0;
	}

	/**
	 * Contains with the specified received index and data parameters and return the boolean result.
	 * @param receivedIndex
	 *		The received index (<code>int</code>) parameter.
	 * @param data
	 *		The data (<code>Dictionary</code>) parameter.
	 * @return Results of the contains (<code>boolean</code>) value.
	 */
	private boolean contains(final int receivedIndex, final Dictionary data) {
		Dictionary received = (Dictionary) this.receivedData.get(receivedIndex);
		return contains(received, data);
	}

	/**
	 * Count key published with the specified key parameter and return the int result.
	 * @param topicKey
	 *		The topic key (<code>String</code>) parameter.
	 * @return Results of the count key published (<code>int</code>) value.
	 * @see #countKeyPublished(String,Dictionary)
	 * @see #countKeyPublished(String,Dictionary,long)
	 * @see #countKeyPublished(String,long)
	 * @see #countKeyPublished(String,Object,Object)
	 * @see #countKeyPublished(String,Object,Object,long)
	 * @see #waitKeyPublished(String,Dictionary,int,int)
	 * @see #waitKeyPublished(String,int,int)
	 * @see #waitKeyPublished(String[],int)
	 * @see #wasKeyPublished(String)
	 * @see #wasKeyPublished(String,Dictionary)
	 * @see #wasKeyPublished(String,Dictionary,int)
	 * @see #wasKeyPublished(String,int)
	 * @see #wasKeyPublished(String,Object,Object)
	 * @see #wasKeyPublished(String,Object,Object,int)
	 * @see #wasKeyPublished(String[])
	 */
	public int countKeyPublished(final String topicKey) {
		int result = 0;
		synchronized (this) {
			for (int i = this.markIndex; i < this.receivedKeys.size(); i++) {
				String receivedKey = (String) this.receivedKeys.get(i);
				if (topicKey.equals(receivedKey)) {
					result++;
				}
			}
		}
		return result;
	}

	/**
	 * Count key published with the specified key and data parameters and return the int result.
	 * @param topicKey
	 *		The topic key (<code>String</code>) parameter.
	 * @param data
	 *		The data (<code>Dictionary</code>) parameter.
	 * @return Results of the count key published (<code>int</code>) value.
	 * @see #countKeyPublished(String)
	 * @see #countKeyPublished(String,Dictionary,long)
	 * @see #countKeyPublished(String,long)
	 * @see #countKeyPublished(String,Object,Object)
	 * @see #countKeyPublished(String,Object,Object,long)
	 * @see #waitKeyPublished(String,Dictionary,int,int)
	 * @see #waitKeyPublished(String,int,int)
	 * @see #waitKeyPublished(String[],int)
	 * @see #wasKeyPublished(String)
	 * @see #wasKeyPublished(String,Dictionary)
	 * @see #wasKeyPublished(String,Dictionary,int)
	 * @see #wasKeyPublished(String,int)
	 * @see #wasKeyPublished(String,Object,Object)
	 * @see #wasKeyPublished(String,Object,Object,int)
	 * @see #wasKeyPublished(String[])
	 */
	public int countKeyPublished(final String topicKey, final Dictionary data) {
		int result = 0;
		synchronized (this) {
			for (int i = this.markIndex; i < this.receivedKeys.size(); i++) {
				String receivedKey = (String) this.receivedKeys.get(i);
				if (topicKey.equals(receivedKey)) {
					if (contains(i, data)) {
						result++;
					}
				}
			}
		}
		return result;
	}

	/**
	 * Count key published with the specified key, data and duration parameters and return the int result.
	 * @param topicKey
	 *		The topic key (<code>String</code>) parameter.
	 * @param data
	 *		The data (<code>Dictionary</code>) parameter.
	 * @param duration
	 *		The duration (<code>long</code>) parameter.
	 * @return Results of the count key published (<code>int</code>) value.
	 * @see #countKeyPublished(String)
	 * @see #countKeyPublished(String,Dictionary)
	 * @see #countKeyPublished(String,long)
	 * @see #countKeyPublished(String,Object,Object)
	 * @see #countKeyPublished(String,Object,Object,long)
	 * @see #waitKeyPublished(String,Dictionary,int,int)
	 * @see #waitKeyPublished(String,int,int)
	 * @see #waitKeyPublished(String[],int)
	 * @see #wasKeyPublished(String)
	 * @see #wasKeyPublished(String,Dictionary)
	 * @see #wasKeyPublished(String,Dictionary,int)
	 * @see #wasKeyPublished(String,int)
	 * @see #wasKeyPublished(String,Object,Object)
	 * @see #wasKeyPublished(String,Object,Object,int)
	 * @see #wasKeyPublished(String[])
	 */
	public int countKeyPublished(final String topicKey, final Dictionary data, final long duration) {
		Delay.milliseconds(duration);
		return countKeyPublished(topicKey, data);
	}

	/**
	 * Count key published with the specified key and duration parameters and return the int result.
	 * @param topicKey
	 *		The topic key (<code>String</code>) parameter.
	 * @param duration
	 *		The duration (<code>long</code>) parameter.
	 * @return Results of the count key published (<code>int</code>) value.
	 * @see #countKeyPublished(String)
	 * @see #countKeyPublished(String,Dictionary)
	 * @see #countKeyPublished(String,Dictionary,long)
	 * @see #countKeyPublished(String,Object,Object)
	 * @see #countKeyPublished(String,Object,Object,long)
	 * @see #waitKeyPublished(String,Dictionary,int,int)
	 * @see #waitKeyPublished(String,int,int)
	 * @see #waitKeyPublished(String[],int)
	 * @see #wasKeyPublished(String)
	 * @see #wasKeyPublished(String,Dictionary)
	 * @see #wasKeyPublished(String,Dictionary,int)
	 * @see #wasKeyPublished(String,int)
	 * @see #wasKeyPublished(String,Object,Object)
	 * @see #wasKeyPublished(String,Object,Object,int)
	 * @see #wasKeyPublished(String[])
	 */
	public int countKeyPublished(final String topicKey, final long duration) {
		Delay.milliseconds(duration);
		return countKeyPublished(topicKey);
	}

	/**
	 * Count key published with the specified key, data key and data value parameters and return the int result.
	 * @param topicKey
	 *		The topic key (<code>String</code>) parameter.
	 * @param dataKey
	 *		The data key (<code>Object</code>) parameter.
	 * @param dataValue
	 *		The data value (<code>Object</code>) parameter.
	 * @return Results of the count key published (<code>int</code>) value.
	 * @see #countKeyPublished(String)
	 * @see #countKeyPublished(String,Dictionary)
	 * @see #countKeyPublished(String,Dictionary,long)
	 * @see #countKeyPublished(String,long)
	 * @see #countKeyPublished(String,Object,Object,long)
	 * @see #waitKeyPublished(String,Dictionary,int,int)
	 * @see #waitKeyPublished(String,int,int)
	 * @see #waitKeyPublished(String[],int)
	 * @see #wasKeyPublished(String)
	 * @see #wasKeyPublished(String,Dictionary)
	 * @see #wasKeyPublished(String,Dictionary,int)
	 * @see #wasKeyPublished(String,int)
	 * @see #wasKeyPublished(String,Object,Object)
	 * @see #wasKeyPublished(String,Object,Object,int)
	 * @see #wasKeyPublished(String[])
	 */
	public int countKeyPublished(final String topicKey, final Object dataKey, final Object dataValue) {
		return countKeyPublished(topicKey, toData(dataKey, dataValue));
	}

	/**
	 * Count key published with the specified key, data key, data value and duration parameters and return the int result.
	 * @param topicKey
	 *		The topic key (<code>String</code>) parameter.
	 * @param dataKey
	 *		The data key (<code>Object</code>) parameter.
	 * @param dataValue
	 *		The data value (<code>Object</code>) parameter.
	 * @param duration
	 *		The duration (<code>long</code>) parameter.
	 * @return Results of the count key published (<code>int</code>) value.
	 * @see #countKeyPublished(String)
	 * @see #countKeyPublished(String,Dictionary)
	 * @see #countKeyPublished(String,Dictionary,long)
	 * @see #countKeyPublished(String,long)
	 * @see #countKeyPublished(String,Object,Object)
	 * @see #waitKeyPublished(String,Dictionary,int,int)
	 * @see #waitKeyPublished(String,int,int)
	 * @see #waitKeyPublished(String[],int)
	 * @see #wasKeyPublished(String)
	 * @see #wasKeyPublished(String,Dictionary)
	 * @see #wasKeyPublished(String,Dictionary,int)
	 * @see #wasKeyPublished(String,int)
	 * @see #wasKeyPublished(String,Object,Object)
	 * @see #wasKeyPublished(String,Object,Object,int)
	 * @see #wasKeyPublished(String[])
	 */
	public int countKeyPublished(final String topicKey, final Object dataKey, final Object dataValue, final long duration) {
		return countKeyPublished(topicKey, toData(dataKey, dataValue), duration);
	}

	/**
	 * Get all published data with the specified key parameter and return the ArrayList result.
	 * @param topicKey
	 *		The topic key (<code>String</code>) parameter.
	 * @return Results of the get all published data (<code>ArrayList</code>) value.
	 */
	public ArrayList getAllPublishedData(final String topicKey) {
		ArrayList data = new ArrayList();
		synchronized (this) {
			for (int i = this.markIndex; i < this.receivedKeys.size(); i++) {
				String receivedKey = (String) this.receivedKeys.get(i);
				if (topicKey.equals(receivedKey)) {
					data.add(this.receivedData.get(i));
				}
			}
		}
		return data;
	}

	/**
	 * Get all published values with the specified topicKey and dataKey parameters and return the ArrayList result.
	 * @param topicKey
	 *		The topic key (<code>String</code>) parameter.
	 * @param dataKey
	 *		The data key (<code>String</code>) parameter.
	 * @return Results of the get all published values (<code>ArrayList</code>) value.
	 */
	public ArrayList getAllPublishedValues(final String topicKey, final String dataKey) {
		ArrayList values = new ArrayList();
		synchronized (this) {
			for (int i = this.markIndex; i < this.receivedKeys.size(); i++) {
				String receivedKey = (String) this.receivedKeys.get(i);
				if (topicKey.equals(receivedKey)) {
					Dictionary data = (Dictionary) this.receivedData.get(i);
					Object value = data.get(dataKey);
					if (value != null) {
						values.add(value);
					}
				}
			}
		}
		return values;
	}

	/**
	 * Get first published data with the specified key parameter and return the Dictionary result.
	 * @param topicKey
	 *		The topic key (<code>String</code>) parameter.
	 * @return Results of the get first published data (<code>Dictionary</code>) value.
	 */
	public Dictionary getFirstPublishedData(final String topicKey) {
		synchronized (this) {
			for (int i = this.markIndex; i < this.receivedKeys.size(); i++) {
				String receivedKey = (String) this.receivedKeys.get(i);
				if (topicKey.equals(receivedKey)) {
					return (Dictionary) this.receivedData.get(i);
				}
			}
		}
		return null;
	}

	/**
	 * Get first published value with the topicKey and dataKey parameters and return the Dictionary result.
	 * @param topicKey
	 *		The topic key (<code>String</code>) parameter.
	 * @param dataKey
	 *		The data key (<code>String</code>) parameter.
	 * @return Results of the get first published value (<code>Object</code>) value.
	 */
	public Object getFirstPublishedValue(final String topicKey, final String dataKey) {
		Dictionary data = getFirstPublishedData(topicKey);
		return data.get(dataKey);
	}

	/**
	 * Get last published data with the specified key parameter and return the Dictionary result.
	 * @param topicKey
	 *		The topic key (<code>String</code>) parameter.
	 * @return Results of the get last published data (<code>Dictionary</code>) value.
	 */
	public Dictionary getLastPublishedData(final String topicKey) {
		synchronized (this) {
			for (int i = this.receivedKeys.size() - 1; i >= this.markIndex; i--) {
				String receivedKey = (String) this.receivedKeys.get(i);
				if (topicKey.equals(receivedKey)) {
					return (Dictionary) this.receivedData.get(i);
				}
			}
		}
		return null;
	}

	/**
	 * Get last published value with the specified topicKey and dataKey parameters and return the Dictionary result.
	 * @param topicKey
	 *		The topic key (<code>String</code>) parameter.
	 * @param dataKey
	 *		The data key (<code>String</code>) parameter.
	 * @return Results of the get last published value (<code>Object</code>) value.
	 */
	public Object getLastPublishedValue(final String topicKey, final String dataKey) {
		Dictionary data = getLastPublishedData(topicKey);
		return data.get(dataKey);
	}

	/**
	 * Notification received with the specified external key and data parameters.
	 * @param externalKey
	 *		The external key (<code>String</code>) parameter.
	 * @param data
	 *		The data (<code>Dictionary</code>) parameter.
	 */
	public synchronized void notificationReceived(final String externalKey, final Dictionary data) {
		this.receivedKeys.add(externalKey);
		this.receivedData.add(data);
		notifyAll();
	}

	/**
	 * Set mark.
	 * @see #clearMark()
	 */
	public synchronized void setMark() {
		this.markIndex = this.receivedKeys.size();
	}

	/**
	 * Wait key published with the specified topic key, data, expected count and duration parameters and return the int result.
	 * @param topicKey
	 *		The topic key (<code>String</code>) parameter.
	 * @param data
	 *		The data (<code>Dictionary</code>) parameter.
	 * @param expectedCount
	 *		The expected count (<code>int</code>) parameter.
	 * @param duration
	 *		The duration (<code>int</code>) parameter.
	 * @return Results of the wait key published (<code>int</code>) value.
	 * @throws InterruptedException Interrupted Exception.
	 * @see #countKeyPublished(String)
	 * @see #countKeyPublished(String,Dictionary)
	 * @see #countKeyPublished(String,Dictionary,long)
	 * @see #countKeyPublished(String,long)
	 * @see #countKeyPublished(String,Object,Object)
	 * @see #countKeyPublished(String,Object,Object,long)
	 * @see #waitKeyPublished(String,int,int)
	 * @see #waitKeyPublished(String[],int)
	 * @see #wasKeyPublished(String)
	 * @see #wasKeyPublished(String,Dictionary)
	 * @see #wasKeyPublished(String,Dictionary,int)
	 * @see #wasKeyPublished(String,int)
	 * @see #wasKeyPublished(String,Object,Object)
	 * @see #wasKeyPublished(String,Object,Object,int)
	 * @see #wasKeyPublished(String[])
	 */
	public int waitKeyPublished(final String topicKey, final Dictionary data, final int expectedCount, final int duration) throws InterruptedException {
		long now = System.currentTimeMillis();
		long limitTime = now + duration;
		int count = 0;
		synchronized (this) {
			while (now < limitTime) {
				count = countKeyPublished(topicKey, data);
				if (count >= expectedCount) {
					return count;
				}
				long time = limitTime - now;
				wait(time);
				now = System.currentTimeMillis();
			}
		}
		return count;
	}

	/**
	 * Wait key published with the specified topic key, expected count and duration parameters and return the int result.
	 * @param topicKey
	 *		The topic key (<code>String</code>) parameter.
	 * @param expectedCount
	 *		The expected count (<code>int</code>) parameter.
	 * @param duration
	 *		The duration (<code>int</code>) parameter.
	 * @return Results of the wait key published (<code>int</code>) value.
	 * @throws InterruptedException Interrupted Exception.
	 * @see #countKeyPublished(String)
	 * @see #countKeyPublished(String,Dictionary)
	 * @see #countKeyPublished(String,Dictionary,long)
	 * @see #countKeyPublished(String,long)
	 * @see #countKeyPublished(String,Object,Object)
	 * @see #countKeyPublished(String,Object,Object,long)
	 * @see #waitKeyPublished(String,Dictionary,int,int)
	 * @see #waitKeyPublished(String[],int)
	 * @see #wasKeyPublished(String)
	 * @see #wasKeyPublished(String,Dictionary)
	 * @see #wasKeyPublished(String,Dictionary,int)
	 * @see #wasKeyPublished(String,int)
	 * @see #wasKeyPublished(String,Object,Object)
	 * @see #wasKeyPublished(String,Object,Object,int)
	 * @see #wasKeyPublished(String[])
	 */
	public int waitKeyPublished(final String topicKey, final int expectedCount, final int duration) throws InterruptedException {
		long now = System.currentTimeMillis();
		long limitTime = now + duration;
		int count = 0;
		synchronized (this) {
			while (now < limitTime) {
				count = countKeyPublished(topicKey);
				if (count >= expectedCount) {
					return count;
				}
				long time = limitTime - now;
				wait(time);
				now = System.currentTimeMillis();
			}
		}
		return count;
	}

	/**
	 * Wait key published with the specified topic keys and duration parameters and return the String result.
	 * @param topicKeys
	 *		The topic keys (<code>String[]</code>) parameter.
	 * @param duration
	 *		The duration (<code>int</code>) parameter.
	 * @return Results of the wait key published (<code>String</code>) value.
	 * @throws InterruptedException Interrupted Exception.
	 * @see #countKeyPublished(String)
	 * @see #countKeyPublished(String,Dictionary)
	 * @see #countKeyPublished(String,Dictionary,long)
	 * @see #countKeyPublished(String,long)
	 * @see #countKeyPublished(String,Object,Object)
	 * @see #countKeyPublished(String,Object,Object,long)
	 * @see #waitKeyPublished(String,Dictionary,int,int)
	 * @see #waitKeyPublished(String,int,int)
	 * @see #wasKeyPublished(String)
	 * @see #wasKeyPublished(String,Dictionary)
	 * @see #wasKeyPublished(String,Dictionary,int)
	 * @see #wasKeyPublished(String,int)
	 * @see #wasKeyPublished(String,Object,Object)
	 * @see #wasKeyPublished(String,Object,Object,int)
	 * @see #wasKeyPublished(String[])
	 */
	public String waitKeyPublished(final String[] topicKeys, final int duration) throws InterruptedException {
		long now = System.currentTimeMillis();
		long limitTime = now + duration;
		String receivedKey = null;
		synchronized (this) {
			while (now < limitTime) {
				receivedKey = wasKeyPublished(topicKeys);
				if (receivedKey != null) {
					return receivedKey;
				}
				long time = limitTime - now;
				wait(time);
				now = System.currentTimeMillis();
			}
		}
		return receivedKey;
	}

	/**
	 * Was data published with the specified data parameter and return the boolean result.
	 * @param data
	 *		The data (<code>Dictionary</code>) parameter.
	 * @return Results of the was data published (<code>boolean</code>) value.
	 * @see #wasDataPublished(Dictionary,int)
	 */
	public boolean wasDataPublished(final Dictionary data) {
		synchronized (this) {
			for (int i = this.receivedData.size() - 1; i >= this.markIndex; i--) {
				if (contains(i, data)) {
					return true;
				}
			}
		}
		return false;
	}

	/**
	 * Was data published with the specified data and duration parameters and return the boolean result.
	 * @param data
	 *		The data (<code>Dictionary</code>) parameter.
	 * @param duration
	 *		The duration (<code>int</code>) parameter.
	 * @return Results of the was data published (<code>boolean</code>) value.
	 * @throws InterruptedException Interrupted Exception.
	 * @see #wasDataPublished(Dictionary)
	 */
	public boolean wasDataPublished(final Dictionary data, final int duration) throws InterruptedException {
		long now = System.currentTimeMillis();
		long limitTime = now + duration;
		synchronized (this) {
			while (now < limitTime) {
				boolean wasPublished = wasDataPublished(data);
				if (wasPublished) {
					return true;
				}
				long time = limitTime - now;
				wait(time);
				now = System.currentTimeMillis();
			}
		}
		return false;
	}

	/**
	 * Was key published with the specified key parameter and return the boolean result.
	 * @param topicKey
	 *		The topic key (<code>String</code>) parameter.
	 * @return Results of the was key published (<code>boolean</code>) value.
	 * @see #countKeyPublished(String)
	 * @see #countKeyPublished(String,Dictionary)
	 * @see #countKeyPublished(String,Dictionary,long)
	 * @see #countKeyPublished(String,long)
	 * @see #countKeyPublished(String,Object,Object)
	 * @see #countKeyPublished(String,Object,Object,long)
	 * @see #waitKeyPublished(String,Dictionary,int,int)
	 * @see #waitKeyPublished(String,int,int)
	 * @see #waitKeyPublished(String[],int)
	 * @see #wasKeyPublished(String,Dictionary)
	 * @see #wasKeyPublished(String,Dictionary,int)
	 * @see #wasKeyPublished(String,int)
	 * @see #wasKeyPublished(String,Object,Object)
	 * @see #wasKeyPublished(String,Object,Object,int)
	 * @see #wasKeyPublished(String[])
	 */
	public boolean wasKeyPublished(final String topicKey) {
		synchronized (this) {
			for (int i = this.receivedKeys.size() - 1; i >= this.markIndex; i--) {
				String receivedKey = (String) this.receivedKeys.get(i);
				if (topicKey.equals(receivedKey)) {
					return true;
				}
			}
		}
		return false;
	}

	/**
	 * Was key published with the specified key and data parameters and return the boolean result.
	 * @param topicKey
	 *		The topic key (<code>String</code>) parameter.
	 * @param data
	 *		The data (<code>Dictionary</code>) parameter.
	 * @return Results of the was key published (<code>boolean</code>) value.
	 * @see #countKeyPublished(String)
	 * @see #countKeyPublished(String,Dictionary)
	 * @see #countKeyPublished(String,Dictionary,long)
	 * @see #countKeyPublished(String,long)
	 * @see #countKeyPublished(String,Object,Object)
	 * @see #countKeyPublished(String,Object,Object,long)
	 * @see #waitKeyPublished(String,Dictionary,int,int)
	 * @see #waitKeyPublished(String,int,int)
	 * @see #waitKeyPublished(String[],int)
	 * @see #wasKeyPublished(String)
	 * @see #wasKeyPublished(String,Dictionary,int)
	 * @see #wasKeyPublished(String,int)
	 * @see #wasKeyPublished(String,Object,Object)
	 * @see #wasKeyPublished(String,Object,Object,int)
	 * @see #wasKeyPublished(String[])
	 */
	public boolean wasKeyPublished(final String topicKey, final Dictionary data) {
		synchronized (this) {
			for (int i = this.receivedKeys.size() - 1; i >= this.markIndex; i--) {
				String receivedKey = (String) this.receivedKeys.get(i);
				if (topicKey.equals(receivedKey)) {
					if (contains(i, data)) {
						return true;
					}
				}
			}
		}
		return false;
	}

	/**
	 * Was key published with the specified key, data and duration parameters and return the boolean result.
	 * @param topicKey
	 *		The topic key (<code>String</code>) parameter.
	 * @param data
	 *		The data (<code>Dictionary</code>) parameter.
	 * @param duration
	 *		The duration (<code>int</code>) parameter.
	 * @return Results of the was key published (<code>boolean</code>) value.
	 * @throws InterruptedException Interrupted Exception.
	 * @see #countKeyPublished(String)
	 * @see #countKeyPublished(String,Dictionary)
	 * @see #countKeyPublished(String,Dictionary,long)
	 * @see #countKeyPublished(String,long)
	 * @see #countKeyPublished(String,Object,Object)
	 * @see #countKeyPublished(String,Object,Object,long)
	 * @see #waitKeyPublished(String,Dictionary,int,int)
	 * @see #waitKeyPublished(String,int,int)
	 * @see #waitKeyPublished(String[],int)
	 * @see #wasKeyPublished(String)
	 * @see #wasKeyPublished(String,Dictionary)
	 * @see #wasKeyPublished(String,int)
	 * @see #wasKeyPublished(String,Object,Object)
	 * @see #wasKeyPublished(String,Object,Object,int)
	 * @see #wasKeyPublished(String[])
	 */
	public boolean wasKeyPublished(final String topicKey, final Dictionary data, final int duration) throws InterruptedException {
		long now = System.currentTimeMillis();
		long limitTime = now + duration;
		synchronized (this) {
			while (now < limitTime) {
				boolean wasPublished = wasKeyPublished(topicKey, data);
				if (wasPublished) {
					return true;
				}
				long time = limitTime - now;
				wait(time);
				now = System.currentTimeMillis();
			}
		}
		return false;
	}

	/**
	 * Was key published with the specified key and duration parameters and return the boolean result.
	 * @param topicKey
	 *		The topic key (<code>String</code>) parameter.
	 * @param duration
	 *		The duration (<code>int</code>) parameter.
	 * @return Results of the was key published (<code>boolean</code>) value.
	 * @throws InterruptedException Interrupted Exception.
	 * @see #countKeyPublished(String)
	 * @see #countKeyPublished(String,Dictionary)
	 * @see #countKeyPublished(String,Dictionary,long)
	 * @see #countKeyPublished(String,long)
	 * @see #countKeyPublished(String,Object,Object)
	 * @see #countKeyPublished(String,Object,Object,long)
	 * @see #waitKeyPublished(String,Dictionary,int,int)
	 * @see #waitKeyPublished(String,int,int)
	 * @see #waitKeyPublished(String[],int)
	 * @see #wasKeyPublished(String)
	 * @see #wasKeyPublished(String,Dictionary)
	 * @see #wasKeyPublished(String,Dictionary,int)
	 * @see #wasKeyPublished(String,Object,Object)
	 * @see #wasKeyPublished(String,Object,Object,int)
	 * @see #wasKeyPublished(String[])
	 */
	public boolean wasKeyPublished(final String topicKey, final int duration) throws InterruptedException {
		long now = System.currentTimeMillis();
		long limitTime = now + duration;
		synchronized (this) {
			while (now < limitTime) {
				boolean wasPublished = wasKeyPublished(topicKey);
				if (wasPublished) {
					return true;
				}
				long time = limitTime - now;
				wait(time);
				now = System.currentTimeMillis();
			}
		}
		return false;
	}

	/**
	 * Was key published with the specified key, data key and data value parameters and return the boolean result.
	 * @param topicKey
	 *		The topic key (<code>String</code>) parameter.
	 * @param dataKey
	 *		The data key (<code>Object</code>) parameter.
	 * @param dataValue
	 *		The data value (<code>Object</code>) parameter.
	 * @return Results of the was key published (<code>boolean</code>) value.
	 * @see #countKeyPublished(String)
	 * @see #countKeyPublished(String,Dictionary)
	 * @see #countKeyPublished(String,Dictionary,long)
	 * @see #countKeyPublished(String,long)
	 * @see #countKeyPublished(String,Object,Object)
	 * @see #countKeyPublished(String,Object,Object,long)
	 * @see #waitKeyPublished(String,Dictionary,int,int)
	 * @see #waitKeyPublished(String,int,int)
	 * @see #waitKeyPublished(String[],int)
	 * @see #wasKeyPublished(String)
	 * @see #wasKeyPublished(String,Dictionary)
	 * @see #wasKeyPublished(String,Dictionary,int)
	 * @see #wasKeyPublished(String,int)
	 * @see #wasKeyPublished(String,Object,Object,int)
	 * @see #wasKeyPublished(String[])
	 */
	public boolean wasKeyPublished(final String topicKey, final Object dataKey, final Object dataValue) {
		return wasKeyPublished(topicKey, toData(dataKey, dataValue));
	}

	/**
	 * Was key published with the specified key, data key, data value and duration parameters and return the boolean result.
	 * @param topicKey
	 *		The topic key (<code>String</code>) parameter.
	 * @param dataKey
	 *		The data key (<code>Object</code>) parameter.
	 * @param dataValue
	 *		The data value (<code>Object</code>) parameter.
	 * @param duration
	 *		The duration (<code>int</code>) parameter.
	 * @return Results of the was key published (<code>boolean</code>) value.
	 * @throws InterruptedException Interrupted Exception.
	 * @see #countKeyPublished(String)
	 * @see #countKeyPublished(String,Dictionary)
	 * @see #countKeyPublished(String,Dictionary,long)
	 * @see #countKeyPublished(String,long)
	 * @see #countKeyPublished(String,Object,Object)
	 * @see #countKeyPublished(String,Object,Object,long)
	 * @see #waitKeyPublished(String,Dictionary,int,int)
	 * @see #waitKeyPublished(String,int,int)
	 * @see #waitKeyPublished(String[],int)
	 * @see #wasKeyPublished(String)
	 * @see #wasKeyPublished(String,Dictionary)
	 * @see #wasKeyPublished(String,Dictionary,int)
	 * @see #wasKeyPublished(String,int)
	 * @see #wasKeyPublished(String,Object,Object)
	 * @see #wasKeyPublished(String[])
	 */
	public boolean wasKeyPublished(final String topicKey, final Object dataKey, final Object dataValue, final int duration) throws InterruptedException {
		return wasKeyPublished(topicKey, toData(dataKey, dataValue), duration);
	}

	/**
	 * Was key published with the specified topic keys parameter and return the String result.
	 * @param topicKeys
	 *		The topic keys (<code>String[]</code>) parameter.
	 * @return Results of the was key published (<code>String</code>) value.
	 * @see #countKeyPublished(String)
	 * @see #countKeyPublished(String,Dictionary)
	 * @see #countKeyPublished(String,Dictionary,long)
	 * @see #countKeyPublished(String,long)
	 * @see #countKeyPublished(String,Object,Object)
	 * @see #countKeyPublished(String,Object,Object,long)
	 * @see #waitKeyPublished(String,Dictionary,int,int)
	 * @see #waitKeyPublished(String,int,int)
	 * @see #waitKeyPublished(String[],int)
	 * @see #wasKeyPublished(String)
	 * @see #wasKeyPublished(String,Dictionary)
	 * @see #wasKeyPublished(String,Dictionary,int)
	 * @see #wasKeyPublished(String,int)
	 * @see #wasKeyPublished(String,Object,Object)
	 * @see #wasKeyPublished(String,Object,Object,int)
	 */
	public String wasKeyPublished(final String[] topicKeys) {
		synchronized (this) {
			for (int i = this.receivedKeys.size() - 1; i >= this.markIndex; i--) {
				String receivedKey = (String) this.receivedKeys.get(i);
				for (int j = 0; j < topicKeys.length; j++) {
					if (topicKeys[j].equals(receivedKey)) {
						return receivedKey;
					}
				}
			}
		}
		return null;
	}

	/**
	 * Was value published with the specified data key and data value parameters and return the boolean result.
	 * @param dataKey
	 *		The data key (<code>Object</code>) parameter.
	 * @param dataValue
	 *		The data value (<code>Object</code>) parameter.
	 * @return Results of the was value published (<code>boolean</code>) value.
	 * @see #wasValuePublished(Object,Object,int)
	 */
	public boolean wasValuePublished(final Object dataKey, final Object dataValue) {
		return wasDataPublished(toData(dataKey, dataValue));
	}

	/**
	 * Was value published with the specified data key, data value and duration parameters and return the boolean result.
	 * @param dataKey
	 *		The data key (<code>Object</code>) parameter.
	 * @param dataValue
	 *		The data value (<code>Object</code>) parameter.
	 * @param duration
	 *		The duration (<code>int</code>) parameter.
	 * @return Results of the was value published (<code>boolean</code>) value.
	 * @throws InterruptedException Interrupted Exception.
	 * @see #wasValuePublished(Object,Object)
	 */
	public boolean wasValuePublished(final Object dataKey, final Object dataValue, final int duration) throws InterruptedException {
		return wasDataPublished(toData(dataKey, dataValue), duration);
	}
}
