package org.eclipse.soda.dk.testcontroller;

/*************************************************************************
 * Copyright (c) 2007, 2009 IBM.                                         *
 * All rights reserved. This program and the accompanying materials      *
 * are made available under the terms of the Eclipse Public License v1.0 *
 * which accompanies this distribution, and is available at              *
 * http://www.eclipse.org/legal/epl-v10.html                             *
 *                                                                       *
 * Contributors:                                                         *
 *     IBM - initial API and implementation                              *
 ************************************************************************/
import java.io.IOException;
import java.io.PrintWriter;
import java.io.StringWriter;
import java.util.Dictionary;
import junit.framework.AssertionFailedError;
import junit.framework.Test;
import junit.framework.TestListener;
import junit.framework.TestResult;
import org.eclipse.soda.dk.nls.Nls;
import org.eclipse.soda.dk.testagent.service.TestAgentService;
import org.eclipse.soda.dk.testagent.service.TestSynchronizationService;
import org.eclipse.soda.dk.testcontroller.service.TestRunListenerService;

/**
 * @author IBM
 * @version 1.2.0
 * @since 1.0
 */
public class TestGroupRunner implements TestListener {
	/**
	 * Define the agents (TestAgentService) field.
	 */
	private TestAgentService agents[];

	/**
	 * Define the run id (String) field.
	 */
	String runId;

	/**
	 * Define the properties (Dictionary) field.
	 */
	Dictionary properties;

	/**
	 * Define the sync service (TestSynchronizationService) field.
	 */
	TestSynchronizationService syncService;

	/**
	 * Define the run listeners (TestRunListenerService) field.
	 */
	TestRunListenerService runListeners[];

	/**
	 * Define the test group (ThreadGroup) field.
	 */
	private ThreadGroup testGroup;

	/**
	 * Define the test results (TestResult) field.
	 */
	private TestResult testResults[];

	/**
	 * Define the runner thread (Thread) field.
	 */
	private Thread runnerThread;

	/**
	 * Define the test threads (Thread) field.
	 */
	private Thread testThreads[];

	/**
	 * Define the action id (String) field.
	 */
	private String actionId;

	/**
	 * Constructs an instance of this class from the specified run id, agents, properties, sync service and run listeners parameters.
	 * @param actionId The action id (<code>String</code>) parameter.
	 * @param runId	The run id (<code>String</code>) parameter.
	 * @param agents	The agents (<code>TestAgentService</code>) parameter.
	 * @param properties	The properties (<code>Dictionary</code>) parameter.
	 * @param syncService	The sync service (<code>TestSynchronizationService</code>) parameter.
	 * @param runListeners	The run listeners (<code>TestRunListenerService[]</code>) parameter.
	 */
	public TestGroupRunner(final String actionId, final String runId, final TestAgentService[] agents, final Dictionary properties, final TestSynchronizationService syncService, final TestRunListenerService[] runListeners) {
		this.actionId = actionId;
		this.runId = runId;
		this.agents = agents;
		this.properties = properties;
		this.syncService = syncService;
		this.runListeners = runListeners;
		init();
	}

	/**
	 * To trace with the specified t parameter and return the String result.
	 * @param t	The t (<code>Throwable</code>) parameter.
	 * @return	Results of the to trace (<code>String</code>) value.
	 */
	private static String toTrace(final Throwable t) {
		StringWriter writer = new StringWriter();
		t.printStackTrace(new PrintWriter(writer));
		return writer.toString();
	}

	/**
	 * Add error with the specified test and t parameters.
	 * @param test	The test (<code>Test</code>) parameter.
	 * @param t	The t (<code>Throwable</code>) parameter.
	 */
	public void addError(final Test test, final Throwable t) {
		try {
			for (int i = 0; i < TestGroupRunner.this.runListeners.length; i++) {
				this.runListeners[i].testError(this.runId, test.toString(), toTrace(t));
			}
		} catch (final IOException e) {
			/* ignore exception */
		}
	}

	/**
	 * Add failure with the specified test and t parameters.
	 * @param test	The test (<code>Test</code>) parameter.
	 * @param t	The t (<code>AssertionFailedError</code>) parameter.
	 */
	public void addFailure(final Test test, final AssertionFailedError t) {
		try {
			for (int i = 0; i < TestGroupRunner.this.runListeners.length; i++) {
				this.runListeners[i].testFailed(this.runId, test.toString(), toTrace(t));
			}
		} catch (final IOException e) {
			/* ignore exception */
		}
	}

	/**
	 * End test with the specified test parameter.
	 * @param test	The test (<code>Test</code>) parameter.
	 * @see #startTest(Test)
	 */
	public void endTest(final Test test) {
		try {
			for (int i = 0; i < TestGroupRunner.this.runListeners.length; i++) {
				this.runListeners[i].testEnded(this.runId, test.toString());
			}
		} catch (final IOException e) {
			/* ignore exception */
		}
	}

	/**
	 * Gets the runner thread (Thread) value.
	 * @return The thread (<code>Thread</code>) value.
	 */
	public Thread getRunnerThread() {
		return this.runnerThread;
	}

	/**
	 * Init before run.
	 */
	public void init() {
		this.testGroup = new ThreadGroup("TestGroup"); //$NON-NLS-1$
		this.testGroup.setDaemon(true);
		this.testResults = new TestResult[this.agents.length];
		this.testThreads = new Thread[this.agents.length];
		try {
			for (int i = 0; i < this.runListeners.length; i++) {
				this.runListeners[i].testRunStarted(this.actionId, this.runId);
			}
		} catch (IOException e) {
			/* Ignore (remote) failure to report event */
		}
	}

	/**
	 * Run tests.
	 */
	public void runTests() {
		try {
			this.runnerThread = Thread.currentThread();
			startTests();
			waitForTests();
		} finally {
			try {
				for (int i = 0; i < this.runListeners.length; i++) {
					this.runListeners[i].testRunEnded(this.runId);
				}
			} catch (final IOException e) {
				/* ignore exception */
			}
		}
	}

	/**
	 * Start test with the specified test parameter.
	 * @param test	The test (<code>Test</code>) parameter.
	 * @see #endTest(Test)
	 */
	public void startTest(final Test test) {
		try {
			for (int i = 0; i < TestGroupRunner.this.runListeners.length; i++) {
				this.runListeners[i].testStarted(this.runId, test.toString());
			}
		} catch (final IOException e) {
			/* ignore exception */
		}
	}

	/**
	 * Start tests.
	 * @see #runTests()
	 */
	private void startTests() {
		for (int i = 0; i < this.agents.length; i++) {
			final TestAgentService agent = this.agents[i];
			final TestResult testResult = new TestResult();
			testResult.addListener(this);
			Runnable runner = new Runnable() {
				/* (non-Javadoc)
				 * @see java.lang.Runnable#run()
				 */
				public void run() {
					try {
						agent.runTest(TestGroupRunner.this.syncService, TestGroupRunner.this.properties, testResult);
					} catch (Throwable t) {
						try {
							for (int i = 0; i < TestGroupRunner.this.runListeners.length; i++) {
								Object[] parms = { t };
								TestGroupRunner.this.runListeners[i].testError(TestGroupRunner.this.runId, agent.getDescription(), Nls.format(TestController.DefaultResourceBundle.getString(Integer.toString(TestController.TEST_FAILED_WHILE_RUN)), parms));
							}
						} catch (IOException e) {
							/* Ignore (remote) failure to report event */
						}
					}
				}
			};
			this.testResults[i] = testResult;
			this.testThreads[i] = new Thread(this.testGroup, runner, "TestRunner_" + i); //$NON-NLS-1$
			this.testThreads[i].start();
		}
	}

	/**
	 * Wait for tests.
	 */
	private void waitForTests() {
		try {
			for (int i = 0; i < this.agents.length; i++) {
				this.testThreads[i].join();
			}
		} catch (final InterruptedException e) {
			for (int i = 0; i < this.testResults.length; i++) {
				this.testResults[i].stop();
			}
		}
	}
}
