package org.eclipse.soda.dk.testcontroller;

/*************************************************************************
 * Copyright (c) 2006, 2009 IBM.                                         *
 * All rights reserved. This program and the accompanying materials      *
 * are made available under the terms of the Eclipse Public License v1.0 *
 * which accompanies this distribution, and is available at              *
 * http://www.eclipse.org/legal/epl-v10.html                             *
 *                                                                       *
 * Contributors:                                                         *
 *     IBM - initial API and implementation                              *
 ************************************************************************/
import java.io.IOException;
import java.util.ArrayList;
import java.util.Collections;
import java.util.Dictionary;
import java.util.HashSet;
import java.util.Iterator;
import java.util.List;
import java.util.Set;
import org.eclipse.soda.dk.nls.Nls;
import org.eclipse.soda.dk.testagent.service.TestAgentService;
import org.eclipse.soda.dk.testagent.service.TestSynchronizationService;
import org.eclipse.soda.dk.testcontroller.service.TestConfigurationService;
import org.eclipse.soda.dk.testcontroller.service.TestRunListenerService;
import org.eclipse.soda.dk.testcontroller.service.TestSessionService;
import org.osgi.framework.InvalidSyntaxException;

/**
 * @author IBM
 * @version 1.2.0
 * @since 1.0
 */
public class TestSession implements TestSessionService {
	/**
	 * Define the test configuration service (TestConfigurationService) field.
	 */
	private TestConfigurationService testConfigurationService;

	/**
	 * Define the sync service (TestSynchronizationService) field.
	 */
	private TestSynchronizationService syncService;

	/**
	 * Define the global run listeners (List) field.
	 */
	private List globalRunListeners;

	/**
	 * Define the interrupted (boolean) field.
	 */
	private boolean interrupted;

	/**
	 * Define the test group runners (Set) field.
	 */
	private Set testGroupRunners;

	/**
	 * Constructs an instance of this class from the specified test configuration service and sync service parameters.
	 * @param testConfigurationService The test configuration service (<code>TestConfigurationService</code>) parameter.
	 * @param syncService The sync service (<code>TestSynchronizationService</code>) parameter.
	 */
	public TestSession(final TestConfigurationService testConfigurationService, final TestSynchronizationService syncService) {
		this.testConfigurationService = testConfigurationService;
		this.syncService = syncService;
		this.globalRunListeners = new ArrayList();
		this.testGroupRunners = Collections.synchronizedSet(new HashSet());
	}

	/**
	 * Add global test run listener with the specified run listener parameter.
	 * @param runListener The run listener (<code>TestRunListenerService</code>) parameter.
	 * @see #removeGlobalTestRunListener(TestRunListenerService)
	 */
	public synchronized void addGlobalTestRunListener(final TestRunListenerService runListener) {
		this.globalRunListeners.add(runListener);
	}

	/**
	 * Close.
	 */
	public void close() {
		try {
			for (final Iterator iterator = this.testGroupRunners.iterator(); iterator.hasNext();) {
				TestGroupRunner runner = (TestGroupRunner) iterator.next();
				runner.getRunnerThread().join();
			}
		} catch (final InterruptedException e) {
			/* ignore exception */
		}
		this.globalRunListeners.clear();
	}

	/**
	 * Returns a copy of the test run listeners of the session plus an optional group runner listener
	 * @param runListener The run listener (<code>TestRunListenerService</code>) parameter.
	 * @return Results of the get run listeners (<code>TestRunListenerService[]</code>) value.
	 */
	private synchronized TestRunListenerService[] getRunListeners(final TestRunListenerService runListener) {
		int size = this.globalRunListeners.size();
		TestRunListenerService listeners[] = new TestRunListenerService[runListener == null ? size : size + 1];
		this.globalRunListeners.toArray(listeners);
		if (runListener != null) {
			listeners[size] = runListener;
		}
		return listeners;
	}

	/**
	 * Gets the test configuration service value.
	 * @return The test configuration service (<code>TestConfigurationService</code>) value.
	 */
	public TestConfigurationService getTestConfigurationService() {
		return this.testConfigurationService;
	}

	/**
	 * Interrupt.
	 */
	public void interrupt() {
		this.interrupted = true;
		for (final Iterator iterator = this.testGroupRunners.iterator(); iterator.hasNext();) {
			TestGroupRunner runner = (TestGroupRunner) iterator.next();
			runner.getRunnerThread().interrupt();
		}
	}

	/**
	 * Gets the interrupted (boolean) value.
	 * @return The interrupted (<code>boolean</code>) value.
	 */
	public boolean isInterrupted() {
		return this.interrupted;
	}

	/**
	 * Remove global test run listener with the specified run listener parameter.
	 * @param runListener The run listener (<code>TestRunListenerService</code>) parameter.
	 * @see #addGlobalTestRunListener(TestRunListenerService)
	 */
	public synchronized void removeGlobalTestRunListener(final TestRunListenerService runListener) {
		this.globalRunListeners.remove(runListener);
	}

	/**
	 * Run multiple tests with the specified run id, filter, class name, properties and run listener parameters.
	 * @param actionId The action id (<code>String</code>) parameter.
	 * @param runId The run id (<code>String</code>) parameter.
	 * @param filter The filter (<code>String</code>) parameter.
	 * @param className The class name (<code>String</code>) parameter.
	 * @param properties The properties (<code>Dictionary</code>) parameter.
	 * @param runListener The run listener (<code>TestRunListenerService</code>) parameter.
	 * @throws InvalidSyntaxException Invalid Syntax Exception.
	 * @throws IOException IOException.
	 */
	public void runMultipleTests(final String actionId, final String runId, final String filter, final String className, final Dictionary properties, final TestRunListenerService runListener) throws InvalidSyntaxException, IOException {
		TestAgentService agents[] = this.testConfigurationService.findTests(filter, className);
		runTests(actionId, runId, properties, runListener, agents);
	}

	/**
	 * Run single test with the specified run id, filter, class name, properties and run listener parameters.
	 * @param actionId The action id (<code>String</code>) parameter.
	 * @param runId The run id (<code>String</code>) parameter.
	 * @param filter The filter (<code>String</code>) parameter.
	 * @param className The class name (<code>String</code>) parameter.
	 * @param properties The properties (<code>Dictionary</code>) parameter.
	 * @param runListener The run listener (<code>TestRunListenerService</code>) parameter.
	 * @throws InvalidSyntaxException Invalid Syntax Exception.
	 * @throws IOException IOException.
	 */
	public void runSingleTest(final String actionId, final String runId, final String filter, final String className, final Dictionary properties, final TestRunListenerService runListener) throws InvalidSyntaxException, IOException {
		TestAgentService agents[] = this.testConfigurationService.findTests(filter, className);
		if (agents.length == 1) {
			TestGroupRunner runner = new TestGroupRunner(actionId, runId, agents, properties, this.syncService, getRunListeners(runListener));
			runner.runTests();
		} else {
			TestRunListenerService listeners[] = getRunListeners(runListener);
			for (int i = 0; i < listeners.length; i++) {
				Object[] parms1 = { new Integer(agents.length) };
				Object[] parms2 = { filter, className };
				listeners[i].testRunStarted(actionId, runId);
				listeners[i].testRunError(runId, Nls.format(TestController.DefaultResourceBundle.getString(Integer.toString(TestController.RUN_SINGLE_REQUIRES)), parms1), Nls.format(TestController.DefaultResourceBundle.getString(Integer
						.toString(TestController.USED_FILTER)), parms2));
				listeners[i].testRunEnded(runId);
			}
		}
	}

	/**
	 * Run tests with the specified run id, properties, run listener and agents parameters.
	 * @param actionId The action id (<code>String</code>) parameter.
	 * @param runId The run id (<code>String</code>) parameter.
	 * @param properties The properties (<code>Dictionary</code>) parameter.
	 * @param runListener The run listener (<code>TestRunListenerService</code>) parameter.
	 * @param agents The agents (<code>TestAgentService[]</code>) parameter.
	 * @see #runMultipleTests(String,String,String,String,Dictionary,TestRunListenerService)
	 */
	private void runTests(final String actionId, final String runId, final Dictionary properties, final TestRunListenerService runListener, final TestAgentService[] agents) {
		TestGroupRunner runner = new TestGroupRunner(actionId, runId, agents, properties, this.syncService, getRunListeners(runListener));
		this.testGroupRunners.add(runner);
		runner.runTests();
		this.testGroupRunners.remove(runner);
	}
}
