package org.eclipse.soda.dk.testcontroller.bundle;

/*************************************************************************
 * Copyright (c) 2005, 2009 IBM.                                         *
 * All rights reserved. This program and the accompanying materials      *
 * are made available under the terms of the Eclipse Public License v1.0 *
 * which accompanies this distribution, and is available at              *
 * http://www.eclipse.org/legal/epl-v10.html                             *
 *                                                                       *
 * Contributors:                                                         *
 *     IBM - initial API and implementation                              *
 ************************************************************************/
import java.util.Hashtable;
import org.eclipse.soda.dk.core.ConfigurableObject;
import org.eclipse.soda.dk.core.EscObject;
import org.eclipse.soda.dk.testagent.service.TestAgentService;
import org.eclipse.soda.dk.testcontroller.TestController;
import org.eclipse.soda.dk.testcontroller.config.osgi.OSGiTestConfiguration;
import org.eclipse.soda.dk.testcontroller.service.TestControllerService;
import org.eclipse.soda.sat.core.framework.BaseBundleActivator;
import org.eclipse.soda.sat.core.util.FactoryUtility;
import org.osgi.framework.Bundle;
import org.osgi.framework.BundleActivator;
import org.osgi.service.cm.ConfigurationAdmin;

/**
 * The AgentBundle is the base bundle activator for all agent bundle activators.
 * @author IBM
 * @version 1.2.0
 * @since 1.0
 */
public class TestControllerBundle extends BaseBundleActivator implements BundleActivator {
	/**
	 * Defines the class name.
	 */
	public static final String CLASS_NAME = "org.eclipse.soda.dk.testcontroller.bundle.TestControllerBundle"; //$NON-NLS-1$

	/**
	 * Defines the service name(s) to be registered by the testAdapter bundle.
	 * <table>
	 * <tr>
	 * <td>org.eclipse.soda.dk.adapter.service.AdapterService</td>
	 * </tr>
	 * </table>
	 */
	public static final String[] EXPORTED_SERVICE_NAMES = new String[] { TestControllerService.SERVICE_NAME };

	/**
	 * Defines the imported service name(s) to be registered by the testAdapter
	 * bundle. <table>
	 * <tr>
	 * <td>org.eclipse.soda.dk.test.adapter.service.testAdapterService</td>
	 * </tr>
	 * <tr>
	 * <td>org.eclipse.soda.dk.notification.service.NotificationService</td>
	 * </tr>
	 * </table>
	 */
	public static final String[] IMPORTED_SERVICE_NAMES = BaseBundleActivator.NO_SERVICES;

	/**
	 * Define the optional services (String[]) constant.
	 */
	public static final String[] OPTIONAL_SERVICES = new String[] { ConfigurationAdmin.class.getName() };

	/**
	 * Define the test controller (TestControllerService) field.
	 */
	private TestController testController;

	/**
	 * Define the test configuration service (OSGiTestConfiguration) field.
	 */
	private OSGiTestConfiguration testConfiguration;

	/**
	 * Constructs an instance of this class.
	 */
	public TestControllerBundle() {
		this.testConfiguration = new OSGiTestConfiguration();
	}

	/**
	 * Create service and return the Object result.
	 */
	public void activate() {
		if (this.testController == null) {
			this.testController = createService();
			if (this.testController != null) {
				final Hashtable properties = createProperties();
				final String[] names = getExportedServiceNames();
				addExportedServices(names, this.testController, properties);
			}
		}
		FactoryUtility util = FactoryUtility.getInstance();
		this.testConfiguration.setServiceDetecter(util.createServiceDetecter(getBundleContext(), TestAgentService.SERVICE_NAME));
		final ConfigurationAdmin configAdmin = getConfigurationAdminService();
		if (configAdmin != null) {
			this.testConfiguration.setConfigurationAdmin(configAdmin);
		}
		this.testController.setTestConfigurationService(this.testConfiguration);
	}

	/**
	 * Create the properties.
	 * @return Results of the create properties (<code>Hashtable</code>)
	 * value.
	 */
	public Hashtable createProperties() {
		final Bundle bundle = getBundle();
		final Hashtable properties = ConfigurableObject.createPropertiesFromHeaders(bundle.getHeaders());
		properties.put(EscObject.ID_KEY, "local"); //$NON-NLS-1$
		return properties;
	}

	/**
	 * Create service and return the TestController result.
	 * @return Results of the create service (<code>Object</code>) value.
	 * The device (<code>DeviceService</code>) parameter.
	 * @see #handleAcquiredOptionalImportedService(String,Object)
	 * @see #handleReleasedOptionalImportedService(String,Object)
	 */
	public TestController createService() {
		return new TestController();
	}

	/**
	 * Deactivate.
	 */
	protected void deactivate() {
		this.testConfiguration.setServiceDetecter(null);
		this.testConfiguration.setConfigurationAdmin(null);
		this.testController.setTestConfigurationService(null);
		this.testController = null;
	}

	/**
	 * Gets the configuration admin service (ConfigurationAdmin) value.
	 * @return	The configuration admin service (<code>ConfigurationAdmin</code>) value.
	 */
	private ConfigurationAdmin getConfigurationAdminService() {
		return (ConfigurationAdmin) getOptionalImportedService(ConfigurationAdmin.class.getName());
	}

	/**
	 * Return the exported service name(s) to be registered by the testAdapter
	 * bundle.
	 * @return String[] The device's service names.
	 */
	public String[] getExportedServiceNames() {
		return EXPORTED_SERVICE_NAMES;
	}

	/**
	 * Return the imported service name(s) to be registered by the testAdapter
	 * bundle.
	 * @return String[] The imported service names.
	 * @see #getOptionalImportedServiceNames()
	 */
	public String[] getImportedServiceNames() {
		return IMPORTED_SERVICE_NAMES;
	}

	/**
	 * Gets the optional imported service names (String[]) value.
	 * @return	The optional imported service names (<code>String[]</code>) value.
	 */
	protected String[] getOptionalImportedServiceNames() {
		return OPTIONAL_SERVICES;
	}

	/**
	 * Handle acquired optional imported service with the specified service name and service parameters.
	 * @param serviceName	The service name (<code>String</code>) parameter.
	 * @param service	The service (<code>Object</code>) parameter.
	 */
	protected void handleAcquiredOptionalImportedService(final String serviceName, final Object service) {
		if (this.testConfiguration != null) {
			if (service instanceof ConfigurationAdmin) {
				this.testConfiguration.setConfigurationAdmin((ConfigurationAdmin) service);
			}
		}
	}

	/**
	 * Handle released optional imported service with the specified service name and service parameters.
	 * @param serviceName	The service name (<code>String</code>) parameter.
	 * @param service	The service (<code>Object</code>) parameter.
	 */
	protected void handleReleasedOptionalImportedService(final String serviceName, final Object service) {
		if (this.testConfiguration != null) {
			if (service instanceof ConfigurationAdmin) {
				this.testConfiguration.setConfigurationAdmin(null);
			}
		}
	}
}
