package org.eclipse.soda.dk.testcontroller.config.osgi;

/*************************************************************************
 * Copyright (c) 2007, 2009 IBM.                                         *
 * All rights reserved. This program and the accompanying materials      *
 * are made available under the terms of the Eclipse Public License v1.0 *
 * which accompanies this distribution, and is available at              *
 * http://www.eclipse.org/legal/epl-v10.html                             *
 *                                                                       *
 * Contributors:                                                         *
 *     IBM - initial API and implementation                              *
 ************************************************************************/
import java.io.IOException;
import java.util.Dictionary;
import java.util.Iterator;
import java.util.List;
import org.eclipse.soda.dk.nls.Nls;
import org.eclipse.soda.dk.notification.service.NotificationService;
import org.eclipse.soda.dk.testagent.service.TestAgentService;
import org.eclipse.soda.dk.testcontroller.TestController;
import org.eclipse.soda.dk.testcontroller.service.TestConfigurationService;
import org.eclipse.soda.sat.core.record.interfaces.IServiceDetecter;
import org.eclipse.soda.sat.core.util.LogUtility;
import org.osgi.framework.InvalidSyntaxException;
import org.osgi.service.cm.Configuration;
import org.osgi.service.cm.ConfigurationAdmin;

/**
 * TestLocator to find test agents from the set of optional services currently
 * acquired by this bundle.
 * @author IBM
 * @version 1.2.0
 * @since 1.0
 */
public class OSGiTestConfiguration implements TestConfigurationService {
	/**
	 * Define the no tests (TestAgentService) constant.
	 */
	private final static TestAgentService NO_TESTS[] = {};

	/**
	 * Define the service detecter (IServiceDetecter) field.
	 */
	private IServiceDetecter serviceDetecter;

	/**
	 * Define the configuration admin (ConfigurationAdmin) field.
	 */
	private ConfigurationAdmin configurationAdmin;

	/**
	 * Create factory configuration with the specified factory pid, bundle location and properties parameters.
	 * @param factoryPid	The factory pid (<code>String</code>) parameter.
	 * @param bundleLocation	The bundle location (<code>String</code>) parameter.
	 * @param properties	The properties (<code>Dictionary</code>) parameter.
	 * @throws IOException IOException.
	 */
	public void createFactoryConfiguration(final String factoryPid, final String bundleLocation, final Dictionary properties) throws IOException {
		Configuration configuration;
		synchronized (this) {
			if (this.configurationAdmin == null) {
				throwUnsupportedOperation();
				configuration = null;
			} else {
				Object[] parms = { factoryPid };
				LogUtility.logDebug(this, Nls.format(TestController.DefaultResourceBundle.getString(Integer.toString(TestController.CREATING_WITH_FACTORY_PID)), parms));
				configuration = this.configurationAdmin.createFactoryConfiguration(factoryPid, bundleLocation);
			}
		}
		if (configuration != null) {
			configuration.update(properties);
			waitForUpdate(configuration.getPid(), bundleLocation);
		}
	}

	/**
	 * Create or update configuration with the specified pid, bundle location and properties parameters.
	 * @param pid	The pid (<code>String</code>) parameter.
	 * @param bundleLocation	The bundle location (<code>String</code>) parameter.
	 * @param properties	The properties (<code>Dictionary</code>) parameter.
	 * @throws IOException IOException.
	 */
	public void createOrUpdateConfiguration(final String pid, final String bundleLocation, final Dictionary properties) throws IOException {
		Configuration configuration;
		synchronized (this) {
			if (this.configurationAdmin == null) {
				throwUnsupportedOperation();
				configuration = null;
			} else {
				Object[] parms = { pid };
				LogUtility.logDebug(this, Nls.format(TestController.DefaultResourceBundle.getString(Integer.toString(TestController.CREATING_WITH_PID)), parms));
				configuration = this.configurationAdmin.getConfiguration(pid, bundleLocation);
			}
		}
		if (configuration != null) {
			configuration.update(properties);
			waitForUpdate(configuration.getPid(), bundleLocation);
		}
	}

	/**
	 * Delete configurations with the specified filter parameter.
	 * @param filter	The filter (<code>String</code>) parameter.
	 * @throws IOException IOException.
	 * @throws InvalidSyntaxException Invalid Syntax Exception.
	 * @see #updateConfigurations(String,Dictionary)
	 */
	public void deleteConfigurations(final String filter) throws IOException, InvalidSyntaxException {
		synchronized (this) {
			if (this.configurationAdmin == null) {
				throwUnsupportedOperation();
			} else {
				Configuration[] configurations = this.configurationAdmin.listConfigurations(filter);
				if (configurations == null) {
					Object[] parms = { filter };
					LogUtility.logDebug(this, Nls.format(TestController.DefaultResourceBundle.getString(Integer.toString(TestController.NO_CONFIG_MATCH_FILTER)), parms));
				} else {
					for (int i = 0; i < configurations.length; i++) {
						try {
							Object[] parms = { configurations[i].getPid() };
							LogUtility.logDebug(this, Nls.format(TestController.DefaultResourceBundle.getString(Integer.toString(TestController.DELETING_WITH_PID)), parms));
							configurations[i].delete();
						} catch (final IOException exception) {
							Object[] parms = { configurations[i].getPid() };
							LogUtility.logError(this, Nls.format(TestController.DefaultResourceBundle.getString(Integer.toString(TestController.UNABLE_DELETING_WITH_PID)), parms), exception);
						}
					}
				}
			}
		}
	}

	/**
	 * Find tests with the specified filter and class name parameters and return the TestAgentService[] result.
	 * @param filter	The filter (<code>String</code>) parameter.
	 * @param className	The class name (<code>String</code>) parameter.
	 * @return	Results of the find tests (<code>TestAgentService[]</code>) value.
	 * @throws InvalidSyntaxException Invalid Syntax Exception.
	 */
	public TestAgentService[] findTests(final String filter, final String className) throws InvalidSyntaxException {
		List aquiredTests = null;
		int count = 10;
		try {
			do {
				this.serviceDetecter.setFilter(filter);
				aquiredTests = this.serviceDetecter.getServices();
				if (aquiredTests.size() == 0) {
					synchronized (this) {
						wait(500);
					}
				}
			} while (aquiredTests.size() == 0 && count-- > 0);
		} catch (final InterruptedException e) {
			/* ignore exception */
		}
		if (aquiredTests == null || aquiredTests.size() == 0) {
			Object[] parms = { filter };
			LogUtility.logWarning(this, Nls.format(TestController.DefaultResourceBundle.getString(Integer.toString(TestController.NO_TEST_MATCH_FILTER)), parms));
			return NO_TESTS;
		}
		if (className != null) {
			for (final Iterator iter = aquiredTests.iterator(); iter.hasNext();) {
				Object test = iter.next();
				if (!test.getClass().getName().equals(className)) {
					iter.remove();
				}
			}
			if (aquiredTests.size() == 0) {
				Object[] parms = { filter, className };
				LogUtility.logWarning(this, Nls.format(TestController.DefaultResourceBundle.getString(Integer.toString(TestController.NO_TEST_MATCH_FILTER_CLASS)), parms));
				return NO_TESTS;
			}
		}
		TestAgentService result[] = new TestAgentService[aquiredTests.size()];
		aquiredTests.toArray(result);
		return result;
	}

	/**
	 * the configurationAdmin to set
	 * @param configurationAdmin	The configuration admin (<code>ConfigurationAdmin</code>) parameter.
	 */
	public synchronized void setConfigurationAdmin(final ConfigurationAdmin configurationAdmin) {
		this.configurationAdmin = configurationAdmin;
	}

	/**
	 * Sets the notification service value.
	 * @param notificationService	The notification service (<code>NotificationService</code>) parameter.
	 */
	public void setNotificationService(final NotificationService notificationService) {
		/* do nothing */
	}

	/**
	 * the serviceDetecter to set
	 * @param serviceDetecter	The service detecter (<code>IServiceDetecter</code>) parameter.
	 */
	public void setServiceDetecter(final IServiceDetecter serviceDetecter) {
		if (this.serviceDetecter != null) {
			this.serviceDetecter.release();
		}
		this.serviceDetecter = serviceDetecter;
		if (serviceDetecter != null) {
			serviceDetecter.acquire();
		}
	}

	/**
	 * Throw unsupported operation.
	 * @throws IOException IOException.
	 */
	private void throwUnsupportedOperation() throws IOException {
		throw new IOException(TestController.DefaultResourceBundle.getString(Integer.toString(TestController.NO_OSGI_SUPPORT)));
	}

	/**
	 * Update configurations with the specified filter and properties parameters.
	 * @param filter	The filter (<code>String</code>) parameter.
	 * @param properties	The properties (<code>Dictionary</code>) parameter.
	 * @throws IOException IOException.
	 * @throws InvalidSyntaxException Invalid Syntax Exception.
	 * @see #deleteConfigurations(String)
	 */
	public void updateConfigurations(final String filter, final Dictionary properties) throws IOException, InvalidSyntaxException {
		Configuration[] configurations;
		synchronized (this) {
			if (this.configurationAdmin == null) {
				throwUnsupportedOperation();
				configurations = null;
			} else {
				configurations = this.configurationAdmin.listConfigurations(filter);
			}
		}
		if (configurations != null) {
			for (int i = 0; i < configurations.length; i++) {
				Object[] parms = { configurations[i].getPid() };
				LogUtility.logDebug(this, Nls.format(TestController.DefaultResourceBundle.getString(Integer.toString(TestController.UPDATING_WITH_PID)), parms));
				configurations[i].update(properties);
				waitForUpdate(configurations[i].getPid(), configurations[i].getBundleLocation());
			}
		}
	}

	/**
	 * Wait for update with the specified pid and bundle location parameters.
	 * @param pid	The pid (<code>String</code>) parameter.
	 * @param bundleLocation	The bundle location (<code>String</code>) parameter.
	 * @throws IOException IOException.
	 */
	private void waitForUpdate(final String pid, final String bundleLocation) throws IOException {
		Thread.yield();
		Thread.yield();
		Thread.yield();
	}
}
