package org.eclipse.soda.dk.testcontroller.console;

/*************************************************************************
 * Copyright (c) 2007, 2009 IBM.                                         *
 * All rights reserved. This program and the accompanying materials      *
 * are made available under the terms of the Eclipse Public License v1.0 *
 * which accompanies this distribution, and is available at              *
 * http://www.eclipse.org/legal/epl-v10.html                             *
 *                                                                       *
 * Contributors:                                                         *
 *     IBM - initial API and implementation                              *
 ************************************************************************/
import java.io.PrintStream;
import org.eclipse.soda.dk.nls.Nls;
import org.eclipse.soda.dk.testagent.ConsoleSynchronization;
import org.eclipse.soda.dk.testcontroller.TestController;
import org.eclipse.soda.dk.testcontroller.service.TestRunListenerService;

/**
 * @author IBM
 * @version 1.2.0
 * @since 1.0
 */
public class TestControllerConsole extends ConsoleSynchronization implements TestRunListenerService {
	/**
	 * Print error event with the specified message and trace parameters.
	 * @param message	The message (<code>String</code>) parameter.
	 * @param trace	The trace (<code>String</code>) parameter.
	 */
	protected void printErrorEvent(final String message, final String trace) {
		printEvent(true, message, trace);
	}

	/**
	 * Print event with the specified report line parameter.
	 * @param reportLine	The report line (<code>String</code>) parameter.
	 * @see #printErrorEvent(String,String)
	 * @see #printEvent(boolean,String,String)
	 */
	protected void printEvent(final String reportLine) {
		printEvent(false, reportLine, null);
	}

	/**
	 * Print event with the specified error, message and detail parameters.
	 * @param error	The error (<code>boolean</code>) parameter.
	 * @param message	The message (<code>String</code>) parameter.
	 * @param detail	The detail (<code>String</code>) parameter.
	 * @see #printErrorEvent(String,String)
	 * @see #printEvent(String)
	 */
	protected synchronized void printEvent(final boolean error, final String message, final String detail) {
		PrintStream out = error ? System.err : System.out;
		String timestamp = Nls.formatTimestamp(System.currentTimeMillis());
		out.print(timestamp);
		out.print(' ');
		out.println(message);
		if (detail != null) {
			out.println(detail);
		}
		out.flush();
	}

	/**
	 * Test ended with the specified run id and test name parameters.
	 * @param runId	The run id (<code>String</code>) parameter.
	 * @param testName	The test name (<code>String</code>) parameter.
	 * @see #testRunEnded(String)
	 */
	public void testEnded(final String runId, final String testName) {
		/* do nothing */
	}

	/**
	 * Test error with the specified run id, test name and trace parameters.
	 * @param runId	The run id (<code>String</code>) parameter.
	 * @param testName	The test name (<code>String</code>) parameter.
	 * @param trace	The trace (<code>String</code>) parameter.
	 * @see #testRunError(String,String,String)
	 */
	public void testError(final String runId, final String testName, final String trace) {
		Object[] parms = { testName, runId };
		printErrorEvent(Nls.format(TestController.DefaultResourceBundle.getString(Integer.toString(TestController.ERROR_IN_TEST)), parms), trace);
	}

	/**
	 * Test failed with the specified run id, test name and trace parameters.
	 * @param runId	The run id (<code>String</code>) parameter.
	 * @param testName	The test name (<code>String</code>) parameter.
	 * @param trace	The trace (<code>String</code>) parameter.
	 */
	public void testFailed(final String runId, final String testName, final String trace) {
		Object[] parms = { testName, runId };
		printErrorEvent(Nls.format(TestController.DefaultResourceBundle.getString(Integer.toString(TestController.TEST_FAILED)), parms), trace);
	}

	/**
	 * Test run ended with the specified run id parameter.
	 * @param runId	The run id (<code>String</code>) parameter.
	 */
	public void testRunEnded(final String runId) {
		/* do nothing */
	}

	/**
	 * Test run error with the specified run id, message and detail parameters.
	 * @param runId	The run id (<code>String</code>) parameter.
	 * @param message	The message (<code>String</code>) parameter.
	 * @param detail	The detail (<code>String</code>) parameter.
	 */
	public void testRunError(final String runId, final String message, final String detail) {
		Object[] parms = { message, runId };
		printErrorEvent(Nls.format(TestController.DefaultResourceBundle.getString(Integer.toString(TestController.ERROR_IN_TEST_RUN)), parms), detail);
	}

	/**
	 * Test run started with the specified run id parameter.
	 * @param actionId The action id (<code>String</code>) parameter.
	 * @param runId	The run id (<code>String</code>) parameter.
	 */
	public void testRunStarted(final String actionId, final String runId) {
		/* do nothing */
	}

	/**
	 * Test started with the specified run id and test name parameters.
	 * @param runId	The run id (<code>String</code>) parameter.
	 * @param testName	The test name (<code>String</code>) parameter.
	 * @see #testRunStarted(String,String)
	 */
	public void testStarted(final String runId, final String testName) {
		Object[] parms = { testName, runId };
		printEvent(Nls.format(TestController.DefaultResourceBundle.getString(Integer.toString(TestController.RUNNING)), parms));
	}
}
