package org.eclipse.soda.dk.testcontroller.managed;

/*************************************************************************
 * Copyright (c) 1999, 2009 IBM.                                         *
 * All rights reserved. This program and the accompanying materials      *
 * are made available under the terms of the Eclipse Public License v1.0 *
 * which accompanies this distribution, and is available at              *
 * http://www.eclipse.org/legal/epl-v10.html                             *
 *                                                                       *
 * Contributors:                                                         *
 *     IBM - initial API and implementation                              *
 ************************************************************************/
import java.util.Dictionary;
import java.util.Enumeration;
import java.util.Hashtable;
import org.eclipse.soda.dk.core.ConfigurableObject;
import org.eclipse.soda.dk.testagent.service.TestAgentService;
import org.eclipse.soda.dk.testcontroller.TestController;
import org.eclipse.soda.dk.testcontroller.config.osgi.OSGiTestConfiguration;
import org.eclipse.soda.dk.testcontroller.service.TestControllerService;
import org.eclipse.soda.sat.core.framework.BaseBundleActivator;
import org.eclipse.soda.sat.core.util.FactoryUtility;
import org.osgi.framework.Bundle;
import org.osgi.service.cm.ConfigurationAdmin;
import org.osgi.service.cm.ConfigurationException;
import org.osgi.service.cm.ManagedService;

/**
 * The DeviceBundle is the standard Bundle Activator for all other transport
 * bundle activators to subclass.
 * @author IBM
 * @version 1.2.0
 * @since 1.0
 */
public class TestControllerManaged extends BaseBundleActivator implements ManagedService {
	/**
	 * Defines the class name.
	 */
	public static final String CLASS_NAME = "org.eclipse.soda.dk.testcontroller.managed.TestControllerManaged"; //$NON-NLS-1$

	/**
	 * Defines the service name(s) to be registered by the testAdapter bundle.
	 * <table>
	 * <tr>
	 * <td>org.eclipse.soda.dk.adapter.service.AdapterService</td>
	 * </tr>
	 * </table>
	 */
	public static final String[] EXPORTED_SERVICE_NAMES = new String[] { TestControllerService.SERVICE_NAME };

	/**
	 * Defines the imported service name(s) to be registered by the testAdapter
	 * bundle. <table>
	 * <tr>
	 * <td>org.eclipse.soda.dk.test.adapter.service.testAdapterService</td>
	 * </tr>
	 * <tr>
	 * <td>org.eclipse.soda.dk.notification.service.NotificationService</td>
	 * </tr>
	 * </table>
	 */
	public static final String[] IMPORTED_SERVICE_NAMES = new String[] { ConfigurationAdmin.class.getName() };

	/**
	 * Define the controller (TestController) field.
	 */
	private TestController testController;

	/**
	 * Define the test configuration service (OSGiTestConfiguration) field.
	 */
	private OSGiTestConfiguration testConfigurationService;

	/**
	 * Constructs an instance of this class.
	 */
	public TestControllerManaged() {
		this.testConfigurationService = new OSGiTestConfiguration();
	}

	/**
	 * Create exported services.
	 */
	protected void activate() {
		final Hashtable properties = createManagedProperties();
		FactoryUtility util = FactoryUtility.getInstance();
		this.testConfigurationService.setServiceDetecter(util.createServiceDetecter(getBundleContext(), TestAgentService.SERVICE_NAME));
		this.testConfigurationService.setConfigurationAdmin(getConfigurationAdminService());
		addExportedServices(new String[] { ManagedService.class.getName(), getClass().getName() }, this, properties);
	}

	/**
	 * Create managed properties and return the Hashtable result.
	 * @return Results of the create managed properties (<code>Hashtable</code>)
	 * value.
	 */
	public Hashtable createManagedProperties() {
		final Hashtable properties = createProperties();
		properties.put(org.osgi.framework.Constants.SERVICE_PID, getClass().getName());
		return properties;
	}

	/**
	 * Create the properties.
	 * @return Results of the create properties (<code>Hashtable</code>)
	 * value.
	 * @see #createManagedProperties()
	 * @see #updateProperties(Dictionary,Dictionary)
	 */
	public Hashtable createProperties() {
		final Bundle bundle = getBundle();
		final Hashtable properties = ConfigurableObject.createPropertiesFromHeaders(bundle.getHeaders());
		return properties;
	}

	/**
	 * Create the service.
	 * @return Results of the create service (<code>Object</code>) value.
	 */
	public TestController createService() {
		return new TestController();
	}

	/**
	 * Deactivate.
	 */
	protected void deactivate() {
		this.testConfigurationService.setServiceDetecter(null);
		this.testConfigurationService.setConfigurationAdmin(null);
		this.testController = null;
	}

	/**
	 * Gets the configuration admin service (ConfigurationAdmin) value.
	 * @return	The configuration admin service (<code>ConfigurationAdmin</code>) value.
	 */
	private ConfigurationAdmin getConfigurationAdminService() {
		return (ConfigurationAdmin) getImportedService(ConfigurationAdmin.class.getName());
	}

	/**
	 * Return the exported service name(s) to be registered by the testAdapter
	 * bundle.
	 * @return String[] The device's service names.
	 */
	public String[] getExportedServiceNames() {
		return EXPORTED_SERVICE_NAMES;
	}

	/**
	 * Return the imported service name(s) to be registered by the testAdapter
	 * bundle.
	 * @return String[] The imported service names.
	 */
	public String[] getImportedServiceNames() {
		return IMPORTED_SERVICE_NAMES;
	}

	/**
	 * Update properties with the specified table and from parameters.
	 * @param table	The table (<code>Dictionary</code>) parameter.
	 * @param from	The from (<code>Dictionary</code>) parameter.
	 * @see #createManagedProperties()
	 * @see #createProperties()
	 */
	public void updateProperties(final Dictionary table, final Dictionary from) {
		if (from != null) {
			final Enumeration enumeration = from.keys();
			while (enumeration.hasMoreElements()) {
				final Object key = enumeration.nextElement();
				final Object value = from.get(key);
				table.put(key, value);
			}
		}
	}

	/**
	 * Updated with the specified properties parameter.
	 * The properties (<code>Dictionary</code>) parameter.
	 * Configuration Exception.
	 * @param properties	The properties (<code>Dictionary</code>) parameter.
	 * @throws ConfigurationException Configuration Exception.
	 */
	public void updated(final Dictionary properties) throws ConfigurationException {
		if (this.testController != null) {
			removeExportedService(TestControllerService.SERVICE_NAME);
			this.testController = null;
		}
		this.testController = createService();
		this.testController.setConfigurationInformation(properties);
		this.testController.setTestConfigurationService(this.testConfigurationService);
		final String[] names = getExportedServiceNames();
		final Hashtable serviceProperties = createProperties();
		updateProperties(serviceProperties, properties);
		addExportedServices(names, this.testController, serviceProperties);
	}
}
