package org.eclipse.soda.dk.testcontroller.remote.rmi;

/*************************************************************************
 * Copyright (c) 2007, 2009 IBM.                                         *
 * All rights reserved. This program and the accompanying materials      *
 * are made available under the terms of the Eclipse Public License v1.0 *
 * which accompanies this distribution, and is available at              *
 * http://www.eclipse.org/legal/epl-v10.html                             *
 *                                                                       *
 * Contributors:                                                         *
 *     IBM - initial API and implementation                              *
 ************************************************************************/
import java.io.IOException;
import java.rmi.server.UnicastRemoteObject;
import java.util.Dictionary;
import org.eclipse.soda.dk.testcontroller.service.TestConfigurationService;
import org.eclipse.soda.dk.testcontroller.service.TestRunListenerService;
import org.eclipse.soda.dk.testcontroller.service.TestSessionService;
import org.osgi.framework.InvalidSyntaxException;

/**
 * @author IBM
 * @version 1.2.0
 */
public class TestSessionSkeleton extends UnicastRemoteObject implements LocalTestSession, RemoteTestSession {
	/**
	 * Define the serial version uid (long) constant.
	 */
	private static final long serialVersionUID = 4188753784229563055L;

	/**
	 * Define the delegate (TestSessionService) field.
	 */
	private TestSessionService delegate;

	/**
	 * Define the wrapped configuration service (TestConfigurationService) field.
	 */
	private TestConfigurationService wrappedConfigurationService;

	/**
	 * Constructs an instance of this class from the specified delegate parameter.
	 * @param delegate	The delegate (<code>TestSessionService</code>) parameter.
	 * @throws IOException IOException.
	 */
	public TestSessionSkeleton(final TestSessionService delegate) throws IOException {
		this.delegate = delegate;
		this.wrappedConfigurationService = new TestConfigurationSkeleton(delegate.getTestConfigurationService());
	}

	/**
	 * Add global test run listener with the specified global run listener parameter.
	 * @param globalRunListener	The global run listener (<code>TestRunListenerService</code>) parameter.
	 * @throws IOException IOException.
	 * @see #addGlobalTestRunListener(LocalTestRunListener)
	 * @see #removeGlobalTestRunListener(TestRunListenerService)
	 * @see #removeGlobalTestRunListener(LocalTestRunListener)
	 */
	public void addGlobalTestRunListener(final TestRunListenerService globalRunListener) throws IOException {
		this.delegate.addGlobalTestRunListener(globalRunListener);
	}

	/**
	 * Add global test run listener with the specified global run listener parameter.
	 * @param globalRunListener The global run listener (<code>LocalTestRunListener</code>) parameter.
	 * @throws IOException IOException.
	 * @see #addGlobalTestRunListener(TestRunListenerService)
	 * @see #removeGlobalTestRunListener(TestRunListenerService)
	 * @see #removeGlobalTestRunListener(LocalTestRunListener)
	 */
	public void addGlobalTestRunListener(final LocalTestRunListener globalRunListener) throws IOException {
		this.delegate.addGlobalTestRunListener(globalRunListener);
	}

	/**
	 * Close.
	 * @throws IOException IOException.
	 */
	public void close() throws IOException {
		this.delegate.close();
	}

	/**
	 * Gets the test configuration service value.
	 * @return	The test configuration service (<code>TestConfigurationService</code>) value.
	 * @throws IOException IOException.
	 */
	public TestConfigurationService getTestConfigurationService() throws IOException {
		return this.wrappedConfigurationService;
	}

	/**
	 * Interrupt.
	 * @throws IOException IOException.
	 */
	public void interrupt() throws IOException {
		this.delegate.interrupt();
	}

	/**
	 * Gets the interrupted (boolean) value.
	 * @return	The interrupted (<code>boolean</code>) value.
	 * @throws IOException IOException.
	 */
	public boolean isInterrupted() throws IOException {
		return this.delegate.isInterrupted();
	}

	/**
	 * Remove global test run listener with the specified global run listener parameter.
	 * @param globalRunListener	The global run listener (<code>TestRunListenerService</code>) parameter.
	 * @throws IOException IOException.
	 * @see #addGlobalTestRunListener(TestRunListenerService)
	 * @see #addGlobalTestRunListener(LocalTestRunListener)
	 * @see #removeGlobalTestRunListener(LocalTestRunListener)
	 */
	public void removeGlobalTestRunListener(final TestRunListenerService globalRunListener) throws IOException {
		this.delegate.removeGlobalTestRunListener(globalRunListener);
	}

	/**
	 * Remove global test run listener with the specified global run listener parameter.
	 * @param globalRunListener The global run listener (<code>LocalTestRunListener</code>) parameter.
	 * @throws IOException IOException.
	 * @see #addGlobalTestRunListener(TestRunListenerService)
	 * @see #addGlobalTestRunListener(LocalTestRunListener)
	 * @see #removeGlobalTestRunListener(TestRunListenerService)
	 */
	public void removeGlobalTestRunListener(final LocalTestRunListener globalRunListener) throws IOException {
		this.delegate.removeGlobalTestRunListener(globalRunListener);
	}

	/**
	 * Run multiple tests with the specified run id, filter, class name, properties and run listener parameters.
	 * @param actionId
	 *		The action ID (<code>String</code>) parameter.
	 * @param runId	The run id (<code>String</code>) parameter.
	 * @param filter	The filter (<code>String</code>) parameter.
	 * @param className	The class name (<code>String</code>) parameter.
	 * @param properties	The properties (<code>Dictionary</code>) parameter.
	 * @param runListener	The run listener (<code>TestRunListenerService</code>) parameter.
	 * @throws InvalidSyntaxException Invalid Syntax Exception.
	 * @throws IOException IOException.
	 * @see #runMultipleTests(String,String,String,String,Dictionary,LocalTestRunListener)
	 */
	public void runMultipleTests(final String actionId, final String runId, final String filter, final String className, final Dictionary properties, final TestRunListenerService runListener) throws InvalidSyntaxException, IOException {
		this.delegate.runMultipleTests(actionId, runId, filter, className, properties, runListener);
	}

	/**
	 * Run multiple tests with the specified run id, filter, class name, properties and run listener parameters.
	 * @param actionId
	 *		The action ID (<code>String</code>) parameter.
	 * @param runId The run id (<code>String</code>) parameter.
	 * @param filter The filter (<code>String</code>) parameter.
	 * @param className The class name (<code>String</code>) parameter.
	 * @param properties The properties (<code>Dictionary</code>) parameter.
	 * @param runListener The run listener (<code>LocalTestRunListener</code>) parameter.
	 * @throws InvalidSyntaxException Invalid Syntax Exception.
	 * @throws IOException IOException.
	 * @see #runMultipleTests(String,String,String,String,Dictionary,TestRunListenerService)
	 */
	public void runMultipleTests(final String actionId, final String runId, final String filter, final String className, final Dictionary properties, final LocalTestRunListener runListener) throws InvalidSyntaxException, IOException {
		this.delegate.runMultipleTests(actionId, runId, filter, className, properties, runListener);
	}

	/**
	 * Run single test with the specified run id, filter, class name, properties and run listener parameters.
	 * @param actionId
	 *		The action ID (<code>String</code>) parameter.
	 * @param runId	The run id (<code>String</code>) parameter.
	 * @param filter	The filter (<code>String</code>) parameter.
	 * @param className	The class name (<code>String</code>) parameter.
	 * @param properties	The properties (<code>Dictionary</code>) parameter.
	 * @param runListener	The run listener (<code>TestRunListenerService</code>) parameter.
	 * @throws InvalidSyntaxException Invalid Syntax Exception.
	 * @throws IOException IOException.
	 * @see #runSingleTest(String,String,String,String,Dictionary,LocalTestRunListener)
	 */
	public void runSingleTest(final String actionId, final String runId, final String filter, final String className, final Dictionary properties, final TestRunListenerService runListener) throws InvalidSyntaxException, IOException {
		this.delegate.runSingleTest(actionId, runId, filter, className, properties, runListener);
	}

	/**
	 * Run single test with the specified run id, filter, class name, properties and run listener parameters.
	 * @param actionId
	 *		The action ID (<code>String</code>) parameter.
	 * @param runId The run id (<code>String</code>) parameter.
	 * @param filter The filter (<code>String</code>) parameter.
	 * @param className The class name (<code>String</code>) parameter.
	 * @param properties The properties (<code>Dictionary</code>) parameter.
	 * @param runListener The run listener (<code>LocalTestRunListener</code>) parameter.
	 * @throws InvalidSyntaxException Invalid Syntax Exception.
	 * @throws IOException IOException.
	 * @see #runSingleTest(String,String,String,String,Dictionary,TestRunListenerService)
	 */
	public void runSingleTest(final String actionId, final String runId, final String filter, final String className, final Dictionary properties, final LocalTestRunListener runListener) throws InvalidSyntaxException, IOException {
		this.delegate.runSingleTest(actionId, runId, filter, className, properties, runListener);
	}
}
