package org.eclipse.soda.dk.testcontroller.remote.rmi.bundle;

/*************************************************************************
 * Copyright (c) 2005, 2009 IBM.                                         *
 * All rights reserved. This program and the accompanying materials      *
 * are made available under the terms of the Eclipse Public License v1.0 *
 * which accompanies this distribution, and is available at              *
 * http://www.eclipse.org/legal/epl-v10.html                             *
 *                                                                       *
 * Contributors:                                                         *
 *     IBM - initial API and implementation                              *
 ************************************************************************/
import java.rmi.server.UnicastRemoteObject;
import java.util.Hashtable;
import org.eclipse.soda.dk.core.EscConfiguration;
import org.eclipse.soda.dk.core.service.ConfigurationService;
import org.eclipse.soda.dk.nls.Nls;
import org.eclipse.soda.dk.testcontroller.TestController;
import org.eclipse.soda.dk.testcontroller.remote.rmi.ControllerRegistry;
import org.eclipse.soda.dk.testcontroller.remote.rmi.LocateController;
import org.eclipse.soda.dk.testcontroller.remote.rmi.RemoteTestController;
import org.eclipse.soda.dk.testcontroller.remote.rmi.TestControllerSkeleton;
import org.eclipse.soda.dk.testcontroller.service.TestControllerService;
import org.eclipse.soda.sat.core.framework.BaseBundleActivator;
import org.eclipse.soda.sat.core.util.LogUtility;
import org.osgi.framework.BundleActivator;

/**
 * The AgentBundle is the base bundle activator for all agent bundle activators.
 * @author IBM
 * @version 1.2.0
 * @since 1.0
 */
public class RemoteTestControllerBundle extends BaseBundleActivator implements BundleActivator {
	/**
	 * Defines the class name.
	 */
	public static final String CLASS_NAME = "org.eclipse.soda.dk.testcontroller.remote.rmi.bundle.RemoteTestControllerBundle"; //$NON-NLS-1$

	/**
	 * Defines the service name(s) to be registered by the testAdapter bundle.
	 * <table>
	 * <tr>
	 * <td>org.eclipse.soda.dk.adapter.service.AdapterService</td>
	 * </tr>
	 * </table>
	 */
	public static final String[] EXPORTED_SERVICE_NAMES = BaseBundleActivator.NO_SERVICES;

	/**
	 * Defines the imported service name(s) to be registered by the testAdapter
	 * bundle. <table>
	 * <tr>
	 * <td>org.eclipse.soda.dk.test.adapter.service.testAdapterService</td>
	 * </tr>
	 * <tr>
	 * <td>org.eclipse.soda.dk.notification.service.NotificationService</td>
	 * </tr>
	 * </table>
	 */
	public static final String[] IMPORTED_SERVICE_NAMES = new String[] { TestControllerService.SERVICE_NAME };

	/**
	 * Define the configuration (ConfigurationService) field.
	 */
	private ConfigurationService configuration;

	/**
	 * Define the controller registry (ControllerRegistry) field.
	 */
	private ControllerRegistry controllerRegistry;

	/**
	 * Constructs an instance of this class.
	 */
	public RemoteTestControllerBundle() {
		this.configuration = EscConfiguration.getConfiguration();
	}

	/**
	 * Create service and return the Object result.
	 */
	public void activate() {
		int port = this.configuration.getInt("testcontroller.registry.port", ControllerRegistry.REGISTRY_PORT); //$NON-NLS-1$
		try {
			this.controllerRegistry = LocateController.createRegistry(port);
			this.controllerRegistry.bind(new TestControllerSkeleton(getTestControllerService()));
		} catch (Exception e) {
			Object parms[] = { Integer.toString(port), e.getMessage() };
			LogUtility.logError(Nls.format(LocateController.DefaultResourceBundle.getString(Integer.toString(LocateController.FAILED_TO_BIND_CONTROLLER)), parms));
		}
	}

	/**
	 * Create the properties.
	 * @return Results of the create properties (<code>Hashtable</code>)
	 * value.
	 */
	public Hashtable createProperties() {
		final Hashtable properties = new Hashtable(37);
		return properties;
	}

	/**
	 * Create service and return the TestController result.
	 * @return Results of the create service (<code>Object</code>) value.
	 * The device (<code>DeviceService</code>) parameter.
	 */
	public TestController createService() {
		return new TestController();
	}

	/**
	 * Deactivate.
	 */
	protected void deactivate() {
		try {
			if (this.controllerRegistry != null) {
				RemoteTestController remoteTestController = this.controllerRegistry.getTestController();
				this.controllerRegistry.unbind();
				this.controllerRegistry = null;
				if (remoteTestController != null) {
					UnicastRemoteObject.unexportObject(remoteTestController, true);
				}
			}
		} catch (final Exception e) {
			LogUtility.logError(Nls.format(LocateController.DefaultResourceBundle.getString(Integer.toString(LocateController.FAILED_TO_UNBIND_CONTROLLER)), e.getMessage()));
		}
	}

	/**
	 * Return the exported service name(s) to be registered by the testAdapter
	 * bundle.
	 * @return String[] The device's service names.
	 */
	public String[] getExportedServiceNames() {
		return EXPORTED_SERVICE_NAMES;
	}

	/**
	 * Return the imported service name(s) to be registered by the testAdapter
	 * bundle.
	 * @return String[] The imported service names.
	 */
	public String[] getImportedServiceNames() {
		return IMPORTED_SERVICE_NAMES;
	}

	/**
	 * Gets the configuration admin service (ConfigurationAdmin) value.
	 * @return	The configuration admin service (<code>ConfigurationAdmin</code>) value.
	 */
	private TestControllerService getTestControllerService() {
		return (TestControllerService) getImportedService(TestControllerService.class.getName());
	}

	/**
	 * Start.
	 * @throws Exception Exception.
	 */
	protected void start() throws Exception {
		addImportedServiceFilter(TestControllerService.SERVICE_NAME, "(!(type=remote))"); //$NON-NLS-1$
	}

	/**
	 * Stop.
	 * @throws Exception Exception.
	 */
	protected void stop() throws Exception {
		removeImportedServiceFilter(TestControllerService.SERVICE_NAME);
	}
}
