package org.eclipse.soda.dk.testcontroller.remote.rmi.factory;

/*************************************************************************
 * Copyright (c) 2005, 2009 IBM.                                         *
 * All rights reserved. This program and the accompanying materials      *
 * are made available under the terms of the Eclipse Public License v1.0 *
 * which accompanies this distribution, and is available at              *
 * http://www.eclipse.org/legal/epl-v10.html                             *
 *                                                                       *
 * Contributors:                                                         *
 *     IBM - initial API and implementation                              *
 ************************************************************************/
import java.util.Dictionary;
import java.util.Enumeration;
import java.util.Hashtable;
import org.eclipse.soda.dk.core.ConfigurableObject;
import org.eclipse.soda.dk.core.EscConfiguration;
import org.eclipse.soda.dk.testcontroller.remote.rmi.ControllerRegistry;
import org.eclipse.soda.dk.testcontroller.remote.rmi.LocateController;
import org.eclipse.soda.dk.testcontroller.remote.rmi.RemoteControllerLocater;
import org.eclipse.soda.dk.testcontroller.service.ControllerLocaterService;
import org.eclipse.soda.dk.testcontroller.service.TestControllerService;
import org.eclipse.soda.sat.core.framework.BaseBundleActivator;
import org.eclipse.soda.sat.core.framework.ManagedServiceFactoryBundleActivator;
import org.eclipse.soda.sat.core.framework.interfaces.IBundleActivationManager;
import org.eclipse.soda.sat.core.framework.interfaces.IManagedServiceFactoryAdvisor;
import org.osgi.service.cm.ConfigurationException;
import org.osgi.service.cm.ManagedServiceFactory;
import org.osgi.service.metatype.MetaTypeInformation;
import org.osgi.service.metatype.MetaTypeService;

/**
 * The AgentBundle is the base bundle activator for all agent bundle activators.
 * @author IBM
 * @version 1.2.0
 * @since 1.0
 */
public class RemoteTestControllerFactory extends ManagedServiceFactoryBundleActivator implements ManagedServiceFactory, IManagedServiceFactoryAdvisor {
	/**
	 * Defines the class name.
	 */
	public static final String CLASS_NAME = "org.eclipse.soda.dk.testcontroller.remote.rmi.factory.RemoteTestControllerFactory"; //$NON-NLS-1$

	/**
	 * Define the metatype service (String) constant.
	 */
	public static final String METATYPE_SERVICE = MetaTypeService.class.getName();

	/**
	 * Defines the service name(s) to be registered by the testAdapter bundle.
	 * <table>
	 * <tr>
	 * <td>org.eclipse.soda.dk.adapter.service.AdapterService</td>
	 * </tr>
	 * </table>
	 */
	public static final String[] EXPORTED_SERVICE_NAMES = new String[] { TestControllerService.SERVICE_NAME };

	/**
	 * Defines the imported service name(s) to be registered by the testAdapter
	 * bundle. <table>
	 * <tr>
	 * <td>org.eclipse.soda.dk.test.adapter.service.testAdapterService</td>
	 * </tr>
	 * <tr>
	 * <td>org.eclipse.soda.dk.notification.service.NotificationService</td>
	 * </tr>
	 * </table>
	 */
	public static final String[] IMPORTED_SERVICE_NAMES = BaseBundleActivator.NO_SERVICES;

	/**
	 * Define the optional services (String[]) constant.
	 */
	public static final String[] OPTIONAL_SERVICES = new String[] { METATYPE_SERVICE };

	/**
	 * Define the controller locater (RemoteControllerLocater) field.
	 */
	private RemoteControllerLocater controllerLocater;

	/**
	 * Activate.
	 */
	protected void activate() {
		super.activate();
		this.controllerLocater = new RemoteControllerLocater();
		addExportedService(ControllerLocaterService.SERVICE_NAME, this.controllerLocater, createProperties());
	}

	/**
	 * Create with the specified pid, properties and manager parameters and
	 * return the Object result. The pid (<code>String</code>) parameter.
	 * The properties (<code>Dictionary</code>) parameter. The manager (<code>IBundleActivationManager</code>)
	 * parameter.
	 * @param pid The pid (<code>String</code>) parameter.
	 * @param properties The properties (<code>Dictionary</code>) parameter.
	 * @param manager The manager (<code>IBundleActivationManager</code>) parameter.
	 * @return Results of the create (<code>Object</code>) value.
	 * The pid (<code>String</code>) parameter.
	 * The properties (<code>Dictionary</code>) parameter.
	 * The manager (<code>IBundleActivationManager</code>)
	 * parameter.
	 */
	public Object create(final String pid, final Dictionary properties, final IBundleActivationManager manager) {
		EscConfiguration config = new EscConfiguration(properties);
		String host = config.getString("host", null); //$NON-NLS-1$
		int port = config.getInt("port", ControllerRegistry.REGISTRY_PORT); //$NON-NLS-1$
		TestControllerService remoteTestController;
		remoteTestController = LocateController.getController(host, port);
		if (remoteTestController != null) {
			final Hashtable serviceProperties = createProperties();
			updateProperties(serviceProperties, properties);
			manager.addExportedServices(getExportedServiceNames(pid, new Hashtable(), properties, manager), remoteTestController, serviceProperties);
		}
		return remoteTestController;
	}

	/**
	 * @see org.eclipse.soda.sat.core.framework.ManagedServiceFactoryBundleActivator#createAdvisor()
	 * @return Results of the create advisor (<code>IManagedServiceFactoryAdvisor</code>)
	 * value.
	 */
	protected IManagedServiceFactoryAdvisor createAdvisor() {
		return this;
	}

	/**
	 * Create imported service filters with the specified pid, old properties,
	 * properties and manager parameters. The pid (<code>String</code>)
	 * parameter. The old properties (<code>Dictionary</code>) parameter.
	 * The properties (<code>Dictionary</code>) parameter. The manager (<code>IBundleActivationManager</code>)
	 * parameter.
	 * The pid (<code>String</code>) parameter.
	 * The old properties (<code>Dictionary</code>) parameter.
	 * The properties (<code>Dictionary</code>) parameter.
	 * The manager (<code>IBundleActivationManager</code>)
	 * parameter.
	 * @param pid The pid (<code>String</code>) parameter.
	 * @param oldProperties The old properties (<code>Dictionary</code>) parameter.
	 * @param properties The properties (<code>Dictionary</code>) parameter.
	 * @param manager The manager (<code>IBundleActivationManager</code>) parameter.
	 */
	public void createImportedServiceFilters(final String pid, final Dictionary oldProperties, final Dictionary properties, final IBundleActivationManager manager) {
		/* do nothing */
	}

	/**
	 * Gets the properties to be registered with the service(s).
	 * @return Hashtable of properties. <table width=100%>
	 * <tr>
	 * <td width=50%>org.eclipse.soda.dk.connection.bundle.ServiceBundle.DEVICE_CLASS</td>
	 * <td width=50%>testAdapter</td>
	 * </tr>
	 * <tr>
	 * <td width=50%>org.osgi.framework.Constants.SERVICE_PID</td>
	 * <td width=50%>testAdapterService.SERVICE_NAME</td>
	 * </tr>
	 * <tr>
	 * <td width=50%>org.osgi.framework.Constants.SERVICE_DESCRIPTION</td>
	 * <td width=50%>test Adapter</td>
	 * </tr>
	 * <tr>
	 * <td width=50%>org.osgi.framework.Constants.SERVICE_VENDOR</td>
	 * <td width=50%>Eclipse</td>
	 * </tr>
	 * <tr>
	 * <td width=50%>org.osgi.service.device.Constants.DRIVER_ID</td>
	 * <td width=50%>testAdapter</td>
	 * </tr>
	 * <tr>
	 * <td width=50%>org.osgi.framework.Constants.SERVICE_RANKING</td>
	 * <td width=50%>6</td>
	 * </tr>
	 * <tr>
	 * <td width=50%>org.osgi.service.device.Constants.DEVICE_CATEGORY</td>
	 * <td width=50%>DeviceService</td>
	 * </tr>
	 * <tr>
	 * <td width=50%>org.eclipse.soda.dk.connection.bundle.ServiceBundle.DEVICE_REVISION</td>
	 * <td width=50%>1.0.0</td>
	 * </tr>
	 * </table>
	 * @see #updateProperties(Dictionary,Dictionary)
	 */
	public Hashtable createProperties() {
		final Hashtable properties = new Hashtable(37);
		properties.put("type", "remote"); //$NON-NLS-1$ //$NON-NLS-2$
		return properties;
	}

	/**
	 * Deactivate.
	 */
	protected void deactivate() {
		removeExportedService(ControllerLocaterService.SERVICE_NAME, this.controllerLocater);
		this.controllerLocater = null;
		super.deactivate();
	}

	/**
	 * Destroy with the specified pid, object, properties and manager
	 * parameters. The pid (<code>String</code>) parameter. The object (<code>Object</code>)
	 * parameter. The properties (<code>Dictionary</code>) parameter. The
	 * manager (<code>IBundleActivationManager</code>) parameter.
	 * The pid (<code>String</code>) parameter.
	 * The object (<code>Object</code>) parameter.
	 * The properties (<code>Dictionary</code>) parameter.
	 * The manager (<code>IBundleActivationManager</code>)
	 * parameter.
	 * @param pid The pid (<code>String</code>) parameter.
	 * @param object The object (<code>Object</code>) parameter.
	 * @param properties The properties (<code>Dictionary</code>) parameter.
	 * @param manager The manager (<code>IBundleActivationManager</code>) parameter.
	 */
	public void destroy(final String pid, final Object object, final Dictionary properties, final IBundleActivationManager manager) {
		/* do nothing */
	}

	/**
	 * Return the exported service name(s) to be registered by the testAdapter
	 * bundle.
	 * @param pid The pid (<code>String</code>) parameter.
	 * @param oldProperties The old properties (<code>Dictionary</code>) parameter.
	 * @param properties The properties (<code>Dictionary</code>) parameter.
	 * @param manager The manager (<code>IBundleActivationManager</code>) parameter.
	 * @return String[] The device's service names.
	 * The pid (<code>String</code>) parameter.
	 * The old properties (<code>Dictionary</code>) parameter.
	 * The properties (<code>Dictionary</code>) parameter.
	 * The manager (<code>IBundleActivationManager</code>)
	 * parameter.
	 */
	public String[] getExportedServiceNames(final String pid, final Dictionary oldProperties, final Dictionary properties, final IBundleActivationManager manager) {
		return EXPORTED_SERVICE_NAMES;
	}

	/**
	 * Return the imported service name(s) to be registered by the testAdapter
	 * bundle.
	 * @param pid The pid (<code>String</code>) parameter.
	 * @param oldProperties The old properties (<code>Dictionary</code>) parameter.
	 * @param properties The properties (<code>Dictionary</code>) parameter.
	 * @param manager The manager (<code>IBundleActivationManager</code>) parameter.
	 * @return String[] The imported service names.
	 * The pid (<code>String</code>) parameter.
	 * The old properties (<code>Dictionary</code>) parameter.
	 * The properties (<code>Dictionary</code>) parameter.
	 * The manager (<code>IBundleActivationManager</code>)
	 * parameter.
	 * @see #getOptionalImportedServiceNames()
	 * @see #getOptionalImportedServiceNames(String,Dictionary,Dictionary,IBundleActivationManager)
	 */
	public String[] getImportedServiceNames(final String pid, final Dictionary oldProperties, final Dictionary properties, final IBundleActivationManager manager) {
		return IMPORTED_SERVICE_NAMES;
	}

	/**
	 * Gets the optional imported service names (String[]) value.
	 * @return The optional imported service names (<code>String[]</code>)
	 * value.
	 * @see #getOptionalImportedServiceNames(String,Dictionary,Dictionary,IBundleActivationManager)
	 */
	protected String[] getOptionalImportedServiceNames() {
		return OPTIONAL_SERVICES;
	}

	/**
	 * Get optional imported service names with the specified arg0, arg1, arg2
	 * and arg3 parameters and return the String[] result. The arg0 (<code>String</code>)
	 * parameter. The arg1 (<code>Dictionary</code>) parameter. The arg2 (<code>Dictionary</code>)
	 * parameter. The arg3 (<code>IBundleActivationManager</code>)
	 * parameter.
	 * @param arg0 The arg0 (<code>String</code>) parameter.
	 * @param arg1 The arg1 (<code>Dictionary</code>) parameter.
	 * @param arg2 The arg2 (<code>Dictionary</code>) parameter.
	 * @param arg3 The arg3 (<code>IBundleActivationManager</code>) parameter.
	 * @return Results of the get optional imported service names (<code>String[]</code>)
	 * value.
	 * The arg0 (<code>String</code>) parameter.
	 * The arg1 (<code>Dictionary</code>) parameter.
	 * The arg2 (<code>Dictionary</code>) parameter.
	 * The arg3 (<code>IBundleActivationManager</code>)
	 * parameter.
	 * @see #getOptionalImportedServiceNames()
	 */
	public String[] getOptionalImportedServiceNames(final String arg0, final Dictionary arg1, final Dictionary arg2, final IBundleActivationManager arg3) {
		return NO_SERVICES;
	}

	/**
	 * Handle acquired optional imported service with the specified pid, object,
	 * imported service name, imported service and manager parameters. The pid (<code>String</code>)
	 * parameter. The object (<code>Object</code>) parameter. The imported
	 * service name (<code>String</code>) parameter. The imported service (<code>Object</code>)
	 * parameter. The manager (<code>IBundleActivationManager</code>)
	 * parameter.
	 * The pid (<code>String</code>) parameter.
	 * The object (<code>Object</code>) parameter.
	 * The imported service name (<code>String</code>) parameter.
	 * The imported service (<code>Object</code>) parameter.
	 * The manager (<code>IBundleActivationManager</code>)
	 * parameter.
	 * @param pid The pid (<code>String</code>) parameter.
	 * @param object The object (<code>Object</code>) parameter.
	 * @param importedServiceName The imported service name (<code>String</code>) parameter.
	 * @param importedService The imported service (<code>Object</code>) parameter.
	 * @param manager The manager (<code>IBundleActivationManager</code>) parameter.
	 */
	public void handleAcquiredOptionalImportedService(final String pid, final Object object, final String importedServiceName, final Object importedService, final IBundleActivationManager manager) {
		/* do nothing */
	}

	/**
	 * Handle released optional imported service with the specified pid, object,
	 * imported service name, imported service and manager parameters. The pid (<code>String</code>)
	 * parameter. The object (<code>Object</code>) parameter. The imported
	 * service name (<code>String</code>) parameter. The imported service (<code>Object</code>)
	 * parameter. The manager (<code>IBundleActivationManager</code>)
	 * parameter.
	 * The pid (<code>String</code>) parameter.
	 * The object (<code>Object</code>) parameter.
	 * The imported service name (<code>String</code>) parameter.
	 * The imported service (<code>Object</code>) parameter.
	 * The manager (<code>IBundleActivationManager</code>)
	 * parameter.
	 * @param pid The pid (<code>String</code>) parameter.
	 * @param object The object (<code>Object</code>) parameter.
	 * @param importedServiceName The imported service name (<code>String</code>) parameter.
	 * @param importedService The imported service (<code>Object</code>) parameter.
	 * @param manager The manager (<code>IBundleActivationManager</code>) parameter.
	 */
	public void handleReleasedOptionalImportedService(final String pid, final Object object, final String importedServiceName, final Object importedService, final IBundleActivationManager manager) {
		/* do nothing */
	}

	/**
	 * Update with the specified pid, object, old properties, properties and
	 * manager parameters and return the Object result. The pid (<code>String</code>)
	 * parameter. The object (<code>Object</code>) parameter. The old
	 * properties (<code>Dictionary</code>) parameter. The properties (<code>Dictionary</code>)
	 * parameter. The manager (<code>IBundleActivationManager</code>)
	 * parameter.
	 * @param pid The pid (<code>String</code>) parameter.
	 * @param object The object (<code>Object</code>) parameter.
	 * @param oldProperties The old properties (<code>Dictionary</code>) parameter.
	 * @param properties The properties (<code>Dictionary</code>) parameter.
	 * @param manager The manager (<code>IBundleActivationManager</code>) parameter.
	 * @return Results of the update (<code>Object</code>) value.
	 * The pid (<code>String</code>) parameter.
	 * The object (<code>Object</code>) parameter.
	 * The old properties (<code>Dictionary</code>) parameter.
	 * The properties (<code>Dictionary</code>) parameter.
	 * The manager (<code>IBundleActivationManager</code>)
	 * parameter.
	 */
	public Object update(final String pid, final Object object, final Dictionary oldProperties, final Dictionary properties, final IBundleActivationManager manager) {
		final Hashtable serviceProperties = createProperties();
		updateProperties(serviceProperties, properties);
		manager.setExportedServiceProperties(TestControllerService.SERVICE_NAME, serviceProperties);
		return object;
	}

	/**
	 * Update properties with the specified table and from parameters.
	 * @param table The table (<code>Dictionary</code>) parameter.
	 * @param from The from (<code>Dictionary</code>) parameter.
	 * @see #createProperties()
	 */
	public void updateProperties(final Dictionary table, final Dictionary from) {
		final Enumeration enumeration = from.keys();
		while (enumeration.hasMoreElements()) {
			final Object key = enumeration.nextElement();
			final Object value = from.get(key);
			table.put(key, value);
		}
	}

	/**
	 * Validate configuration with the specified pid and properties parameters.
	 * The pid (<code>String</code>) parameter.
	 * The properties (<code>Dictionary</code>) parameter.
	 * Configuration Exception.
	 * @param pid The pid (<code>String</code>) parameter.
	 * @param properties The properties (<code>Dictionary</code>) parameter.
	 * @throws ConfigurationException Configuration Exception.
	 */
	public void validateConfiguration(final String pid, final Dictionary properties) throws ConfigurationException {
		final Object service = getOptionalImportedService(METATYPE_SERVICE);
		if (service instanceof MetaTypeService) {
			final MetaTypeService metatypeService = (MetaTypeService) service;
			final MetaTypeInformation info = metatypeService.getMetaTypeInformation(getBundle());
			ConfigurableObject.validateConfiguration(pid, properties, info);
		}
	}
}
