package org.eclipse.soda.dk.testmanager;

/*************************************************************************
 * Copyright (c) 2007, 2009 IBM.                                         *
 * All rights reserved. This program and the accompanying materials      *
 * are made available under the terms of the Eclipse Public License v1.0 *
 * which accompanies this distribution, and is available at              *
 * http://www.eclipse.org/legal/epl-v10.html                             *
 *                                                                       *
 * Contributors:                                                         *
 *     IBM - initial API and implementation                              *
 ************************************************************************/
import java.io.IOException;
import java.util.Hashtable;
import java.util.Iterator;
import java.util.List;
import java.util.Map;
import org.eclipse.soda.dk.testagent.service.TestSynchronizationService;
import org.eclipse.soda.dk.testcontroller.service.ControllerLocaterService;
import org.eclipse.soda.dk.testcontroller.service.TestControllerService;
import org.eclipse.soda.dk.testcontroller.service.TestRunListenerService;
import org.eclipse.soda.dk.testcontroller.service.TestSessionService;
import org.eclipse.soda.dk.testmanager.service.DistributedTestSessionService;

/**
 * @author IBM
 * @version 1.2.0
 * @since 1.0
 */
public class DistributedTestSession implements DistributedTestSessionService {
	/**
	 * Define the test sessions (Map) field.
	 */
	private Map testSessions;

	/**
	 * Define the remote controller locaters (List) field.
	 */
	private List remoteControllerLocaters;

	/**
	 * Define the sync service (TestSynchronizationService) field.
	 */
	private TestSynchronizationService syncService;

	/**
	 * Define the global run listener (TestRunListenerService) field.
	 */
	private TestRunListenerService globalRunListener;

	/**
	 * Define the interrupted (boolean) field.
	 */
	private boolean interrupted;

	/**
	 * Define the local controller locater (ControllerLocaterService) field.
	 */
	private ControllerLocaterService localControllerLocater;

	/**
	 * Constructs an instance of this class from the specified local controller locater, remote controller locaters, sync service and global run listener parameters.
	 * @param localControllerLocater
	 *		The local controller locater (<code>ControllerLocaterService</code>) parameter.
	 * @param remoteControllerLocaters
	 *		The remote controller locaters (<code>List</code>) parameter.
	 * @param syncService
	 *		The sync service (<code>TestSynchronizationService</code>) parameter.
	 * @param globalRunListener
	 *		The global run listener (<code>TestRunListenerService</code>) parameter.
	 */
	public DistributedTestSession(final ControllerLocaterService localControllerLocater, final List remoteControllerLocaters, final TestSynchronizationService syncService, final TestRunListenerService globalRunListener) {
		this.localControllerLocater = localControllerLocater;
		this.remoteControllerLocaters = remoteControllerLocaters;
		this.syncService = syncService;
		this.globalRunListener = globalRunListener;
		this.testSessions = new Hashtable();
	}

	/**
	 * Close.
	 */
	public void close() {
		for (final Iterator iter = this.testSessions.values().iterator(); iter.hasNext();) {
			final TestSessionService testSession = (TestSessionService) iter.next();
			try {
				testSession.close();
			} catch (final IOException e) {
				/* ignore exception */
			}
			iter.remove();
		}
	}

	/**
	 * Get test session with the specified controller id parameter and return the TestSessionService result.
	 * @param controllerId
	 *		The controller ID (<code>String</code>) parameter.
	 * @return Results of the get test session (<code>TestSessionService</code>) value.
	 * @throws IOException IOException.
	 * @see #DistributedTestSession(ControllerLocaterService,List,TestSynchronizationService,TestRunListenerService)
	 */
	public synchronized TestSessionService getTestSession(final String controllerId) throws IOException {
		final String nonNullControllerId = String.valueOf(controllerId);
		TestSessionService testSession = (TestSessionService) this.testSessions.get(nonNullControllerId);
		if (testSession == null) {
			TestControllerService testController = null;
			for (final Iterator iterator = this.remoteControllerLocaters.iterator(); (testController == null) && iterator.hasNext();) {
				final ControllerLocaterService controllerLocater = (ControllerLocaterService) iterator.next();
				testController = controllerLocater.findController(controllerId);
			}
			if (testController == null) {
				testController = this.localControllerLocater.findController(controllerId);
			}
			if (testController != null) {
				testSession = testController.createTestSession(this.syncService);
				testSession.addGlobalTestRunListener(this.globalRunListener);
				this.testSessions.put(nonNullControllerId, testSession);
				return testSession;
			}
		}
		return testSession;
	}

	/**
	 * Interrupt.
	 */
	public void interrupt() {
		for (final Iterator iter = this.testSessions.values().iterator(); iter.hasNext();) {
			final TestSessionService testSession = (TestSessionService) iter.next();
			try {
				testSession.interrupt();
			} catch (final IOException e) {
				/* ignore exception */
			}
		}
		this.interrupted = true;
	}

	/**
	 * Gets the interrupted (boolean) value.
	 * @return The interrupted (<code>boolean</code>) value.
	 */
	public boolean isInterrupted() {
		return this.interrupted;
	}
}
