package org.eclipse.soda.dk.testmanager;

/*************************************************************************
 * Copyright (c) 2006, 2009 IBM.                                         *
 * All rights reserved. This program and the accompanying materials      *
 * are made available under the terms of the Eclipse Public License v1.0 *
 * which accompanies this distribution, and is available at              *
 * http://www.eclipse.org/legal/epl-v10.html                             *
 *                                                                       *
 * Contributors:                                                         *
 *     IBM - initial API and implementation                              *
 ************************************************************************/
import java.io.IOException;
import java.io.PrintWriter;
import java.io.StringWriter;
import java.util.ArrayList;
import java.util.Hashtable;
import java.util.Iterator;
import java.util.List;
import java.util.Map;
import org.eclipse.soda.dk.testmanager.action.AbstractAction;
import org.eclipse.soda.dk.testmanager.action.Action;
import org.eclipse.soda.dk.testmanager.service.DistributedTestSessionService;
import org.eclipse.soda.dk.testmanager.service.ScriptRunListenerService;
import org.eclipse.soda.dk.testmanager.service.TestScriptService;
import org.eclipse.soda.dk.testmanager.service.TestStatisticsService;
import org.eclipse.soda.sat.core.util.LogUtility;

/**
 * @author IBM
 * @version 1.2.0
 * @since 1.0
 */
public class TestScript implements TestScriptService, ActionContainer, Runnable {
	/**
	 * Define the script listener (ScriptRunListenerService) field.
	 */
	private ScriptRunListenerService scriptListener;

	/**
	 * Define the attributes (Map) field.
	 */
	private Map attributes;

	/**
	 * Define the test statistics (TestStatisticsService) field.
	 */
	private TestStatisticsService testStatistics;

	/**
	 * Define the actions (List) field.
	 */
	private List actions;

	/**
	 * Define the distributed test session (DistributedTestSessionService) field.
	 */
	private DistributedTestSessionService distributedTestSession;

	/**
	 * Constructs an instance of this class.
	 */
	public TestScript() {
		this.actions = new ArrayList();
		this.attributes = new Hashtable();
	}

	/**
	 * Add with the specified action parameter.
	 * @param action
	 *		The action (<code>Action</code>) parameter.
	 */
	public void add(final Action action) {
		this.actions.add(action);
		this.testStatistics = null;
	}

	/**
	 * Execute.
	 * @throws InterruptedException Interrupted Exception.
	 */
	private void execute() throws InterruptedException {
		try {
			this.scriptListener.scriptRunStarted(this);
			for (final Iterator iter = this.actions.iterator(); iter.hasNext();) {
				AbstractAction.execute(this, (Action) iter.next());
			}
		} catch (final RuntimeException e) {
			LogUtility.logError(this, TestManager.DefaultResourceBundle.getString(Integer.toString(TestManager.ERROR_RUNNING_SCRIPT)), e);
			final StringWriter writer = new StringWriter();
			e.printStackTrace(new PrintWriter(writer));
			getTestStatistics().incrementCount(TestStatisticsService.STATS_SCRIPT_ERROR_COUNT, 1);
			this.scriptListener.scriptRunError(e.toString(), writer.toString());
			throw e;
		} finally {
			this.distributedTestSession.close();
			this.scriptListener.scriptRunEnded(getTestStatistics());
		}
	}

	/**
	 * Gets the attributes (Map) value.
	 * @return The attributes (<code>Map</code>) value.
	 */
	public Map getAttributes() {
		return this.attributes;
	}

	/**
	 * Gets the distributed test session (DistributedTestSessionService) value.
	 * @return The distributed test session (<code>DistributedTestSessionService</code>) value.
	 */
	public DistributedTestSessionService getDistributedTestSession() {
		return this.distributedTestSession;
	}

	/**
	 * Gets the script listener (ScriptRunListenerService) value.
	 * @return The script listener (<code>ScriptRunListenerService</code>) value.
	 */
	public ScriptRunListenerService getScriptListener() {
		return this.scriptListener;
	}

	/**
	 * Gets the test statistics (TestStatisticsService) value.
	 * @return The test statistics (<code>Map</code>) value.
	 */
	public synchronized TestStatisticsService getTestStatistics() {
		if (this.testStatistics == null) {
			this.testStatistics = new TestStatistics();
			for (final Iterator iterator = this.actions.iterator(); iterator.hasNext();) {
				final Action action = (Action) iterator.next();
				action.addCounts(this.testStatistics);
			}
		}
		return this.testStatistics;
	}

	/**
	 * Run with the specified distributed test session, script listener and asynchronous parameters.
	 * @param distributedTestSession
	 *		The distributed test session (<code>DistributedTestSessionService</code>) parameter.
	 * @param scriptListener
	 *		The script listener (<code>ScriptRunListenerService</code>) parameter.
	 * @param asynchronous
	 *		The asynchronous (<code>boolean</code>) parameter.
	 * @throws InterruptedException Interrupted Exception.
	 * @throws IOException IOException.
	 * @see #run()
	 */
	public void run(final DistributedTestSessionService distributedTestSession, final ScriptRunListenerService scriptListener, final boolean asynchronous) throws InterruptedException, IOException {
		this.distributedTestSession = distributedTestSession;
		this.scriptListener = scriptListener;
		if (asynchronous) {
			new Thread(this, TestManager.DefaultResourceBundle.getString(Integer.toString(TestManager.TEST_SCRIPT_RUNNER))).start();
		} else {
			execute();
		}
	}

	/**
	 * Run.
	 * @see #run(DistributedTestSessionService,ScriptRunListenerService,boolean)
	 */
	public void run() {
		try {
			execute();
		} catch (final InterruptedException e) {
			/* ignore exception */
		}
	}
}
