package org.eclipse.soda.dk.testmanager.action;

/*************************************************************************
 * Copyright (c) 2007, 2009 IBM.                                         *
 * All rights reserved. This program and the accompanying materials      *
 * are made available under the terms of the Eclipse Public License v1.0 *
 * which accompanies this distribution, and is available at              *
 * http://www.eclipse.org/legal/epl-v10.html                             *
 *                                                                       *
 * Contributors:                                                         *
 *     IBM - initial API and implementation                              *
 ************************************************************************/
import org.eclipse.soda.dk.nls.Nls;
import org.eclipse.soda.dk.testmanager.TestManager;
import org.eclipse.soda.dk.testmanager.service.ScriptRunListenerService;
import org.eclipse.soda.dk.testmanager.service.TestScriptService;

/**
 * @author IBM
 * @version 1.2.0
 * @since 1.0
 */
public abstract class AbstractAction implements Action {
	/**
	 * Define the id (String) field.
	 */
	private String id;

	/**
	 * Define the parent action (Action) field.
	 */
	private Action parentAction;

	/**
	 * Constructs an instance of this class from the specified id parameter.
	 * @param id
	 *		The ID (<code>String</code>) parameter.
	 */
	protected AbstractAction(final String id) {
		this.id = id;
	}

	/**
	 * Execute with the specified test script and action parameters.
	 * @param testScript
	 *		The test script (<code>TestScriptService</code>) parameter.
	 * @param action
	 *		The action (<code>Action</code>) parameter.
	 * @throws InterruptedException Interrupted Exception.
	 */
	public static void execute(final TestScriptService testScript, final Action action) throws InterruptedException {
		final ScriptRunListenerService scriptListener = testScript.getScriptListener();
		final Action parentAction = action.getParentAction();
		final String parentActionId = parentAction == null ? null : parentAction.getId();
		scriptListener.scriptActionStarted(parentActionId, action.getId(), action.getDescription());
		try {
			action.execute(testScript);
		} finally {
			scriptListener.scriptActionEnded(action.getId(), action.getDescription());
		}
	}

	/**
	 * Gets the ID (String) value.
	 * @return The id (<code>String</code>) value.
	 */
	public String getId() {
		return this.id;
	}

	/**
	 * Gets the parent action (Action) value.
	 * @return The parent action (<code>Action</code>) value.
	 * @see #setParentAction(Action)
	 */
	public Action getParentAction() {
		return this.parentAction;
	}

	/**
	 * Handle script action error
	 * @param testScript
	 *		The test script (<code>TestScriptService</code>) parameter.
	 * @param detail
	 *		The detail (<code>String</code>) parameter.
	 */
	protected void handleScriptActionError(final TestScriptService testScript, final String detail) {
		final Object[] parms = { getDescription() };
		testScript.getScriptListener().scriptActionError(getId(), Nls.format(TestManager.DefaultResourceBundle.getString(Integer.toString(TestManager.CANNOT_PROCESS)), parms), detail);
	}

	/**
	 * Sets the parent action value.
	 * @param parentAction
	 *		The parent action (<code>Action</code>) parameter.
	 * @see #getParentAction()
	 */
	public void setParentAction(final Action parentAction) {
		this.parentAction = parentAction;
	}
}
