package org.eclipse.soda.dk.testmanager.action;

/*************************************************************************
 * Copyright (c) 2007, 2009 IBM.                                         *
 * All rights reserved. This program and the accompanying materials      *
 * are made available under the terms of the Eclipse Public License v1.0 *
 * which accompanies this distribution, and is available at              *
 * http://www.eclipse.org/legal/epl-v10.html                             *
 *                                                                       *
 * Contributors:                                                         *
 *     IBM - initial API and implementation                              *
 ************************************************************************/
import java.io.IOException;
import java.util.ArrayList;
import java.util.Dictionary;
import java.util.Iterator;
import java.util.List;
import org.eclipse.soda.dk.nls.Nls;
import org.eclipse.soda.dk.testcontroller.service.TestSessionService;
import org.eclipse.soda.dk.testmanager.TestManager;
import org.eclipse.soda.dk.testmanager.service.TestScriptService;
import org.eclipse.soda.dk.testmanager.service.TestStatisticsService;
import org.eclipse.soda.sat.core.util.LogUtility;
import org.osgi.framework.InvalidSyntaxException;

/**
 * @author IBM
 * @version 1.2.0
 * @since 1.0
 */
public abstract class ConfigurationAction extends AbstractAction implements Action {
	/**
	 * Define the configuration templates (List) field.
	 */
	private List configurationTemplates;

	/**
	 * Constructs an instance of this class.
	 * @param id
	 *		The ID (<code>String</code>) parameter.
	 */
	protected ConfigurationAction(final String id) {
		super(id);
		this.configurationTemplates = new ArrayList();
	}

	/**
	 * Add configuration with the specified id, controller id, pid, factory pid, bundle location, filter and properties parameters.
	 * @param id
	 *		The ID (<code>String</code>) parameter.
	 * @param controllerId
	 *		The controller ID (<code>String</code>) parameter.
	 * @param pid
	 *		The PID (<code>String</code>) parameter.
	 * @param factoryPid
	 *		The factory PID (<code>String</code>) parameter.
	 * @param bundleLocation
	 *		The bundle location (<code>String</code>) parameter.
	 * @param filter
	 *		The filter (<code>String</code>) parameter.
	 * @param properties
	 *		The properties (<code>Dictionary</code>) parameter.
	 */
	public void addConfiguration(final String id, final String controllerId, final String pid, final String factoryPid, final String bundleLocation, final String filter, final Dictionary properties) {
		final ConfigurationTemplate template = new ConfigurationTemplate(id, controllerId, pid, factoryPid, bundleLocation, filter, properties);
		this.configurationTemplates.add(template);
	}

	/**
	 * Add counts with the specified test script parameter.
	 * @param testStatistics
	 *		The test statistics (<code>TestStatisticsService</code>) parameter.
	 */
	public void addCounts(final TestStatisticsService testStatistics) {
		testStatistics.incrementCount(TestStatisticsService.STATS_CONFIGURATION_COUNT, getConfigurationTemplates().size());
	}

	/**
	 * Execute with the specified test script parameter.
	 * @param testScript
	 *		The test script (<code>TestScriptService</code>) parameter.
	 * @see #execute(TestScriptService,ConfigurationTemplate)
	 */
	public void execute(final TestScriptService testScript) {
		final TestStatisticsService testStatistics = testScript.getTestStatistics();
		for (final Iterator iterator = getConfigurationTemplates().iterator(); iterator.hasNext();) {
			final ConfigurationTemplate template = (ConfigurationTemplate) iterator.next();
			try {
				execute(testScript, template);
			} catch (final InvalidSyntaxException exception) {
				final String filter = template.getFilter();
				final Object[] parms = { filter };
				LogUtility.logError(Nls.format(TestManager.DefaultResourceBundle.getString(Integer.toString(TestManager.INVALID_FILTER)), parms));
				testScript.getScriptListener().scriptActionError(getId(), getDescription(), Nls.format(TestManager.DefaultResourceBundle.getString(Integer.toString(TestManager.INVALID_FILTER)), parms));
				testStatistics.incrementCount(TestStatisticsService.STATS_CONFIGURATION_ERROR_COUNT, 1);
			} catch (final IOException exception) {
				LogUtility.logError(this, TestManager.DefaultResourceBundle.getString(Integer.toString(TestManager.COMMUNICATION_ERROR)), exception);
				testScript.getScriptListener().scriptActionError(getId(), getDescription(), TestManager.DefaultResourceBundle.getString(Integer.toString(TestManager.COMMUNICATION_ERROR)));
				testStatistics.incrementCount(TestStatisticsService.STATS_CONFIGURATION_ERROR_COUNT, 1);
			}
		}
	}

	/**
	 * Execute with the specified test script and template parameters.
	 * @param testScript
	 *		The test script (<code>TestScriptService</code>) parameter.
	 * @param template
	 *		The template (<code>ConfigurationTemplate</code>) parameter.
	 * @throws InvalidSyntaxException Invalid Syntax Exception.
	 * @throws IOException IOException.
	 * @see #execute(TestScriptService)
	 */
	protected abstract void execute(final TestScriptService testScript, final ConfigurationTemplate template) throws InvalidSyntaxException, IOException;

	/**
	 * Gets the configuration templates (List) value.
	 * @return The configuration templates (<code>List</code>) value.
	 */
	protected List getConfigurationTemplates() {
		return this.configurationTemplates;
	}

	/**
	 * Gets the description (String) value.
	 * @return The description (<code>String</code>) value.
	 */
	public String getDescription() {
		final Object[] parms = { getName() };
		return Nls.format(TestManager.DefaultResourceBundle.getString(Integer.toString(TestManager.CONFIGURATION_REQUEST)), parms);
	}

	/**
	 * Gets the name (String) value.
	 * @return The name (<code>String</code>) value.
	 */
	protected abstract String getName();

	/**
	 * Get test session with the specified test script and controller id parameters and return the TestSessionService result.
	 * @param testScript
	 *		The test script (<code>TestScriptService</code>) parameter.
	 * @param controllerId
	 *		The controller ID (<code>String</code>) parameter.
	 * @return Results of the get test session (<code>TestSessionService</code>) value.
	 * @throws IOException IOException.
	 */
	protected TestSessionService getTestSession(final TestScriptService testScript, final String controllerId) throws IOException {
		final TestSessionService testSession = testScript.getDistributedTestSession().getTestSession(controllerId);
		if (testSession == null) {
			String detail;
			if (controllerId == null) {
				detail = TestManager.DefaultResourceBundle.getString(Integer.toString(TestManager.LOCAL_CONTROLLER_NOT_FOUND));
			} else {
				detail = Nls.format(TestManager.DefaultResourceBundle.getString(Integer.toString(TestManager.REMOTE_CONTROLLER_NOT_FOUND)), controllerId);
			}
			handleScriptActionError(testScript, detail);
		}
		return testSession;
	}
}
