package org.eclipse.soda.dk.testmanager.action;

/*************************************************************************
 * Copyright (c) 2007, 2009 IBM.                                         *
 * All rights reserved. This program and the accompanying materials      *
 * are made available under the terms of the Eclipse Public License v1.0 *
 * which accompanies this distribution, and is available at              *
 * http://www.eclipse.org/legal/epl-v10.html                             *
 *                                                                       *
 * Contributors:                                                         *
 *     IBM - initial API and implementation                              *
 ************************************************************************/
import java.io.IOException;
import java.util.ArrayList;
import java.util.Dictionary;
import java.util.Iterator;
import java.util.List;
import org.eclipse.soda.dk.nls.Nls;
import org.eclipse.soda.dk.testcontroller.service.TestSessionService;
import org.eclipse.soda.dk.testmanager.ActionContainer;
import org.eclipse.soda.dk.testmanager.TestManager;
import org.eclipse.soda.dk.testmanager.service.TestScriptService;
import org.eclipse.soda.dk.testmanager.service.TestStatisticsService;
import org.eclipse.soda.sat.core.util.LogUtility;
import org.osgi.framework.InvalidSyntaxException;

/**
 * @author IBM
 * @version 1.2.0
 * @since 1.0
 */
public abstract class TestAction extends AbstractAction implements ActionContainer {
	/**
	 * Define the test templates and actions (List) field.
	 */
	private List testTemplatesAndActions;

	/**
	 * Constructs an instance of this class.
	 * @param id
	 *		The ID (<code>String</code>) parameter.
	 */
	protected TestAction(final String id) {
		super(id);
		this.testTemplatesAndActions = new ArrayList();
	}

	/**
	 * Add with the specified action parameter.
	 * @param action
	 *		The action (<code>Action</code>) parameter.
	 */
	public void add(final Action action) {
		this.testTemplatesAndActions.add(action);
		action.setParentAction(this);
	}

	/**
	 * Add counts with the specified test script parameter.
	 * @param testStatistics
	 *		The test statistics (<code>TestStatisticsService</code>) parameter.
	 */
	public void addCounts(final TestStatisticsService testStatistics) {
		for (final Iterator iterator = getTestTemplatesAndActions().iterator(); iterator.hasNext();) {
			final Object templateOrAction = iterator.next();
			if (templateOrAction instanceof TestTemplate) {
				testStatistics.incrementCount(TestStatisticsService.STATS_TEST_COUNT, 1);
			} else {
				final Action action = (Action) templateOrAction;
				action.addCounts(testStatistics);
			}
		}
	}

	/**
	 * Add test with the specified run id, filter, class name, match and properties parameters.
	 * @param id
	 *		The ID (<code>String</code>) parameter.
	 * @param controllerId
	 *		The controller ID (<code>String</code>) parameter.
	 * @param filter
	 *		The filter (<code>String</code>) parameter.
	 * @param className
	 *		The class name (<code>String</code>) parameter.
	 * @param match
	 *		The match (<code>String</code>) parameter.
	 * @param properties
	 *		The properties (<code>Dictionary</code>) parameter.
	 */
	public void addTest(final String id, final String controllerId, final String filter, final String className, final String match, final Dictionary properties) {
		final TestTemplate template = new TestTemplate(this, id, controllerId, filter, className, match, properties);
		this.testTemplatesAndActions.add(template);
	}

	/**
	 * Execute with the specified test script and template parameters and return the int result.
	 * @param testScript
	 *		The test script (<code>TestScriptService</code>) parameter.
	 * @param template
	 *		The template (<code>TestTemplate</code>) parameter.
	 * @throws InvalidSyntaxException Invalid Syntax Exception.
	 * @throws IOException IOException.
	 * @throws InterruptedException Interrupted Exception.
	 */
	protected abstract void execute(final TestScriptService testScript, final TestTemplate template) throws InvalidSyntaxException, IOException, InterruptedException;

	/**
	 * Execute template or action with the specified test script and template or action parameters.
	 * @param testScript
	 *		The test script (<code>TestScriptService</code>) parameter.
	 * @param templateOrAction
	 *		The template or action (<code>Object</code>) parameter.
	 * @throws InterruptedException Interrupted Exception.
	 */
	protected void executeTemplateOrAction(final TestScriptService testScript, final Object templateOrAction) throws InterruptedException {
		final TestStatisticsService testStatistics = testScript.getTestStatistics();
		if (templateOrAction instanceof TestTemplate) {
			final TestTemplate template = (TestTemplate) templateOrAction;
			try {
				execute(testScript, template);
			} catch (final InvalidSyntaxException exception) {
				final String filter = template.getFilter();
				final Object[] parms = { filter };
				LogUtility.logError(Nls.format(TestManager.DefaultResourceBundle.getString(Integer.toString(TestManager.INVALID_FILTER)), parms));
				testScript.getScriptListener().scriptActionError(getId(), getDescription(), Nls.format(TestManager.DefaultResourceBundle.getString(Integer.toString(TestManager.INVALID_FILTER)), parms));
				testStatistics.incrementCount(TestStatisticsService.STATS_TEST_ERROR_COUNT, 1);
			} catch (final IOException exception) {
				LogUtility.logError(this, TestManager.DefaultResourceBundle.getString(Integer.toString(TestManager.COMMUNICATION_ERROR)), exception);
				testScript.getScriptListener().scriptActionError(getId(), getDescription(), TestManager.DefaultResourceBundle.getString(Integer.toString(TestManager.COMMUNICATION_ERROR)));
				testStatistics.incrementCount(TestStatisticsService.STATS_TEST_ERROR_COUNT, 1);
			}
		} else {
			execute(testScript, (Action) templateOrAction);
		}
	}

	/**
	 * Gets the description (String) value.
	 * @return The description (<code>String</code>) value.
	 * @see #getMatchDescription(String,String)
	 */
	public String getDescription() {
		return getName() + TestManager.DefaultResourceBundle.getString(Integer.toString(TestManager.TEST_ACTION));
	}

	/**
	 * Get match description with the specified filter and class name parameters and return the String result.
	 * @param filter
	 *		The filter (<code>String</code>) parameter.
	 * @param className
	 *		The class name (<code>String</code>) parameter.
	 * @return Results of the get match description (<code>String</code>) value.
	 */
	protected String getMatchDescription(final String filter, final String className) {
		String description = null;
		if (filter != null) {
			description = filter;
		}
		if (className != null) {
			if (description == null) {
				description = className;
			} else {
				description += " (" + className + ')'; //$NON-NLS-1$
			}
		}
		return description;
	}

	/**
	 * Gets the name (String) value.
	 * @return The name (<code>String</code>) value.
	 */
	protected abstract String getName();

	/**
	 * Get test session with the specified test script and controller id parameters and return the TestSessionService result.
	 * @param testScript
	 *		The test script (<code>TestScriptService</code>) parameter.
	 * @param controllerId
	 *		The controller ID (<code>String</code>) parameter.
	 * @return Results of the get test session (<code>TestSessionService</code>) value.
	 * @throws IOException IOException.
	 */
	protected TestSessionService getTestSession(final TestScriptService testScript, final String controllerId) throws IOException {
		final TestSessionService testSession = testScript.getDistributedTestSession().getTestSession(controllerId);
		if (testSession == null) {
			String detail;
			if (controllerId == null) {
				detail = TestManager.DefaultResourceBundle.getString(Integer.toString(TestManager.LOCAL_CONTROLLER_NOT_FOUND));
			} else {
				detail = Nls.format(TestManager.DefaultResourceBundle.getString(Integer.toString(TestManager.REMOTE_CONTROLLER_NOT_FOUND)), controllerId);
			}
			handleScriptActionError(testScript, detail);
		}
		return testSession;
	}

	/**
	 * Gets the test templates and actions (List) value.
	 * @return The test templates and actions (<code>List</code>) value.
	 */
	protected List getTestTemplatesAndActions() {
		return this.testTemplatesAndActions;
	}
}
