package org.eclipse.soda.dk.testmanager.console;

/*************************************************************************
 * Copyright (c) 2007, 2009 IBM.                                         *
 * All rights reserved. This program and the accompanying materials      *
 * are made available under the terms of the Eclipse Public License v1.0 *
 * which accompanies this distribution, and is available at              *
 * http://www.eclipse.org/legal/epl-v10.html                             *
 *                                                                       *
 * Contributors:                                                         *
 *     IBM - initial API and implementation                              *
 ************************************************************************/
import org.eclipse.soda.dk.nls.Nls;
import org.eclipse.soda.dk.testcontroller.console.TestControllerConsole;
import org.eclipse.soda.dk.testmanager.TestManager;
import org.eclipse.soda.dk.testmanager.service.ScriptRunListenerService;
import org.eclipse.soda.dk.testmanager.service.TestScriptService;
import org.eclipse.soda.dk.testmanager.service.TestStatisticsService;

/**
 * @author IBM
 * @version 1.2.0
 */
public class TestManagerConsole extends TestControllerConsole implements ScriptRunListenerService {
	/**
	 * Append attribute with the specified detail, name and value parameters.
	 * @param detail
	 *		The detail (<code>StringBuffer</code>) parameter.
	 * @param name
	 *		The name (<code>String</code>) parameter.
	 * @param value
	 *		The value (<code>Object</code>) parameter.
	 */
	private static void appendAttribute(final StringBuffer detail, final String name, final Object value) {
		if (value != null) {
			detail.append(name);
			detail.append(": "); //$NON-NLS-1$
			detail.append(value);
			detail.append('\n');
		}
	}

	/**
	 * Script action ended with the specified message parameter.
	 * @param actionId
	 *		The action ID (<code>String</code>) parameter.
	 * @param message
	 *		The message (<code>String</code>) parameter.
	 */
	public void scriptActionEnded(final String actionId, final String message) {
		final Object[] parms = { actionId, message };
		printEvent(Nls.format(TestManager.DefaultResourceBundle.getString(Integer.toString(TestManager.SCRIPT_ACTION_ENDED)), parms));
	}

	/**
	 * Script action error with the specified message and detail parameters.
	 * @param actionId
	 *		The action ID (<code>String</code>) parameter.
	 * @param message
	 *		The message (<code>String</code>) parameter.
	 * @param detail
	 *		The detail (<code>String</code>) parameter.
	 */
	public void scriptActionError(final String actionId, final String message, final String detail) {
		final Object[] parms = { actionId, message };
		printErrorEvent(Nls.format(TestManager.DefaultResourceBundle.getString(Integer.toString(TestManager.ERROR_EXECUTING)), parms), detail);
	}

	/**
	 * Script action info with the specified message parameter.
	 * @param actionId
	 *		The action ID (<code>String</code>) parameter.
	 * @param message
	 *		The message (<code>String</code>) parameter.
	 */
	public void scriptActionInfo(final String actionId, final String message) {
		printEvent(message);
	}

	/**
	 * Script action started with the specified message parameter.
	 * @param parentActionId
	 *		The parent action ID (<code>String</code>) parameter.
	 * @param actionId
	 *		The action ID (<code>String</code>) parameter.
	 * @param message
	 *		The message (<code>String</code>) parameter.
	 */
	public void scriptActionStarted(final String parentActionId, final String actionId, final String message) {
		final Object[] parms = { actionId, message };
		printEvent(Nls.format(TestManager.DefaultResourceBundle.getString(Integer.toString(TestManager.SCRIPT_ACTION_STARTING)), parms));
	}

	/**
	 * Script run ended with the specified test statistics parameter.
	 * @param testStatistics
	 *		The test statistics (<code>TestStatisticsService</code>) parameter.
	 */
	public void scriptRunEnded(final TestStatisticsService testStatistics) {
		boolean error = false;
		final StringBuffer trace = new StringBuffer();
		final int scriptErrorCount = testStatistics.getCount(TestStatisticsService.STATS_SCRIPT_ERROR_COUNT);
		if (scriptErrorCount > 0) {
			error = true;
		}
		final int configActionCount = testStatistics.getCount(TestStatisticsService.STATS_CONFIGURATION_COUNT);
		if (configActionCount > 0) {
			final int configActionErrorCount = testStatistics.getCount(TestStatisticsService.STATS_CONFIGURATION_ERROR_COUNT);
			if (configActionErrorCount > 0) {
				error = true;
			}
			final Object[] parms = { new Integer(configActionCount), new Integer(configActionErrorCount) };
			trace.append(Nls.format(TestManager.DefaultResourceBundle.getString(Integer.toString(TestManager.CONFIG_CHANGES)), parms));
		}
		final int testActionCount = testStatistics.getCount(TestStatisticsService.STATS_TEST_COUNT);
		if (testActionCount > 0) {
			final int testActionErrorCount = testStatistics.getCount(TestStatisticsService.STATS_TEST_ERROR_COUNT);
			if (testActionErrorCount > 0) {
				error = true;
			}
			final Object[] parms = { new Integer(testActionCount), new Integer(testActionErrorCount) };
			trace.append(Nls.format(TestManager.DefaultResourceBundle.getString(Integer.toString(TestManager.TESTS)), parms));
		}
		final int testCount = testStatistics.getCount(TestStatisticsService.STATS_TEST_CASE_COUNT);
		if (testCount > 0) {
			final int testErrorCount = testStatistics.getCount(TestStatisticsService.STATS_TEST_CASE_ERROR_COUNT);
			final int testFailedCount = testStatistics.getCount(TestStatisticsService.STATS_TEST_CASE_FAILED_COUNT);
			if ((testErrorCount > 0) || (testFailedCount > 0)) {
				error = true;
			}
			final Object[] parms = { new Integer(testCount), new Integer(testErrorCount), new Integer(testFailedCount) };
			trace.append(Nls.format(TestManager.DefaultResourceBundle.getString(Integer.toString(TestManager.TEST_CASES)), parms));
		}
		printEvent(error, TestManager.DefaultResourceBundle.getString(Integer.toString(TestManager.TEST_COMPLETED)), trace.toString());
	}

	/**
	 * Script run error with the specified message and trace parameters.
	 * @param message
	 *		The message (<code>String</code>) parameter.
	 * @param trace
	 *		The trace (<code>String</code>) parameter.
	 */
	public void scriptRunError(final String message, final String trace) {
		final Object[] parms = { message };
		printErrorEvent(Nls.format(TestManager.DefaultResourceBundle.getString(Integer.toString(TestManager.ERROR_RUNNING_SCRIPT)), parms), trace);
	}

	/**
	 * Script run started with the specified script parameter.
	 * @param script
	 *		The script (<code>TestScriptService</code>) parameter.
	 */
	public void scriptRunStarted(final TestScriptService script) {
		Object title = script.getAttributes().get(TestScriptService.ATTR_TITLE);
		final Object version = script.getAttributes().get(TestScriptService.ATTR_VERSION);
		final Object vendor = script.getAttributes().get(TestScriptService.ATTR_VENDOR);
		final Object description = script.getAttributes().get(TestScriptService.ATTR_DESCRIPTION);
		final Object url = script.getAttributes().get(TestScriptService.ATTR_URL);
		if (title == null) {
			title = TestManager.DefaultResourceBundle.getString(Integer.toString(TestManager.UNTITLED));
		}
		final StringBuffer detail = new StringBuffer();
		appendAttribute(detail, TestScriptService.ATTR_VERSION, version);
		appendAttribute(detail, TestScriptService.ATTR_VENDOR, vendor);
		appendAttribute(detail, TestScriptService.ATTR_DESCRIPTION, description);
		appendAttribute(detail, TestScriptService.ATTR_URL, url);
		final Object[] parms = { title };
		printEvent(false, Nls.format(TestManager.DefaultResourceBundle.getString(Integer.toString(TestManager.RUNNING_SCRIPT)), parms), detail.toString());
	}
}
