package org.eclipse.soda.dk.testmanager.managed;

/*************************************************************************
 * Copyright (c) 1999, 2009 IBM.                                         *
 * All rights reserved. This program and the accompanying materials      *
 * are made available under the terms of the Eclipse Public License v1.0 *
 * which accompanies this distribution, and is available at              *
 * http://www.eclipse.org/legal/epl-v10.html                             *
 *                                                                       *
 * Contributors:                                                         *
 *     IBM - initial API and implementation                              *
 ************************************************************************/
import java.util.Dictionary;
import java.util.Enumeration;
import java.util.Hashtable;
import javax.xml.parsers.SAXParserFactory;
import org.eclipse.soda.dk.core.ConfigurableObject;
import org.eclipse.soda.dk.testcontroller.service.ControllerLocaterService;
import org.eclipse.soda.dk.testcontroller.service.TestControllerService;
import org.eclipse.soda.dk.testmanager.TestManager;
import org.eclipse.soda.dk.testmanager.locater.osgi.OSGiControllerLocater;
import org.eclipse.soda.dk.testmanager.service.TestManagerService;
import org.eclipse.soda.sat.core.framework.BaseBundleActivator;
import org.eclipse.soda.sat.core.util.FactoryUtility;
import org.osgi.framework.Bundle;
import org.osgi.service.cm.ConfigurationException;
import org.osgi.service.cm.ManagedService;
import org.osgi.util.xml.XMLParserActivator;

/**
 * The DeviceBundle is the standard Bundle Activator for all other transport
 * bundle activators to subclass.
 * @author IBM
 * @version 1.2.0
 * @since 1.0
 */
public class TestManagerManaged extends BaseBundleActivator implements ManagedService {
	/**
	 * Defines the class name.
	 */
	public static final String CLASS_NAME = "org.eclipse.soda.dk.testmanager.managed.TestManagerManaged"; //$NON-NLS-1$

	/**
	 * Defines the service name(s) to be registered by the testAdapter bundle.
	 * <table>
	 * <tr>
	 * <td>org.eclipse.soda.dk.adapter.service.AdapterService</td>
	 * </tr>
	 * </table>
	 */
	public static final String[] EXPORTED_SERVICE_NAMES = new String[] { TestManagerService.SERVICE_NAME };

	/**
	 * Defines the imported service name(s) to be registered by the testAdapter
	 * bundle. <table>
	 * <tr>
	 * <td>org.eclipse.soda.dk.test.adapter.service.testAdapterService</td>
	 * </tr>
	 * <tr>
	 * <td>org.eclipse.soda.dk.notification.service.NotificationService</td>
	 * </tr>
	 * </table>
	 */
	public static final String[] IMPORTED_SERVICE_NAMES = BaseBundleActivator.NO_SERVICES;

	/**
	 * Define the optional services (String[]) constant.
	 */
	public static final String[] OPTIONAL_SERVICES = new String[] { SAXParserFactory.class.getName() };

	/**
	 * Define the local controller locater (OSGiControllerLocater) field.
	 */
	private OSGiControllerLocater localControllerLocater;

	/**
	 * Define the controller (TestManager) field.
	 */
	private TestManager testManager;

	/**
	 * Define the remote controller locater detecter (ControllerLocaterDetecter) field.
	 */
	private ControllerLocaterDetecter remoteControllerLocaterDetecter;

	/**
	 * Constructs an instance of this class.
	 */
	public TestManagerManaged() {
		this.localControllerLocater = new OSGiControllerLocater();
	}

	/**
	 * Create exported services.
	 */
	protected void activate() {
		final Hashtable properties = createManagedProperties();
		FactoryUtility util = FactoryUtility.getInstance();
		this.localControllerLocater.setServiceDetecter(util.createServiceDetecter(getBundleContext(), TestControllerService.SERVICE_NAME));
		addExportedService(ControllerLocaterService.SERVICE_NAME, this.localControllerLocater, properties);
		addExportedServices(new String[] { ManagedService.class.getName(), getClass().getName() }, this, properties);
	}

	/**
	 * Create managed properties and return the Hashtable result.
	 * @return Results of the create managed properties (<code>Hashtable</code>)
	 * value.
	 */
	public Hashtable createManagedProperties() {
		final Hashtable properties = createProperties();
		properties.put(org.osgi.framework.Constants.SERVICE_PID, getClass().getName());
		return properties;
	}

	/**
	 * Create the properties.
	 * @return Results of the create properties (<code>Hashtable</code>)
	 * value.
	 * @see #createManagedProperties()
	 * @see #updateProperties(Dictionary,Dictionary)
	 */
	public Hashtable createProperties() {
		final Bundle bundle = getBundle();
		final Hashtable properties = ConfigurableObject.createPropertiesFromHeaders(bundle.getHeaders());
		return properties;
	}

	/**
	 * Create the service.
	 * @return Results of the create service (<code>Object</code>) value.
	 * @see #handleAcquiredOptionalImportedService(String,Object)
	 * @see #handleReleasedOptionalImportedService(String,Object)
	 */
	public TestManager createService() {
		return new TestManager();
	}

	/**
	 * Deactivate.
	 */
	protected void deactivate() {
		this.localControllerLocater.setServiceDetecter(null);
		this.testManager = null;
	}

	/**
	 * Return the exported service name(s) to be registered by the testAdapter
	 * bundle.
	 * @return String[] The device's service names.
	 */
	public String[] getExportedServiceNames() {
		return EXPORTED_SERVICE_NAMES;
	}

	/**
	 * Return the imported service name(s) to be registered by the testAdapter
	 * bundle.
	 * @return String[] The imported service names.
	 * @see #getOptionalImportedServiceNames()
	 */
	public String[] getImportedServiceNames() {
		return IMPORTED_SERVICE_NAMES;
	}

	/**
	 * Gets the optional imported service names (String[]) value.
	 * @return The optional imported service names (<code>String[]</code>)
	 * value.
	 */
	protected String[] getOptionalImportedServiceNames() {
		return OPTIONAL_SERVICES;
	}

	/**
	 * Handle acquired optional imported service with the specified service name and service parameters.
	 * @param serviceName	The service name (<code>String</code>) parameter.
	 * @param service	The service (<code>Object</code>) parameter.
	 */
	protected void handleAcquiredOptionalImportedService(final String serviceName, final Object service) {
		if (this.testManager != null) {
			if (service instanceof SAXParserFactory) {
				this.testManager.setSaxParserFactory(((SAXParserFactory) service));
			}
		}
	}

	/**
	 * Handle released optional imported service with the specified service name and service parameters.
	 * @param serviceName	The service name (<code>String</code>) parameter.
	 * @param service	The service (<code>Object</code>) parameter.
	 */
	protected void handleReleasedOptionalImportedService(final String serviceName, final Object service) {
		if (this.testManager != null) {
			if (service instanceof SAXParserFactory) {
				this.testManager.setSaxParserFactory(null);
			}
		}
	}

	/**
	 * Start.
	 * @throws Exception Exception.
	 */
	protected void start() throws Exception {
		addOptionalImportedServiceFilter(SAXParserFactory.class.getName(), "(" + XMLParserActivator.PARSER_NAMESPACEAWARE + "=true)"); //$NON-NLS-1$//$NON-NLS-2$
	}

	/**
	 * Stop.
	 * @throws Exception Exception.
	 */
	protected void stop() throws Exception {
		removeOptionalImportedServiceFilter(SAXParserFactory.class.getName());
	}

	/**
	 * Update properties with the specified table and from parameters.
	 * @param table	The table (<code>Dictionary</code>) parameter.
	 * @param from	The from (<code>Dictionary</code>) parameter.
	 * @see #createManagedProperties()
	 * @see #createProperties()
	 */
	public void updateProperties(final Dictionary table, final Dictionary from) {
		if (from != null) {
			final Enumeration enumeration = from.keys();
			while (enumeration.hasMoreElements()) {
				final Object key = enumeration.nextElement();
				final Object value = from.get(key);
				table.put(key, value);
			}
		}
	}

	/**
	 * Updated with the specified properties parameter.
	 * The properties (<code>Dictionary</code>) parameter.
	 * Configuration Exception.
	 * @param properties	The properties (<code>Dictionary</code>) parameter.
	 * @throws ConfigurationException Configuration Exception.
	 */
	public void updated(final Dictionary properties) throws ConfigurationException {
		if (this.testManager != null) {
			this.remoteControllerLocaterDetecter.deactivate();
			this.remoteControllerLocaterDetecter = null;
			this.testManager.setLocalControllerLocater(null);
			removeExportedService(TestManagerService.SERVICE_NAME);
			this.testManager = null;
		}
		this.testManager = createService();
		this.testManager.setConfigurationInformation(properties);
		this.testManager.setLocalControllerLocater(this.localControllerLocater);
		this.remoteControllerLocaterDetecter = new ControllerLocaterDetecter(this.testManager);
		this.remoteControllerLocaterDetecter.activate(getBundleContext());
		this.testManager.setSaxParserFactory((SAXParserFactory) getOptionalImportedService(SAXParserFactory.class.getName()));
		final String[] names = getExportedServiceNames();
		final Hashtable serviceProperties = createProperties();
		updateProperties(serviceProperties, properties);
		addExportedServices(names, this.testManager, serviceProperties);
	}
}
