package org.eclipse.soda.dk.testmanager.parser;

/*************************************************************************
 * Copyright (c) 2007, 2009 IBM.                                         *
 * All rights reserved. This program and the accompanying materials      *
 * are made available under the terms of the Eclipse Public License v1.0 *
 * which accompanies this distribution, and is available at              *
 * http://www.eclipse.org/legal/epl-v10.html                             *
 *                                                                       *
 * Contributors:                                                         *
 *     IBM - initial API and implementation                              *
 ************************************************************************/
import java.io.InputStream;
import java.io.Reader;
import java.util.Dictionary;
import java.util.Hashtable;
import java.util.Map;
import javax.xml.parsers.SAXParser;
import org.eclipse.soda.dk.nls.Nls;
import org.eclipse.soda.dk.testmanager.TestManager;
import org.eclipse.soda.dk.testmanager.TestScript;
import org.eclipse.soda.dk.testmanager.action.ConfigurationAction;
import org.eclipse.soda.dk.testmanager.action.ConfigurationActionFactory;
import org.eclipse.soda.dk.testmanager.action.TestAction;
import org.eclipse.soda.dk.testmanager.action.TestActionFactory;
import org.eclipse.soda.dk.testmanager.service.TestScriptService;
import org.xml.sax.Attributes;
import org.xml.sax.InputSource;
import org.xml.sax.SAXException;
import org.xml.sax.SAXNotRecognizedException;
import org.xml.sax.SAXParseException;
import org.xml.sax.helpers.DefaultHandler;

/**
 * @author IBM
 * @version 1.2.0
 * @since 1.0
 */
public class ContentHandler extends DefaultHandler {
	/**
	 * Define the jaxp schema language (String) constant.
	 */
	private final static String JAXP_SCHEMA_LANGUAGE = "http://java.sun.com/xml/jaxp/properties/schemaLanguage"; //$NON-NLS-1$

	/**
	 * Define the w3c xml schema (String) constant.
	 */
	private final static String W3C_XML_SCHEMA = "http://www.w3.org/2001/XMLSchema"; //$NON-NLS-1$

	/**
	 * Define the jaxp schema source (String) constant.
	 */
	private final static String JAXP_SCHEMA_SOURCE = "http://java.sun.com/xml/jaxp/properties/schemaSource"; //$NON-NLS-1$

	/**
	 * Define the schema source (String) constant.
	 */
	private final static String SCHEMA_SOURCE = "/dktest.xsd"; //$NON-NLS-1$

	/**
	 * Define the script (TestScript) field.
	 */
	TestScript script;

	/**
	 * Define the state (State) field.
	 */
	State state;

	/**
	 * Define the lineNumber (int) field.
	 */
	int lineNumber;

	/**
	 * Define the action number (int) field.
	 */
	private int actionNumber;

	/**
	 * Constructs an instance of this class.
	 */
	public ContentHandler() {
		super();
		this.script = new TestScript();
		this.state = new ContentHandler.EndDocumentState();
		this.lineNumber = 1;
		this.actionNumber = 1;
	}

	/**
	 * Parse with the specified sax parser factory and in parameters and return the TestScript result.
	 * @param saxParser
	 *		The sax parser (<code>SAXParser</code>) parameter.
	 * @param in
	 *		The in (<code>Reader</code>) parameter.
	 * @return Results of the parse (<code>TestScript</code>) value.
	 * @throws Exception Exception.
	 */
	public static TestScript parse(final SAXParser saxParser, final Reader in) throws Exception {
		if (saxParser.isValidating()) {
			try {
				saxParser.setProperty(ContentHandler.JAXP_SCHEMA_LANGUAGE, ContentHandler.W3C_XML_SCHEMA);
				final InputStream schema = ContentHandler.class.getResourceAsStream(ContentHandler.SCHEMA_SOURCE);
				saxParser.setProperty(ContentHandler.JAXP_SCHEMA_SOURCE, schema);
			} catch (final SAXNotRecognizedException e) {
				/* ignore exception */
			}
		}
		final InputSource inputSource = new InputSource(in);
		final ContentHandler contentHandler = new ContentHandler();
		saxParser.parse(inputSource, contentHandler);
		return contentHandler.getTestScript();
	}

	/**
	 * Characters with the specified ch, start and length parameters.
	 * @param ch
	 *		The ch (<code>char[]</code>) parameter.
	 * @param start
	 *		The start (<code>int</code>) parameter.
	 * @param length
	 *		The length (<code>int</code>) parameter.
	 * @throws SAXException SAXException.
	 */
	public void characters(final char[] ch, final int start, final int length) throws SAXException {
		this.state.characters(ch, start, length);
		countLines(ch, start, length);
	}

	/**
	 * End document.
	 * @see #startDocument()
	 */
	public void endDocument() {
		this.state.endDocument();
	}

	/**
	 * End element with the specified namespace uri, local name and q name parameters.
	 * @param namespaceURI
	 *		The namespace uri (<code>String</code>) parameter.
	 * @param localName
	 *		The local name (<code>String</code>) parameter.
	 * @param qName
	 *		The q name (<code>String</code>) parameter.
	 * @throws SAXException SAXException.
	 * @see #startElement(String,String,String,Attributes)
	 */
	public void endElement(final String namespaceURI, final String localName, final String qName) throws SAXException {
		this.state.endElement(localName);
	}

	/**
	 * Get action id with the specified id parameter and return the String result.
	 * @param id
	 *		The ID (<code>String</code>) parameter.
	 * @return Results of the get action id (<code>String</code>) value.
	 */
	public String getActionId(final String id) {
		if ((id == null) || (id.length() == 0)) {
			return Integer.toString(this.actionNumber++);
		}
		return id;
	}

	/**
	 * Gets the test script value.
	 * @return The test script (<code>TestScript</code>) value.
	 */
	private TestScript getTestScript() {
		return this.script;
	}

	/**
	 * Start document.
	 * @see #endDocument()
	 */
	public void startDocument() {
		this.state.startDocument();
	}

	/**
	 * Start element with the specified namespace uri, local name, q name and atts parameters.
	 * @param namespaceURI
	 *		The namespace uri (<code>String</code>) parameter.
	 * @param localName
	 *		The local name (<code>String</code>) parameter.
	 * @param qName
	 *		The q name (<code>String</code>) parameter.
	 * @param atts
	 *		The atts (<code>Attributes</code>) parameter.
	 * @throws SAXException SAXException.
	 * @see #endElement(String,String,String)
	 */
	public void startElement(final String namespaceURI, final String localName, final String qName, final Attributes atts) throws SAXException {
		this.state.startElement(localName, atts);
	}

	/**
	 * @author IBM
	 * @version 1.2.0
	 */
	abstract class State {
		/**
		 * Define the parent state (State) field.
		 */
		protected State parentState;

		/**
		 * Constructs an instance of this class from the specified parent state parameter.
		 * @param parentState
		 *		The parent state (<code>State</code>) parameter.
		 */
		protected State(final ContentHandler.State parentState) {
			this.parentState = parentState;
		}

		/**
		 * Characters with the specified ch, start and length parameters.
		 * @param ch
		 *		The ch (<code>char[]</code>) parameter.
		 * @param start
		 *		The start (<code>int</code>) parameter.
		 * @param length
		 *		The length (<code>int</code>) parameter.
		 */
		public void characters(final char[] ch, final int start, final int length) {
			/* do nothing */
		}

		/**
		 * End document.
		 * @see #startDocument()
		 */
		void endDocument() {
			ContentHandler.this.state = new ContentHandler.EndDocumentState();
		}

		/**
		 * End element with the specified local name parameter.
		 * @param localName
		 *		The local name (<code>String</code>) parameter.
		 * @throws SAXParseException SAXParse Exception.
		 * @see #startElement(String,Attributes)
		 * @see #unexpectedEndElement(String)
		 * @see #unexpectedStartElement(String)
		 */
		void endElement(final String localName) throws SAXParseException {
			/* do nothing */
		}

		/**
		 * Return to parent.
		 */
		protected void returnToParent() {
			ContentHandler.this.state = this.parentState;
		}

		/**
		 * Start document.
		 * @see #endDocument()
		 */
		void startDocument() {
			ContentHandler.this.state = new ContentHandler.StartDocumentState();
		}

		/**
		 * Start element with the specified local name and atts parameters.
		 * @param localName
		 *		The local name (<code>String</code>) parameter.
		 * @param atts
		 *		The atts (<code>Attributes</code>) parameter.
		 * @throws SAXParseException SAXParse Exception.
		 * @see #endElement(String)
		 * @see #unexpectedEndElement(String)
		 * @see #unexpectedStartElement(String)
		 */
		void startElement(final String localName, final Attributes atts) throws SAXParseException {
			/* do nothing */
		}

		/**
		 * Trim with the specified to trim parameter and return the String result.
		 * @param toTrim
		 *		The to trim (<code>String</code>) parameter.
		 * @return Results of the trim (<code>String</code>) value.
		 */
		String trim(final String toTrim) {
			if (toTrim != null) {
				return toTrim.trim();
			}
			return null;
		}

		/**
		 * Unexpected attribute value with the specified attr, value and message parameters.
		 * @param attr
		 *		The attr (<code>String</code>) parameter.
		 * @param value
		 *		The value (<code>String</code>) parameter.
		 * @param message
		 *		The message (<code>String</code>) parameter.
		 * @throws SAXParseException SAXParse Exception.
		 */
		void unexpectedAttributeValue(final String attr, final String value, final String message) throws SAXParseException {
			final Object[] parms = { attr, value, message, new Integer(ContentHandler.this.lineNumber) };
			throw new SAXParseException(Nls.format(TestManager.DefaultResourceBundle.getString(Integer.toString(TestManager.UNEXPECTED_ATTR_VALUE)), parms), null, null, ContentHandler.this.lineNumber, 0);
		}

		/**
		 * Unexpected end element with the specified local name parameter.
		 * @param localName
		 *		The local name (<code>String</code>) parameter.
		 * @throws SAXParseException SAXParse Exception.
		 */
		void unexpectedEndElement(final String localName) throws SAXParseException {
			final Object[] parms = { localName, new Integer(ContentHandler.this.lineNumber) };
			throw new SAXParseException(Nls.format(TestManager.DefaultResourceBundle.getString(Integer.toString(TestManager.UNEXPECTED_XML_END)), parms), null, null, ContentHandler.this.lineNumber, 0);
		}

		/**
		 * Unexpected start element with the specified local name parameter.
		 * @param localName
		 *		The local name (<code>String</code>) parameter.
		 * @throws SAXParseException SAXParse Exception.
		 */
		void unexpectedStartElement(final String localName) throws SAXParseException {
			final Object[] parms = { localName, new Integer(ContentHandler.this.lineNumber) };
			throw new SAXParseException(Nls.format(TestManager.DefaultResourceBundle.getString(Integer.toString(TestManager.UNEXPECTED_XML_START)), parms), null, null, ContentHandler.this.lineNumber, 0);
		}
	}

	/**
	 * @author IBM
	 * @version 1.2.0
	 */
	class StartDocumentState extends ContentHandler.State {
		/**
		 * Constructs an instance of this class.
		 */
		public StartDocumentState() {
			super(null);
		}

		/**
		 * Start element with the specified local name and atts parameters.
		 * @param localName
		 *		The local name (<code>String</code>) parameter.
		 * @param atts
		 *		The atts (<code>Attributes</code>) parameter.
		 * @throws SAXParseException SAXParse Exception.
		 */
		void startElement(final String localName, final Attributes atts) throws SAXParseException {
			if ("validationTest".equals(localName)) { //$NON-NLS-1$
				ContentHandler.this.state = new ValidationTestState(this);
			} else {
				unexpectedStartElement(localName);
			}
		}
	}

	/**
	 * @author IBM
	 * @version 1.2.0
	 */
	class AttributeState extends ContentHandler.State {
		/**
		 * Define the attr name (String) field.
		 */
		private String attrName;

		/**
		 * Define the attributes (Map) field.
		 */
		private Map attributes;

		/**
		 * Constructs an instance of this class from the specified parent state, attr name and attributes parameters.
		 * @param parentState
		 *		The parent state (<code>State</code>) parameter.
		 * @param attrName
		 *		The attr name (<code>String</code>) parameter.
		 * @param attributes
		 *		The attributes (<code>Map</code>) parameter.
		 */
		public AttributeState(final ContentHandler.State parentState, final String attrName, final Map attributes) {
			super(parentState);
			this.attrName = attrName;
			this.attributes = attributes;
		}

		/**
		 * Characters with the specified ch, start and length parameters.
		 * @param ch
		 *		The ch (<code>char[]</code>) parameter.
		 * @param start
		 *		The start (<code>int</code>) parameter.
		 * @param length
		 *		The length (<code>int</code>) parameter.
		 */
		public void characters(final char[] ch, final int start, final int length) {
			this.attributes.put(this.attrName, new String(ch, start, length));
		}

		/**
		 * End element with the specified local name parameter.
		 * @param localName
		 *		The local name (<code>String</code>) parameter.
		 * @throws SAXParseException SAXParse Exception.
		 * @see #startElement(String,Attributes)
		 */
		void endElement(final String localName) throws SAXParseException {
			if (this.attrName.equals(localName)) {
				returnToParent();
			} else {
				unexpectedEndElement(localName);
			}
		}

		/**
		 * Start element with the specified local name and atts parameters.
		 * @param localName
		 *		The local name (<code>String</code>) parameter.
		 * @param atts
		 *		The atts (<code>Attributes</code>) parameter.
		 * @throws SAXParseException SAXParse Exception.
		 * @see #endElement(String)
		 */
		void startElement(final String localName, final Attributes atts) throws SAXParseException {
			unexpectedStartElement(localName);
		}
	}

	/**
	 * @author IBM
	 * @version 1.2.0
	 */
	class ValidationTestState extends ContentHandler.State {
		/**
		 * Constructs an instance of this class from the specified parent state parameter.
		 * @param parentState
		 *		The parent state (<code>State</code>) parameter.
		 */
		public ValidationTestState(final ContentHandler.State parentState) {
			super(parentState);
		}

		/**
		 * End element with the specified local name parameter.
		 * @param localName
		 *		The local name (<code>String</code>) parameter.
		 * @throws SAXParseException SAXParse Exception.
		 * @see #startElement(String,Attributes)
		 */
		void endElement(final String localName) throws SAXParseException {
			if ("validationTest".equals(localName)) { //$NON-NLS-1$
				returnToParent();
			} else {
				unexpectedEndElement(localName);
			}
		}

		/**
		 * Start element with the specified local name and atts parameters.
		 * @param localName
		 *		The local name (<code>String</code>) parameter.
		 * @param atts
		 *		The atts (<code>Attributes</code>) parameter.
		 * @throws SAXParseException SAXParse Exception.
		 * @see #endElement(String)
		 */
		void startElement(final String localName, final Attributes atts) throws SAXParseException {
			if ("actions".equals(localName)) { //$NON-NLS-1$
				ContentHandler.this.state = new ActionsState(this);
			} else if (TestScriptService.ATTR_TITLE.equals(localName) || TestScriptService.ATTR_PROVIDER.equals(localName) || TestScriptService.ATTR_VERSION.equals(localName) || TestScriptService.ATTR_VENDOR.equals(localName)
					|| TestScriptService.ATTR_DESCRIPTION.equals(localName)) {
				ContentHandler.this.state = new AttributeState(this, localName, ContentHandler.this.script.getAttributes());
			} else {
				unexpectedStartElement(localName);
			}
		}
	}

	/**
	 * @author IBM
	 * @version 1.2.0
	 */
	class ActionsState extends ContentHandler.State {
		/**
		 * Constructs an instance of this class from the specified parent state parameter.
		 * @param parentState
		 *		The parent state (<code>State</code>) parameter.
		 */
		public ActionsState(final ContentHandler.State parentState) {
			super(parentState);
		}

		/**
		 * End element with the specified local name parameter.
		 * @param localName
		 *		The local name (<code>String</code>) parameter.
		 * @throws SAXParseException SAXParse Exception.
		 * @see #startElement(String,Attributes)
		 */
		void endElement(final String localName) throws SAXParseException {
			if ("actions".equals(localName)) { //$NON-NLS-1$
				returnToParent();
			} else {
				unexpectedEndElement(localName);
			}
		}

		/**
		 * Start element with the specified local name and atts parameters.
		 * @param localName
		 *		The local name (<code>String</code>) parameter.
		 * @param atts
		 *		The atts (<code>Attributes</code>) parameter.
		 * @throws SAXParseException SAXParse Exception.
		 * @see #endElement(String)
		 */
		void startElement(final String localName, final Attributes atts) throws SAXParseException {
			if ("configurationAction".equals(localName)) { //$NON-NLS-1$
				final String id = getActionId(super.trim(atts.getValue("id"))); //$NON-NLS-1$
				final String type = super.trim(atts.getValue("type")); //$NON-NLS-1$
				final ConfigurationAction configurationAction = ConfigurationActionFactory.create(id, type);
				ContentHandler.this.script.add(configurationAction);
				ContentHandler.this.state = new ConfigurationActionState(this, configurationAction);
			} else if ("testAction".equals(localName)) { //$NON-NLS-1$
				final String id = getActionId(super.trim(atts.getValue("id"))); //$NON-NLS-1$
				final String type = super.trim(atts.getValue("type")); //$NON-NLS-1$
				final TestAction testAction = TestActionFactory.create(id, type);
				ContentHandler.this.script.add(testAction);
				ContentHandler.this.state = new TestActionState(this, testAction);
			} else {
				unexpectedStartElement(localName);
			}
		}
	}

	/**
	 * @author IBM
	 * @version 1.2.0
	 */
	class ConfigurationActionState extends ContentHandler.State {
		/**
		 * Define the configuration action (ConfigurationAction) field.
		 */
		private ConfigurationAction configurationAction;

		/**
		 * Constructs an instance of this class from the specified parent state and configuration action parameters.
		 * @param parentState
		 *		The parent state (<code>State</code>) parameter.
		 * @param configurationAction
		 *		The configuration action (<code>ConfigurationAction</code>) parameter.
		 */
		public ConfigurationActionState(final ContentHandler.State parentState, final ConfigurationAction configurationAction) {
			super(parentState);
			this.configurationAction = configurationAction;
		}

		/**
		 * End element with the specified local name parameter.
		 * @param localName
		 *		The local name (<code>String</code>) parameter.
		 * @throws SAXParseException SAXParse Exception.
		 * @see #startElement(String,Attributes)
		 */
		void endElement(final String localName) throws SAXParseException {
			if ("configurationAction".equals(localName)) { //$NON-NLS-1$
				returnToParent();
			} else {
				unexpectedEndElement(localName);
			}
		}

		/**
		 * Start element with the specified local name and atts parameters.
		 * @param localName
		 *		The local name (<code>String</code>) parameter.
		 * @param atts
		 *		The atts (<code>Attributes</code>) parameter.
		 * @throws SAXParseException SAXParse Exception.
		 * @see #endElement(String)
		 */
		void startElement(final String localName, final Attributes atts) throws SAXParseException {
			if ("configurations".equals(localName)) { //$NON-NLS-1$
				ContentHandler.this.state = new ConfigurationsState(this, this.configurationAction);
			} else {
				unexpectedStartElement(localName);
			}
		}
	}

	/**
	 * @author IBM
	 * @version 1.2.0
	 */
	class ConfigurationsState extends ContentHandler.State {
		/**
		 * Define the configuration action (ConfigurationAction) field.
		 */
		private ConfigurationAction configurationAction;

		/**
		 * Constructs an instance of this class from the specified parent state and configuration action parameters.
		 * @param parentState
		 *		The parent state (<code>State</code>) parameter.
		 * @param configurationAction
		 *		The configuration action (<code>ConfigurationAction</code>) parameter.
		 */
		public ConfigurationsState(final ContentHandler.State parentState, final ConfigurationAction configurationAction) {
			super(parentState);
			this.configurationAction = configurationAction;
		}

		/**
		 * End element with the specified local name parameter.
		 * @param localName
		 *		The local name (<code>String</code>) parameter.
		 * @throws SAXParseException SAXParse Exception.
		 * @see #startElement(String,Attributes)
		 */
		void endElement(final String localName) throws SAXParseException {
			if ("configurations".equals(localName)) { //$NON-NLS-1$
				returnToParent();
			} else {
				unexpectedEndElement(localName);
			}
		}

		/**
		 * Start element with the specified local name and atts parameters.
		 * @param localName
		 *		The local name (<code>String</code>) parameter.
		 * @param atts
		 *		The atts (<code>Attributes</code>) parameter.
		 * @throws SAXParseException SAXParse Exception.
		 * @see #endElement(String)
		 */
		void startElement(final String localName, final Attributes atts) throws SAXParseException {
			if ("configuration".equals(localName)) { //$NON-NLS-1$
				final String id = getActionId(super.trim(atts.getValue("id"))); //$NON-NLS-1$
				final String controllerId = super.trim(atts.getValue("controller")); //$NON-NLS-1$
				final String pid = super.trim(atts.getValue("pid")); //$NON-NLS-1$
				final String factoryPid = super.trim(atts.getValue("factoryPid")); //$NON-NLS-1$
				final String bundleLocation = super.trim(atts.getValue("bundleLocation")); //$NON-NLS-1$
				final String filter = super.trim(atts.getValue("filter")); //$NON-NLS-1$
				final Dictionary properties = new Hashtable();
				this.configurationAction.addConfiguration(id, controllerId, pid, factoryPid, bundleLocation, filter, properties);
				ContentHandler.this.state = new ConfigurationState(this, properties);
			} else {
				unexpectedStartElement(localName);
			}
		}
	}

	/**
	 * @author IBM
	 * @version 1.2.0
	 */
	class ConfigurationState extends ContentHandler.State {
		/**
		 * Define the properties (Dictionary) field.
		 */
		private Dictionary properties;

		/**
		 * Constructs an instance of this class from the specified parent state and properties parameters.
		 * @param parentState
		 *		The parent state (<code>State</code>) parameter.
		 * @param properties
		 *		The properties (<code>Dictionary</code>) parameter.
		 */
		public ConfigurationState(final ContentHandler.State parentState, final Dictionary properties) {
			super(parentState);
			this.properties = properties;
		}

		/**
		 * End element with the specified local name parameter.
		 * @param localName
		 *		The local name (<code>String</code>) parameter.
		 * @throws SAXParseException SAXParse Exception.
		 * @see #startElement(String,Attributes)
		 */
		void endElement(final String localName) throws SAXParseException {
			if ("configuration".equals(localName)) { //$NON-NLS-1$
				returnToParent();
			} else {
				unexpectedEndElement(localName);
			}
		}

		/**
		 * Start element with the specified local name and atts parameters.
		 * @param localName
		 *		The local name (<code>String</code>) parameter.
		 * @param atts
		 *		The atts (<code>Attributes</code>) parameter.
		 * @throws SAXParseException SAXParse Exception.
		 * @see #endElement(String)
		 */
		void startElement(final String localName, final Attributes atts) throws SAXParseException {
			if ("properties".equals(localName)) { //$NON-NLS-1$
				ContentHandler.this.state = new PropertiesState(this, this.properties);
			} else {
				unexpectedStartElement(localName);
			}
		}
	}

	/**
	 * @author IBM
	 * @version 1.2.0
	 */
	class PropertiesState extends ContentHandler.State {
		/**
		 * Define the properties (Dictionary) field.
		 */
		private Dictionary properties;

		/**
		 * Constructs an instance of this class from the specified parent state and properties parameters.
		 * @param parentState
		 *		The parent state (<code>State</code>) parameter.
		 * @param properties
		 *		The properties (<code>Dictionary</code>) parameter.
		 */
		public PropertiesState(final ContentHandler.State parentState, final Dictionary properties) {
			super(parentState);
			this.properties = properties;
		}

		/**
		 * End element with the specified local name parameter.
		 * @param localName
		 *		The local name (<code>String</code>) parameter.
		 * @throws SAXParseException SAXParse Exception.
		 * @see #startElement(String,Attributes)
		 */
		void endElement(final String localName) throws SAXParseException {
			if ("properties".equals(localName)) { //$NON-NLS-1$
				returnToParent();
			} else {
				unexpectedEndElement(localName);
			}
		}

		/**
		 * Start element with the specified local name and atts parameters.
		 * @param localName
		 *		The local name (<code>String</code>) parameter.
		 * @param atts
		 *		The atts (<code>Attributes</code>) parameter.
		 * @throws SAXParseException SAXParse Exception.
		 * @see #endElement(String)
		 */
		void startElement(final String localName, final Attributes atts) throws SAXParseException {
			if ("property".equals(localName)) { //$NON-NLS-1$
				final String key = super.trim(atts.getValue("key")); //$NON-NLS-1$
				final String value = super.trim(atts.getValue("value")); //$NON-NLS-1$
				final String type = super.trim(atts.getValue("type")); //$NON-NLS-1$
				final String cardinality = super.trim(atts.getValue("cardinality")); //$NON-NLS-1$
				try {
					if (cardinality == null) {
						this.properties.put(key, PropertyConverterUtility.convert(value, type));
					} else {
						int c = 1;
						try {
							c = Integer.parseInt(cardinality);
						} catch (final NumberFormatException e) {
							unexpectedAttributeValue("cardinality", cardinality, e.getMessage()); //$NON-NLS-1$
						}
						this.properties.put(key, PropertyConverterUtility.convert(value, type, c));
					}
				} catch (final NumberFormatException e) {
					unexpectedAttributeValue("value", value, e.getLocalizedMessage()); //$NON-NLS-1$
				}
				ContentHandler.this.state = new PropertyState(this);
			} else {
				unexpectedStartElement(localName);
			}
		}
	}

	/**
	 * @author IBM
	 * @version 1.2.0
	 */
	class PropertyState extends ContentHandler.State {
		/**
		 * Constructs an instance of this class from the specified parent state parameter.
		 * @param parentState
		 *		The parent state (<code>State</code>) parameter.
		 */
		public PropertyState(final ContentHandler.State parentState) {
			super(parentState);
		}

		/**
		 * End element with the specified local name parameter.
		 * @param localName
		 *		The local name (<code>String</code>) parameter.
		 * @throws SAXParseException SAXParse Exception.
		 * @see #startElement(String,Attributes)
		 */
		void endElement(final String localName) throws SAXParseException {
			if ("property".equals(localName)) { //$NON-NLS-1$
				returnToParent();
			} else {
				unexpectedEndElement(localName);
			}
		}

		/**
		 * Start element with the specified local name and atts parameters.
		 * @param localName
		 *		The local name (<code>String</code>) parameter.
		 * @param atts
		 *		The atts (<code>Attributes</code>) parameter.
		 * @throws SAXParseException SAXParse Exception.
		 * @see #endElement(String)
		 */
		void startElement(final String localName, final Attributes atts) throws SAXParseException {
			unexpectedStartElement(localName);
		}
	}

	/**
	 * @author IBM
	 * @version 1.2.0
	 */
	class TestActionState extends ContentHandler.State {
		/**
		 * Define the test action (TestAction) field.
		 */
		private TestAction testAction;

		/**
		 * Constructs an instance of this class from the specified parent state and test action parameters.
		 * @param parentState
		 *		The parent state (<code>State</code>) parameter.
		 * @param testAction
		 *		The test action (<code>TestAction</code>) parameter.
		 */
		public TestActionState(final ContentHandler.State parentState, final TestAction testAction) {
			super(parentState);
			this.testAction = testAction;
		}

		/**
		 * End element with the specified local name parameter.
		 * @param localName
		 *		The local name (<code>String</code>) parameter.
		 * @throws SAXParseException SAXParse Exception.
		 * @see #startElement(String,Attributes)
		 */
		void endElement(final String localName) throws SAXParseException {
			if ("testAction".equals(localName)) { //$NON-NLS-1$
				returnToParent();
			} else {
				unexpectedEndElement(localName);
			}
		}

		/**
		 * Start element with the specified local name and atts parameters.
		 * @param localName
		 *		The local name (<code>String</code>) parameter.
		 * @param atts
		 *		The atts (<code>Attributes</code>) parameter.
		 * @throws SAXParseException SAXParse Exception.
		 * @see #endElement(String)
		 */
		void startElement(final String localName, final Attributes atts) throws SAXParseException {
			if ("tests".equals(localName)) { //$NON-NLS-1$
				ContentHandler.this.state = new TestsState(this, this.testAction);
			} else if ("testAction".equals(localName)) { //$NON-NLS-1$
				final String id = getActionId(super.trim(atts.getValue("id"))); //$NON-NLS-1$
				final String type = super.trim(atts.getValue("type")); //$NON-NLS-1$
				final TestAction testAction = TestActionFactory.create(id, type);
				this.testAction.add(testAction);
				ContentHandler.this.state = new TestActionState(this, testAction);
			} else {
				unexpectedStartElement(localName);
			}
		}
	}

	/**
	 * @author IBM
	 * @version 1.2.0
	 */
	class TestsState extends ContentHandler.State {
		/**
		 * Define the test action (TestAction) field.
		 */
		private TestAction testAction;

		/**
		 * Constructs an instance of this class from the specified parent state and test action parameters.
		 * @param parentState
		 *		The parent state (<code>State</code>) parameter.
		 * @param testAction
		 *		The test action (<code>TestAction</code>) parameter.
		 */
		public TestsState(final ContentHandler.State parentState, final TestAction testAction) {
			super(parentState);
			this.testAction = testAction;
		}

		/**
		 * End element with the specified local name parameter.
		 * @param localName
		 *		The local name (<code>String</code>) parameter.
		 * @throws SAXParseException SAXParse Exception.
		 * @see #startElement(String,Attributes)
		 */
		void endElement(final String localName) throws SAXParseException {
			if ("tests".equals(localName)) { //$NON-NLS-1$
				returnToParent();
			} else {
				unexpectedEndElement(localName);
			}
		}

		/**
		 * Start element with the specified local name and atts parameters.
		 * @param localName
		 *		The local name (<code>String</code>) parameter.
		 * @param atts
		 *		The atts (<code>Attributes</code>) parameter.
		 * @throws SAXParseException SAXParse Exception.
		 * @see #endElement(String)
		 */
		void startElement(final String localName, final Attributes atts) throws SAXParseException {
			if ("test".equals(localName)) { //$NON-NLS-1$
				final String id = getActionId(super.trim(atts.getValue("id"))); //$NON-NLS-1$
				final String controllerId = super.trim(atts.getValue("controller")); //$NON-NLS-1$
				final String filter = super.trim(atts.getValue("filter")); //$NON-NLS-1$
				final String className = super.trim(atts.getValue("class")); //$NON-NLS-1$
				final String match = super.trim(atts.getValue("match")); //$NON-NLS-1$
				final Dictionary properties = new Hashtable();
				this.testAction.addTest(id, controllerId, filter, className, match, properties);
				ContentHandler.this.state = new TestState(this, properties);
			} else {
				unexpectedStartElement(localName);
			}
		}
	}

	/**
	 * @author IBM
	 * @version 1.2.0
	 */
	class TestState extends ContentHandler.State {
		/**
		 * Define the properties (Dictionary) field.
		 */
		private Dictionary properties;

		/**
		 * Constructs an instance of this class from the specified parent state and properties parameters.
		 * @param parentState
		 *		The parent state (<code>State</code>) parameter.
		 * @param properties
		 *		The properties (<code>Dictionary</code>) parameter.
		 */
		public TestState(final ContentHandler.State parentState, final Dictionary properties) {
			super(parentState);
			this.properties = properties;
		}

		/**
		 * End element with the specified local name parameter.
		 * @param localName
		 *		The local name (<code>String</code>) parameter.
		 * @throws SAXParseException SAXParse Exception.
		 * @see #startElement(String,Attributes)
		 */
		void endElement(final String localName) throws SAXParseException {
			if ("test".equals(localName)) { //$NON-NLS-1$
				returnToParent();
			} else {
				unexpectedEndElement(localName);
			}
		}

		/**
		 * Start element with the specified local name and atts parameters.
		 * @param localName
		 *		The local name (<code>String</code>) parameter.
		 * @param atts
		 *		The atts (<code>Attributes</code>) parameter.
		 * @throws SAXParseException SAXParse Exception.
		 * @see #endElement(String)
		 */
		void startElement(final String localName, final Attributes atts) throws SAXParseException {
			if ("properties".equals(localName)) { //$NON-NLS-1$
				ContentHandler.this.state = new PropertiesState(this, this.properties);
			} else {
				unexpectedStartElement(localName);
			}
		}
	}

	/**
	 * @author IBM
	 * @version 1.2.0
	 */
	class EndDocumentState extends ContentHandler.State {
		/**
		 * Constructs an instance of this class.
		 */
		public EndDocumentState() {
			super(null);
		}
	}

	/**
	 * Count lines with the specified ch, start and length parameters.
	 * @param ch
	 *		The ch (<code>char[]</code>) parameter.
	 * @param start
	 *		The start (<code>int</code>) parameter.
	 * @param length
	 *		The length (<code>int</code>) parameter.
	 */
	private void countLines(final char[] ch, final int start, final int length) {
		int i = start;
		final int end = start + length;
		while (i < end) {
			if (ch[i++] == '\n') {
				this.lineNumber += 1;
			}
		}
	}

	/**
	 * Ignorable whitespace with the specified ch, start and length parameters.
	 * @param ch
	 *		The ch (<code>char[]</code>) parameter.
	 * @param start
	 *		The start (<code>int</code>) parameter.
	 * @param length
	 *		The length (<code>int</code>) parameter.
	 * @throws SAXException SAXException.
	 */
	public void ignorableWhitespace(final char[] ch, final int start, final int length) throws SAXException {
		countLines(ch, start, length);
	}
}
