package org.eclipse.soda.dk.testmanager.servlet;

/*************************************************************************
 * Copyright (c) 2006, 2009 IBM.                                         *
 * All rights reserved. This program and the accompanying materials      *
 * are made available under the terms of the Eclipse Public License v1.0 *
 * which accompanies this distribution, and is available at              *
 * http://www.eclipse.org/legal/epl-v10.html                             *
 *                                                                       *
 * Contributors:                                                         *
 *     IBM - initial API and implementation                              *
 ************************************************************************/
import java.io.FilterInputStream;
import java.io.IOException;
import java.io.UnsupportedEncodingException;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

/**
 * @author IBM
 * @version 1.2.0
 * @since 1.0
 */
public class MimeMultiPart {
	/**
	 * Define the request (HttpServletRequest) field.
	 */
	private HttpServletRequest request;

	/**
	 * Define the response (HttpServletResponse) field.
	 */
	private HttpServletResponse response;

	/**
	 * Define the part nput stream (PartInputStream) field.
	 */
	private PartInputStream partInputStream;

	/**
	 * Define the content type (String) field.
	 */
	private String contentType;

	/**
	 * Define the boundary (String) field.
	 */
	String boundary;

	/**
	 * Define the character encoding (String) field.
	 */
	String characterEncoding;

	/**
	 * Constructs an instance of this class from the specified request and response parameters.
	 * @param request	The request (<code>HttpServletRequest</code>) parameter.
	 * @param response	The response (<code>HttpServletResponse</code>) parameter.
	 */
	public MimeMultiPart(final HttpServletRequest request, final HttpServletResponse response) {
		this.request = request;
		this.response = response;
		String contentTypeAndAttr = request.getContentType();
		this.characterEncoding = request.getCharacterEncoding();
		if (contentTypeAndAttr != null) {
			HeaderValue headerValue = new HeaderValue(contentTypeAndAttr);
			this.contentType = headerValue.getValue();
			this.boundary = headerValue.getAttribute("boundary"); //$NON-NLS-1$
		}
	}

	/**
	 * Gets the content type (String) value.
	 * @return the content type
	 */
	public String getContentType() {
		return this.contentType;
	}

	/**
	 * Gets the next part (MimePart) value.
	 * @return	The next part (<code>MimePart</code>) value.
	 * @throws IOException IOException.
	 */
	public MimePart getNextPart() throws IOException {
		if (this.partInputStream == null) {
			this.partInputStream = new MimeMultiPart.PartInputStream();
		}
		if (!this.partInputStream.isLastPart()) {
			this.partInputStream = new MimeMultiPart.PartInputStream();
			return new MimePart(this.partInputStream);
		}
		return null;
	}

	/**
	 * Gets the request (HttpServletRequest) value.
	 * @return the request
	 */
	public HttpServletRequest getRequest() {
		return this.request;
	}

	/**
	 * Gets the response (HttpServletResponse) value.
	 * @return the response
	 */
	public HttpServletResponse getResponse() {
		return this.response;
	}

	/**
	 * @author IBM
	 * @version 1.2.0
	 */
	public class PartInputStream extends FilterInputStream {
		/**
		 * Define the count (int) field.
		 */
		private int count;

		/**
		 * Define the pos (int) field.
		 */
		private int pos;

		/**
		 * Define the buf (byte) field.
		 */
		private byte buf[] = new byte[4096];

		/**
		 * Define the line (String) field.
		 */
		private String line;

		/**
		 * Define the at end (boolean) field.
		 */
		private boolean atEnd;

		/**
		 * Define the last part (boolean) field.
		 */
		private boolean lastPart;

		/**
		 * Constructs an instance of this class.
		 * @throws IOException IOException.
		 */
		public PartInputStream() throws IOException {
			super(MimeMultiPart.this.getRequest().getInputStream());
		}

		/**
		 * Available and return the int result.
		 * @return	Results of the available (<code>int</code>) value.
		 * @throws IOException IOException.
		 */
		public int available() throws IOException {
			if (this.buf == null) {
				throw new IOException(DeviceTestManagerServlet.DefaultResourceBundle.getString(Integer.toString(DeviceTestManagerServlet.STREAM_CLOSED)));
			}
			return this.atEnd ? 0 : this.count - this.pos + super.available();
		}

		/**
		 * Close.
		 * @throws IOException IOException.
		 */
		public void close() throws IOException {
			this.buf = null;
		}

		/**
		 * Fill line.
		 * @throws UnsupportedEncodingException Unsupported Encoding Exception.
		 * @see #readLine()
		 */
		private void fillLine() throws UnsupportedEncodingException {
			if (getCharacterEncoding() == null) {
				this.line = new String(this.buf, this.pos, this.count);
			} else {
				this.line = new String(this.buf, this.pos, this.count, getCharacterEncoding());
			}
			this.line = this.line.trim();
		}

		/**
		 * Fillbuf and return the int result.
		 * @return	Results of the fillbuf (<code>int</code>) value.
		 * @throws IOException IOException.
		 */
		private int fillbuf() throws IOException {
			if (this.atEnd) {
				return -1;
			}
			int result = MimeMultiPart.this.getRequest().getInputStream().readLine(this.buf, 0, this.buf.length);
			this.pos = 0;
			if (result > 0) {
				this.count = result;
				fillLine();
				if (this.line.startsWith("--" + MimeMultiPart.this.boundary)) { //$NON-NLS-1$
					result = -1;
					this.count = 0;
					this.atEnd = true;
					this.lastPart = this.line.endsWith(MimeMultiPart.this.boundary + "--"); //$NON-NLS-1$
				}
			} else {
				this.count = 0;
			}
			return result;
		}

		/**
		 * Gets the character encoding (String) value.
		 * @return	The character encoding (<code>String</code>) value.
		 */
		public String getCharacterEncoding() {
			return MimeMultiPart.this.characterEncoding;
		}

		/**
		 * Gets the last part (boolean) value.
		 * @return the lastPart
		 * @throws IOException IOException.
		 */
		public boolean isLastPart() throws IOException {
			while (!this.atEnd)
				fillbuf();
			return this.lastPart;
		}

		/**
		 * Read and return the int result.
		 * @return	Results of the read (<code>int</code>) value.
		 * @throws IOException IOException.
		 * @see #read(byte[],int,int)
		 */
		public int read() throws IOException {
			if (this.buf != null) {
				if (this.pos >= this.count && fillbuf() == -1)
					return -1;
				if (this.count - this.pos > 0)
					return this.buf[this.pos++] & 0xFF;
				return -1;
			}
			throw new IOException(DeviceTestManagerServlet.DefaultResourceBundle.getString(Integer.toString(DeviceTestManagerServlet.STREAM_CLOSED)));
		}

		/**
		 * Read with the specified buffer, offset and length parameters and return the int result.
		 * @param buffer	The buffer (<code>byte[]</code>) parameter.
		 * @param offset	The offset (<code>int</code>) parameter.
		 * @param length	The length (<code>int</code>) parameter.
		 * @return	Results of the read (<code>int</code>) value.
		 * @throws IOException IOException.
		 * @see #read()
		 */
		public int read(final byte[] buffer, final int offset, final int length) throws IOException {
			if (this.buf == null) {
				throw new IOException(DeviceTestManagerServlet.DefaultResourceBundle.getString(Integer.toString(DeviceTestManagerServlet.STREAM_CLOSED)));
			}
			if (0 <= offset && offset <= buffer.length && 0 <= length && length <= buffer.length - offset) {
				if (length == 0)
					return 0;
				int required;
				int i = offset;
				if (this.pos < this.count) {
					int copylength = this.count - this.pos >= length ? length : this.count - this.pos;
					System.arraycopy(this.buf, this.pos, buffer, i, copylength);
					this.pos += copylength;
					if (copylength == length || this.in.available() == 0)
						return copylength;
					i += copylength;
					required = length - copylength;
				} else
					required = length;
				while (true) {
					int read;
					if (fillbuf() == -1)
						return required == length ? -1 : length - required;
					read = this.count - this.pos >= required ? required : this.count - this.pos;
					System.arraycopy(this.buf, this.pos, buffer, i, read);
					this.pos += read;
					required -= read;
					if (required == 0)
						return length;
					if (this.in.available() == 0)
						return length - required;
					i += read;
				}
			}
			throw new ArrayIndexOutOfBoundsException();
		}

		/**
		 * Read line and return the String result.
		 * @return	Results of the read line (<code>String</code>) value.
		 * @throws IOException IOException.
		 */
		protected String readLine() throws IOException {
			if (this.buf == null) {
				throw new IOException(DeviceTestManagerServlet.DefaultResourceBundle.getString(Integer.toString(DeviceTestManagerServlet.STREAM_CLOSED)));
			}
			if (this.pos >= this.count && fillbuf() == -1)
				return null;
			this.pos = this.count;
			return this.line;
		}
	}
}
