package org.eclipse.soda.dk.testmanager.servlet;

/*************************************************************************
 * Copyright (c) 2006, 2009 IBM.                                         *
 * All rights reserved. This program and the accompanying materials      *
 * are made available under the terms of the Eclipse Public License v1.0 *
 * which accompanies this distribution, and is available at              *
 * http://www.eclipse.org/legal/epl-v10.html                             *
 *                                                                       *
 * Contributors:                                                         *
 *     IBM - initial API and implementation                              *
 ************************************************************************/
import java.util.ArrayList;
import java.util.List;
import org.eclipse.soda.dk.testagent.service.TestSynchronizationService;
import org.eclipse.soda.dk.testmanager.service.DistributedTestSessionService;

/**
 * @author IBM
 * @version 1.2.0
 * @since 1.0
 */
public class TestControl implements TestSynchronizationService {
	/**
	 * Define the test session (TestSessionService) field.
	 */
	private DistributedTestSessionService distributedTestSession;

	/**
	 * Define the requests (List) field.
	 */
	private List requests;

	/**
	 * Define the next request (int) field.
	 */
	private int nextRequest;

	/**
	 * Define the confirmation lock (Object) field.
	 */
	private Object confirmationLock;

	/**
	 * Define the confirmation value (boolean) field.
	 */
	private boolean confirmationValue;

	/**
	 * Define the finished (boolean) field.
	 */
	private boolean finished;

	/**
	 * @author IBM
	 * @version 1.2.0
	 */
	public static class Request {
		/**
		 * Define the prompt (String) field.
		 */
		public String prompt;

		/**
		 * Define the test name (String) field.
		 */
		public String testName;
	}

	/**
	 * Constructs an instance of this class.
	 */
	public TestControl() {
		this.requests = new ArrayList();
		this.confirmationLock = new Object();
	}

	/**
	 * Confirm request with the specified confirmation value parameter.
	 * @param confirmationValue	The confirmation value (<code>boolean</code>) parameter.
	 */
	public void confirmRequest(final boolean confirmationValue) {
		synchronized (this.confirmationLock) {
			this.confirmationValue = confirmationValue;
			this.confirmationLock.notify();
		}
	}

	/**
	 * Finished.
	 */
	public void finished() {
		if (!this.finished) {
			this.finished = true;
			this.distributedTestSession.interrupt();
			synchronized (this.requests) {
				this.requests.notify();
			}
			synchronized (this.confirmationLock) {
				this.confirmationLock.notify();
			}
		}
	}

	/**
	 * Request confirmation with the specified prompt and test name parameters and return the boolean result.
	 * @param prompt	The prompt (<code>String</code>) parameter.
	 * @param testName	The test name (<code>String</code>) parameter.
	 * @return	Results of the request confirmation (<code>boolean</code>) value.
	 * @throws InterruptedException Interrupted Exception.
	 */
	public boolean requestConfirmation(final String prompt, final String testName) throws InterruptedException {
		TestControl.Request request = new TestControl.Request();
		request.prompt = prompt;
		request.testName = testName;
		synchronized (this.confirmationLock) {
			synchronized (this.requests) {
				this.requests.add(request);
				this.requests.notify();
			}
			this.confirmationLock.wait();
		}
		return this.confirmationValue;
	}

	/**
	 * Sets the test session manager value.
	 * @param distributedTestSession the testSession to set
	 */
	public void setTestSessionManager(final DistributedTestSessionService distributedTestSession) {
		this.distributedTestSession = distributedTestSession;
	}

	/**
	 * Wait for next event and return the Request result.
	 * @return	Results of the wait for next event (<code>Request</code>) value.
	 * @throws InterruptedException Interrupted Exception.
	 */
	public TestControl.Request waitForNextEvent() throws InterruptedException {
		synchronized (this.requests) {
			while (this.nextRequest == this.requests.size() && !this.finished) {
				this.requests.wait();
			}
			if (this.nextRequest < this.requests.size()) {
				return (TestControl.Request) this.requests.get(this.nextRequest++);
			}
		}
		return null;
	}
}
