package org.eclipse.soda.dk.testmanager.servlet;

/*************************************************************************
 * Copyright (c) 2006, 2009 IBM.                                         *
 * All rights reserved. This program and the accompanying materials      *
 * are made available under the terms of the Eclipse Public License v1.0 *
 * which accompanies this distribution, and is available at              *
 * http://www.eclipse.org/legal/epl-v10.html                             *
 *                                                                       *
 * Contributors:                                                         *
 *     IBM - initial API and implementation                              *
 ************************************************************************/
import java.util.ArrayList;
import java.util.Date;
import java.util.List;
import org.eclipse.soda.dk.nls.Nls;
import org.eclipse.soda.dk.testcontroller.service.TestRunListenerService;
import org.eclipse.soda.dk.testmanager.service.ScriptRunListenerService;
import org.eclipse.soda.dk.testmanager.service.TestScriptService;
import org.eclipse.soda.dk.testmanager.service.TestStatisticsService;

/**
 * @author IBM
 * @version 1.2.0
 * @since 1.0
 */
public class TestReport implements ScriptRunListenerService, TestRunListenerService {
	/**
	 * Define the events (List) field.
	 */
	private List events;

	/**
	 * Define the event index (int) field.
	 */
	private int eventIndex;

	/**
	 * Define the finished (boolean) field.
	 */
	private boolean finished;

	/**
	 * @author IBM
	 * @version 1.2.0
	 */
	public static class Event {
		/**
		 * Define the timestamp (Date) field.
		 */
		public Date timestamp;

		/**
		 * Define the error (boolean) field.
		 */
		public boolean error;

		/**
		 * Define the message (String) field.
		 */
		public String message;

		/**
		 * Define the detail (String) field.
		 */
		public String detail;
	}

	/**
	 * Constructs an instance of this class.
	 */
	public TestReport() {
		this.events = new ArrayList();
	}

	/**
	 * Append attribute with the specified detail, name and value parameters.
	 * @param detail	The detail (<code>StringBuffer</code>) parameter.
	 * @param name	The name (<code>String</code>) parameter.
	 * @param value	The value (<code>Object</code>) parameter.
	 */
	private static void appendAttribute(final StringBuffer detail, final String name, final Object value) {
		if (value != null) {
			detail.append(name);
			detail.append(": "); //$NON-NLS-1$
			detail.append(value);
			detail.append('\n');
		}
	}

	/**
	 * Add error event with the specified message and trace parameters.
	 * @param message	The message (<code>String</code>) parameter.
	 * @param trace	The trace (<code>String</code>) parameter.
	 */
	private void addErrorEvent(final String message, final String trace) {
		addEvent(true, message, trace);
	}

	/**
	 * Add event with the specified report line parameter.
	 * @param reportLine	The report line (<code>String</code>) parameter.
	 */
	private void addEvent(final String reportLine) {
		addEvent(false, reportLine, null);
	}

	/**
	 * Add event with the specified error, message and detail parameters.
	 * @param error	The error (<code>boolean</code>) parameter.
	 * @param message	The message (<code>String</code>) parameter.
	 * @param detail	The detail (<code>String</code>) parameter.
	 */
	private void addEvent(final boolean error, final String message, final String detail) {
		TestReport.Event event = new TestReport.Event();
		event.timestamp = new Date();
		event.error = error;
		event.message = message;
		event.detail = detail;
		synchronized (this.events) {
			this.events.add(event);
			this.events.notify();
		}
	}

	/**
	 * Script action ended with the specified message parameter.
	 * @param actionId The action id (<code>String</code>) parameter.
	 * @param message	The message (<code>String</code>) parameter.
	 */
	public void scriptActionEnded(final String actionId, final String message) {
		Object[] parms = { actionId, message };
		addEvent(Nls.format(DeviceTestManagerServlet.DefaultResourceBundle.getString(Integer.toString(DeviceTestManagerServlet.ENDING_SCRIPT_ACTION)), parms));
	}

	/**
	 * Script action error with the specified message and detail parameters.
	 * @param actionId The action id (<code>String</code>) parameter.
	 * @param message	The message (<code>String</code>) parameter.
	 * @param detail	The detail (<code>String</code>) parameter.
	 */
	public void scriptActionError(final String actionId, final String message, final String detail) {
		Object[] parms = { actionId, message };
		addErrorEvent(Nls.format(DeviceTestManagerServlet.DefaultResourceBundle.getString(Integer.toString(DeviceTestManagerServlet.ERROR_EXECUTING_ACTION)), parms), detail);
	}

	/**
	 * Script action info with the specified message parameter.
	 * @param actionId The action id (<code>String</code>) parameter.
	 * @param message	The message (<code>String</code>) parameter.
	 */
	public void scriptActionInfo(final String actionId, final String message) {
		addEvent(message);
	}

	/**
	 * Script action started with the specified message parameter.
	 * @param parentActionId
	 *		The parent action ID (<code>String</code>) parameter.
	 * @param actionId The action id (<code>String</code>) parameter.
	 * @param message	The message (<code>String</code>) parameter.
	 */
	public void scriptActionStarted(final String parentActionId, final String actionId, final String message) {
		Object[] parms = { actionId, message };
		addEvent(Nls.format(DeviceTestManagerServlet.DefaultResourceBundle.getString(Integer.toString(DeviceTestManagerServlet.STARTING_SCRIPT_ACTION)), parms));
	}

	/**
	 * Script run ended with the specified test statistics parameter.
	 * @param testStatistics	The test statistics (<code>Map</code>) parameter.
	 */
	public void scriptRunEnded(final TestStatisticsService testStatistics) {
		this.finished = true;
		boolean error = false;
		StringBuffer trace = new StringBuffer();
		int scriptErrorCount = testStatistics.getCount(TestStatisticsService.STATS_SCRIPT_ERROR_COUNT);
		if (scriptErrorCount > 0) {
			error = true;
		}
		int configActionCount = testStatistics.getCount(TestStatisticsService.STATS_CONFIGURATION_COUNT);
		if (configActionCount > 0) {
			int configActionErrorCount = testStatistics.getCount(TestStatisticsService.STATS_CONFIGURATION_ERROR_COUNT);
			if (configActionErrorCount > 0) {
				error = true;
			}
			Object[] parms = { new Integer(configActionCount), new Integer(configActionErrorCount) };
			trace.append(Nls.format(DeviceTestManagerServlet.DefaultResourceBundle.getString(Integer.toString(DeviceTestManagerServlet.CONFIG_CHANGES)), parms));
		}
		int testActionCount = testStatistics.getCount(TestStatisticsService.STATS_TEST_COUNT);
		if (testActionCount > 0) {
			int testActionErrorCount = testStatistics.getCount(TestStatisticsService.STATS_TEST_ERROR_COUNT);
			if (testActionErrorCount > 0) {
				error = true;
			}
			Object[] parms = { new Integer(testActionCount), new Integer(testActionErrorCount) };
			trace.append(Nls.format(DeviceTestManagerServlet.DefaultResourceBundle.getString(Integer.toString(DeviceTestManagerServlet.TESTS)), parms));
		}
		int testCount = testStatistics.getCount(TestStatisticsService.STATS_TEST_CASE_COUNT);
		if (testCount > 0) {
			int testErrorCount = testStatistics.getCount(TestStatisticsService.STATS_TEST_CASE_ERROR_COUNT);
			int testFailedCount = testStatistics.getCount(TestStatisticsService.STATS_TEST_CASE_FAILED_COUNT);
			if (testErrorCount > 0 || testFailedCount > 0) {
				error = true;
			}
			Object[] parms = { new Integer(testCount), new Integer(testErrorCount), new Integer(testFailedCount) };
			trace.append(Nls.format(DeviceTestManagerServlet.DefaultResourceBundle.getString(Integer.toString(DeviceTestManagerServlet.TEST_CASES)), parms));
		}
		addEvent(error, DeviceTestManagerServlet.DefaultResourceBundle.getString(Integer.toString(DeviceTestManagerServlet.TEST_COMPLETED)), trace.toString());
	}

	/**
	 * Script run error with the specified message and trace parameters.
	 * @param message	The message (<code>String</code>) parameter.
	 * @param trace	The trace (<code>String</code>) parameter.
	 */
	public void scriptRunError(final String message, final String trace) {
		Object[] parms = { message };
		addErrorEvent(Nls.format(DeviceTestManagerServlet.DefaultResourceBundle.getString(Integer.toString(DeviceTestManagerServlet.ERROR_RUNNING_SCRIPT)), parms), trace);
	}

	/**
	 * Script run started with the specified script parameter.
	 * @param script	The script (<code>TestScriptService</code>) parameter.
	 */
	public void scriptRunStarted(final TestScriptService script) {
		Object title = script.getAttributes().get(TestScriptService.ATTR_TITLE);
		Object version = script.getAttributes().get(TestScriptService.ATTR_VERSION);
		Object vendor = script.getAttributes().get(TestScriptService.ATTR_VENDOR);
		Object description = script.getAttributes().get(TestScriptService.ATTR_DESCRIPTION);
		Object url = script.getAttributes().get(TestScriptService.ATTR_URL);
		if (title == null)
			title = DeviceTestManagerServlet.DefaultResourceBundle.getString(Integer.toString(DeviceTestManagerServlet.UNTITLED));
		StringBuffer detail = new StringBuffer();
		appendAttribute(detail, TestScriptService.ATTR_VERSION, version);
		appendAttribute(detail, TestScriptService.ATTR_VENDOR, vendor);
		appendAttribute(detail, TestScriptService.ATTR_DESCRIPTION, description);
		appendAttribute(detail, TestScriptService.ATTR_URL, url);
		Object[] parms = { title };
		addEvent(false, Nls.format(DeviceTestManagerServlet.DefaultResourceBundle.getString(Integer.toString(DeviceTestManagerServlet.RUNNING_TEST_SCRIPT)), parms), detail.toString());
	}

	/**
	 * Test ended with the specified run id and test name parameters.
	 * @param runId	The run id (<code>String</code>) parameter.
	 * @param testName	The test name (<code>String</code>) parameter.
	 */
	public synchronized void testEnded(final String runId, final String testName) {
		/* do nothing */
	}

	/**
	 * Test error with the specified run id, test name and trace parameters.
	 * @param runId	The run id (<code>String</code>) parameter.
	 * @param testName	The test name (<code>String</code>) parameter.
	 * @param trace	The trace (<code>String</code>) parameter.
	 */
	public synchronized void testError(final String runId, final String testName, final String trace) {
		Object[] parms = { testName, runId };
		addErrorEvent(Nls.format(DeviceTestManagerServlet.DefaultResourceBundle.getString(Integer.toString(DeviceTestManagerServlet.ERROR_IN_TEST)), parms), trace);
	}

	/**
	 * Test failed with the specified run id, test name and trace parameters.
	 * @param runId	The run id (<code>String</code>) parameter.
	 * @param testName	The test name (<code>String</code>) parameter.
	 * @param trace	The trace (<code>String</code>) parameter.
	 */
	public synchronized void testFailed(final String runId, final String testName, final String trace) {
		Object[] parms = { testName, runId };
		addErrorEvent(Nls.format(DeviceTestManagerServlet.DefaultResourceBundle.getString(Integer.toString(DeviceTestManagerServlet.TEST_FAILED)), parms), trace);
	}

	/**
	 * Test run ended with the specified run id parameter.
	 * @param runId	The run id (<code>String</code>) parameter.
	 */
	public synchronized void testRunEnded(final String runId) {
		/* do nothing */
	}

	/**
	 * Test run error with the specified run id, message and detail parameters.
	 * @param runId	The run id (<code>String</code>) parameter.
	 * @param message	The message (<code>String</code>) parameter.
	 * @param detail	The detail (<code>String</code>) parameter.
	 */
	public synchronized void testRunError(final String runId, final String message, final String detail) {
		Object[] parms = { message, runId };
		addErrorEvent(Nls.format(DeviceTestManagerServlet.DefaultResourceBundle.getString(Integer.toString(DeviceTestManagerServlet.ERROR_IN_TEST_RUN)), parms), detail);
	}

	/**
	 * Test run started with the specified run id parameter.
	 * @param actionId The action id (<code>String</code>) parameter.
	 * @param runId	The run id (<code>String</code>) parameter.
	 */
	public void testRunStarted(final String actionId, final String runId) {
		/* do nothing */
	}

	/**
	 * Test started with the specified run id and test name parameters.
	 * @param runId	The run id (<code>String</code>) parameter.
	 * @param testName	The test name (<code>String</code>) parameter.
	 */
	public synchronized void testStarted(final String runId, final String testName) {
		Object[] parms = { testName, runId };
		addEvent(Nls.format(DeviceTestManagerServlet.DefaultResourceBundle.getString(Integer.toString(DeviceTestManagerServlet.RUNNING)), parms));
	}

	/**
	 * Wait for next event and return the Event result.
	 * @return	Results of the wait for next event (<code>Event</code>) value.
	 * @throws InterruptedException Interrupted Exception.
	 */
	public TestReport.Event waitForNextEvent() throws InterruptedException {
		synchronized (this.events) {
			while (this.eventIndex == this.events.size() && !this.finished) {
				this.events.wait();
			}
			if (this.eventIndex < this.events.size()) {
				return (TestReport.Event) this.events.get(this.eventIndex++);
			}
		}
		return null;
	}
}
